@@ -98,8 +98,14 @@
 /* define the machine (cpu) type include file here */
 #define MACHINE_INCLUDE_FILE <net-snmp/machine/generic.h>
 
-/* SNMPLIBDIR contains important files */
+/* define the UDP buffer defaults undefined means use the OS buffers
+ * by default */
+#undef DEFAULT_SERVER_SEND_BUF
+#undef DEFAULT_SERVER_RECV_BUF
+#undef DEFAULT_CLIENT_SEND_BUF
+#undef DEFAULT_CLIENT_RECV_BUF
 
+/* SNMPLIBDIR contains important files */
 #undef SNMPLIBPATH
 #undef SNMPSHAREPATH
 #undef SNMPCONFPATH
@@ -297,9 +303,6 @@
 /* des_ks_struct.weak_key */
 #undef STRUCT_DES_KS_STRUCT_HAS_WEAK_KEY
 
-/* mbstat.m_clusters */
-#undef STRUCT_MBSTAT_HAS_M_CLUSTERS
-
 /* ifnet needs to have _KERNEL defined */
 #undef IFNET_NEEDS_KERNEL
 
@@ -515,6 +518,9 @@
 /* got in_addr_t? */
 #undef HAVE_IN_ADDR_T
 
+/* got ssize_t? */
+#undef HAVE_SSIZE_T
+
 #ifndef HAVE_STRCHR
 #ifdef HAVE_INDEX
 # define strchr index
@@ -524,15 +530,10 @@
 
 #ifndef HAVE_INDEX
 #ifdef HAVE_STRCHR
-#ifdef mingw32
-# define index(a,b) strchr(a,b)
-# define rindex(a,b) strrchr(a,b)
-#else
 # define index strchr
 # define rindex strrchr
 #endif
 #endif
-#endif
 
 #ifndef HAVE_MEMCPY
 #ifdef HAVE_BCOPY
@@ -645,20 +646,22 @@
 /* define this if we're using the new MIT crypto API */
 #undef MIT_NEW_CRYPTO
 
-/* define if you want to build with reentrant/threaded code (incomplete) */
+/* define if you want to build with reentrant/threaded code (incomplete)*/
 #undef NS_REENTRANT
 
+/* define if you want to build MFD module rewrites*/
+#undef NETSNMP_ENABLE_MFD_REWRITES
+
 /* on aix, if you have perfstat */
 #undef HAVE_PERFSTAT
 
-/* define if you have ssize_t */
-#undef HAVE_SSIZE_T
-
 /* Not-to-be-compiled macros for use by configure only */
 #define config_require(x)
+#define config_exclude(x)
 #define config_arch_require(x,y)
 #define config_parse_dot_conf(w,x,y,z)
 #define config_add_mib(x)
+#define config_belongs_in(x)
   
 #if defined (WIN32) || defined (mingw32) || defined (cygwin)
 #define ENV_SEPARATOR ";"
@@ -739,9 +742,6 @@
 /* define if agentx transport is to use domain sockets only */
 #undef AGENTX_DOM_SOCK_ONLY
 
-/* define if you do not want snmptrapd to register as an AgentX subagent */
-#undef SNMPTRAPD_DISABLE_AGENTX
-
 #undef HEIMDAL
 
 #endif /* NET_SNMP_CONFIG_H */
