@@ -20,6 +20,10 @@
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 
 #include <net-snmp/agent/bulk_to_next.h>
+
+
+static netsnmp_mib_handler *_clone_handler(netsnmp_mib_handler *it);
+
 /***********************************************************************/
 /*
  * New Handler based API 
@@ -103,8 +107,12 @@ netsnmp_create_handler(const char *name,
 {
     netsnmp_mib_handler *ret = SNMP_MALLOC_TYPEDEF(netsnmp_mib_handler);
     if (ret) {
-        ret->handler_name = strdup(name);
         ret->access_method = handler_access_method;
+        if (NULL != name) {
+            ret->handler_name = strdup(name);
+            if (NULL == ret->handler_name)
+                SNMP_FREE(ret);
+        }
     }
     return ret;
 }
@@ -113,7 +121,8 @@ netsnmp_create_handler(const char *name,
  *  access_method function, a registration location oid and the modes
  *  the handler supports. If modes == 0, then modes will automatically
  *  be set to the default value of only HANDLER_CAN_DEFAULT, which is
- *  by default read-only GET and GETNEXT requests.
+ *  by default read-only GET and GETNEXT requests. A hander which supports
+ *  sets but not row creation should set us a mode of HANDLER_CAN_SET_ONLY.
  *  @note This ends up calling netsnmp_create_handler(name, handler_access_method)
  *  @param name is the handler name and is copied then assigned to
  *              netsnmp_handler_registration->handlerName.
@@ -151,10 +160,10 @@ netsnmp_create_handler(const char *name,
  *  @see netsnmp_register_handler()
  */
 netsnmp_handler_registration *
-netsnmp_create_handler_registration(const char *name,
-                                    Netsnmp_Node_Handler *
-                                    handler_access_method, oid * reg_oid,
-                                    size_t reg_oid_len, int modes)
+netsnmp_handler_registration_create(const char *name,
+                                    netsnmp_mib_handler *handler,
+                                    oid * reg_oid, size_t reg_oid_len,
+                                    int modes)
 {
     netsnmp_handler_registration *the_reg;
     the_reg = SNMP_MALLOC_TYPEDEF(netsnmp_handler_registration);
@@ -166,7 +175,7 @@ netsnmp_create_handler_registration(const char *name,
     else
         the_reg->modes = HANDLER_CAN_DEFAULT;
 
-    the_reg->handler = netsnmp_create_handler(name, handler_access_method);
+    the_reg->handler = handler;
     the_reg->priority = DEFAULT_MIB_PRIORITY;
     if (name)
         the_reg->handlerName = strdup(name);
@@ -176,6 +185,18 @@ netsnmp_create_handler_registration(const char *name,
     return the_reg;
 }
 
+netsnmp_handler_registration *
+netsnmp_create_handler_registration(const char *name,
+                                    Netsnmp_Node_Handler *
+                                    handler_access_method, oid * reg_oid,
+                                    size_t reg_oid_len, int modes)
+{
+    return
+        netsnmp_handler_registration_create(name,
+                                            netsnmp_create_handler(name, handler_access_method),
+                                            reg_oid, reg_oid_len, modes);
+}
+
 /** register a handler, as defined by the netsnmp_handler_registration pointer. */
 int
 netsnmp_register_handler(netsnmp_handler_registration *reginfo)
@@ -183,6 +204,7 @@ netsnmp_register_handler(netsnmp_handler_registration *reginfo)
     netsnmp_mib_handler *handler;
     if (reginfo == NULL) {
         snmp_log(LOG_ERR, "netsnmp_register_handler() called illegally\n");
+        netsnmp_assert(reginfo != NULL);
         return SNMP_ERR_GENERR;
     }
 
@@ -211,6 +233,8 @@ netsnmp_register_handler(netsnmp_handler_registration *reginfo)
      */
     if (0 == reginfo->modes) {
         reginfo->modes = HANDLER_CAN_DEFAULT;
+        snmp_log(LOG_WARNING, "no registration modes specified for %s. "
+                 "Defaulting to %p\n", reginfo->handlerName, reginfo->modes);
     }
 
     /*
@@ -248,6 +272,7 @@ netsnmp_register_handler_nocallback(netsnmp_handler_registration *reginfo)
     netsnmp_mib_handler *handler;
     if (reginfo == NULL) {
         snmp_log(LOG_ERR, "netsnmp_register_handler_nocallback() called illegally\n");
+        netsnmp_assert(reginfo != NULL);
         return SNMP_ERR_GENERR;
     }
     DEBUGIF("handler::register") {
@@ -298,18 +323,25 @@ netsnmp_inject_handler_before(netsnmp_handler_registration *reginfo,
                               netsnmp_mib_handler *handler,
                               const char *before_what)
 {
-    netsnmp_mib_handler *nexth, *prevh = NULL;
     if (handler == NULL || reginfo == NULL) {
         snmp_log(LOG_ERR, "netsnmp_inject_handler() called illegally\n");
+        netsnmp_assert(reginfo != NULL);
+        netsnmp_assert(handler != NULL);
         return SNMP_ERR_GENERR;
     }
     if (reginfo->handler == NULL) {
-        snmp_log(LOG_ERR, "no handler specified.");
-        return SNMP_ERR_GENERR;
+        DEBUGMSGTL(("handler:inject", "injecting %s\n", handler->handler_name));
+    }
+    else {
+        DEBUGMSGTL(("handler:inject", "injecting %s before %s\n",
+                    handler->handler_name, reginfo->handler->handler_name));
     }
-    DEBUGMSGTL(("handler:inject", "injecting %s before %s\n",
-                handler->handler_name, reginfo->handler->handler_name));
     if (before_what) {
+        netsnmp_mib_handler *nexth, *prevh = NULL;
+        if (reginfo->handler == NULL) {
+            snmp_log(LOG_ERR, "no handler to inject before\n");
+            return SNMP_ERR_GENERR;
+        }
         for(nexth = reginfo->handler; nexth;
             prevh = nexth, nexth = nexth->next) {
             if (strcmp(nexth->handler_name, before_what) == 0)
@@ -358,9 +390,14 @@ netsnmp_call_handler(netsnmp_mib_handler *next_handler,
     if (next_handler == NULL || reginfo == NULL || reqinfo == NULL ||
         requests == NULL) {
         snmp_log(LOG_ERR, "netsnmp_call_handler() called illegally\n");
+        netsnmp_assert(next_handler != NULL);
+        netsnmp_assert(reqinfo != NULL);
+        netsnmp_assert(reginfo != NULL);
+        netsnmp_assert(requests != NULL);
         return SNMP_ERR_GENERR;
     }
 
+    do {
     nh = next_handler->access_method;
     if (!nh) {
         if (next_handler->next) {
@@ -388,6 +425,21 @@ netsnmp_call_handler(netsnmp_mib_handler *next_handler,
     DEBUGMSGTL(("handler:returned", "handler %s returned %d\n",
                 next_handler->handler_name, ret));
 
+    if (! (next_handler->flags & MIB_HANDLER_AUTO_NEXT))
+        break;
+
+    /*
+     * did handler signal that it didn't want auto next this time around?
+     */
+    if(next_handler->flags & MIB_HANDLER_AUTO_NEXT_OVERRIDE_ONCE) {
+        next_handler->flags &= ~MIB_HANDLER_AUTO_NEXT_OVERRIDE_ONCE;
+        break;
+    }
+
+    next_handler = next_handler->next;
+
+    } while(next_handler);
+
     return ret;
 }
 
@@ -404,6 +456,9 @@ netsnmp_call_handlers(netsnmp_handler_registration *reginfo,
 
     if (reginfo == NULL || reqinfo == NULL || requests == NULL) {
         snmp_log(LOG_ERR, "netsnmp_call_handlers() called illegally\n");
+        netsnmp_assert(reqinfo != NULL);
+        netsnmp_assert(reginfo != NULL);
+        netsnmp_assert(requests != NULL);
         return SNMP_ERR_GENERR;
     }
 
@@ -463,6 +518,10 @@ netsnmp_call_next_handler(netsnmp_mib_handler *current,
     if (current == NULL || reginfo == NULL || reqinfo == NULL ||
         requests == NULL) {
         snmp_log(LOG_ERR, "netsnmp_call_next_handler() called illegally\n");
+        netsnmp_assert(current != NULL);
+        netsnmp_assert(reginfo != NULL);
+        netsnmp_assert(reqinfo != NULL);
+        netsnmp_assert(requests != NULL);
         return SNMP_ERR_GENERR;
     }
 
@@ -482,6 +541,7 @@ netsnmp_call_next_handler_one_request(netsnmp_mib_handler *current,
     
     if (!requests) {
         snmp_log(LOG_ERR, "netsnmp_call_next_handler_ONE_REQUEST() called illegally\n");
+        netsnmp_assert(requests != NULL);
         return SNMP_ERR_GENERR;
     }
 
@@ -498,6 +558,8 @@ netsnmp_handler_free(netsnmp_mib_handler *handler)
 {
     if (handler != NULL) {
         if (handler->next != NULL) {
+            /** make sure we aren't pointing to ourselves.  */
+            netsnmp_assert(handler != handler->next); /* bugs caught: 1 */
             netsnmp_handler_free(handler->next);
             handler->next = NULL;
         }
@@ -510,7 +572,9 @@ netsnmp_handler_free(netsnmp_mib_handler *handler)
     }
 }
 
-/** dulpicates a handler */
+/** dulpicates a handler and all subsequent handlers
+ * see also _clone_handler
+ */
 netsnmp_mib_handler *
 netsnmp_handler_dup(netsnmp_mib_handler *handler)
 {
@@ -520,27 +584,15 @@ netsnmp_handler_dup(netsnmp_mib_handler *handler)
         return NULL;
     }
 
-    h = (netsnmp_mib_handler *) calloc(1, sizeof(netsnmp_mib_handler));
+    h = _clone_handler(handler);
 
     if (h != NULL) {
         h->myvoid = handler->myvoid;
-        h->access_method = handler->access_method;
-
-        if (handler->handler_name != NULL) {
-            h->handler_name = strdup(handler->handler_name);
-            if (h->handler_name == NULL) {
-                SNMP_FREE(h);
-                return NULL;
-            }
-        }
 
         if (handler->next != NULL) {
             h->next = netsnmp_handler_dup(handler->next);
             if (h->next == NULL) {
-                if (h->handler_name) {
-                    SNMP_FREE(h->handler_name);
-                }
-                SNMP_FREE(h);
+                netsnmp_handler_free(h);
                 return NULL;
             }
             h->next->prev = h;
@@ -711,6 +763,25 @@ netsnmp_request_add_list_data(netsnmp_request_info *request,
     }
 }
 
+/** remove data from a request
+ *
+ * @param requset the netsnmp request info structure
+ *
+ * @param name this is the name of the previously added data
+ *
+ * @return 0 on successful find-and-delete, 1 otherwise.
+ *
+ */
+NETSNMP_INLINE int
+netsnmp_request_remove_list_data(netsnmp_request_info *request,
+                                 const char *name)
+{
+    if ((NULL == request) || (NULL ==request->parent_data))
+        return 1;
+
+    return netsnmp_remove_list_node(&request->parent_data, name);
+}
+
 /** extract data from a request that was added previously by a parent module
  *
  * @param request the netsnmp request info function
@@ -777,12 +848,22 @@ netsnmp_find_handler_data_by_name(netsnmp_handler_registration *reginfo,
     return NULL;
 }
 
-/** clones a mib handler (it's name and access methods onlys; not myvoid)
+/** clones a mib handler (name, flags and access methods only; not myvoid)
+ * see also netsnmp_handler_dup
  */
-netsnmp_mib_handler *
-clone_handler(netsnmp_mib_handler *it)
+static netsnmp_mib_handler *
+_clone_handler(netsnmp_mib_handler *it)
 {
-    return netsnmp_create_handler(it->handler_name, it->access_method);
+    netsnmp_mib_handler *dup;
+
+    if(NULL == it)
+        return NULL;
+
+    dup = netsnmp_create_handler(it->handler_name, it->access_method);
+    if(NULL != dup)
+        dup->flags = it->flags;
+
+    return dup;
 }
 
 static netsnmp_data_list *handler_reg = NULL;
@@ -834,14 +915,14 @@ netsnmp_inject_handler_into_subtree(netsnmp_subtree *tp, const char *name,
         if (strcmp(tptr->label_a, name) == 0) {
             DEBUGMSGTL(("injectHandler", "injecting handler %s into %s\n",
                         handler->handler_name, tptr->label_a));
-            netsnmp_inject_handler_before(tptr->reginfo, clone_handler(handler),
+            netsnmp_inject_handler_before(tptr->reginfo, _clone_handler(handler),
                                           before_what);
         } else if (tptr->reginfo != NULL &&
 		   tptr->reginfo->handlerName != NULL &&
                    strcmp(tptr->reginfo->handlerName, name) == 0) {
             DEBUGMSGTL(("injectHandler", "injecting handler into %s/%s\n",
                         tptr->label_a, tptr->reginfo->handlerName));
-            netsnmp_inject_handler_before(tptr->reginfo, clone_handler(handler),
+            netsnmp_inject_handler_before(tptr->reginfo, _clone_handler(handler),
                                           before_what);
         } else {
             for (mh = tptr->reginfo->handler; mh != NULL; mh = mh->next) {
@@ -849,7 +930,7 @@ netsnmp_inject_handler_into_subtree(netsnmp_subtree *tp, const char *name,
                     DEBUGMSGTL(("injectHandler", "injecting handler into %s\n",
                                 tptr->label_a));
                     netsnmp_inject_handler_before(tptr->reginfo,
-                                                  clone_handler(handler),
+                                                  _clone_handler(handler),
                                                   before_what);
                     break;
                 } else {
@@ -941,6 +1022,38 @@ netsnmp_init_handler_conf(void)
     se_add_pair_to_slist("agent_mode", strdup("SET_FREE"), MODE_SET_FREE);
     se_add_pair_to_slist("agent_mode", strdup("SET_UNDO"), MODE_SET_UNDO);
 
+    se_add_pair_to_slist("babystep_mode", strdup("pre-request"),
+                         MODE_BSTEP_PRE_REQUEST);
+    se_add_pair_to_slist("babystep_mode", strdup("object_lookup"),
+                         MODE_BSTEP_OBJECT_LOOKUP);
+    se_add_pair_to_slist("babystep_mode", strdup("check_value"),
+                         MODE_BSTEP_CHECK_VALUE);
+    se_add_pair_to_slist("babystep_mode", strdup("row_create"),
+                         MODE_BSTEP_ROW_CREATE);
+    se_add_pair_to_slist("babystep_mode", strdup("undo_setup"),
+                         MODE_BSTEP_UNDO_SETUP);
+    se_add_pair_to_slist("babystep_mode", strdup("set_value"),
+                         MODE_BSTEP_SET_VALUE);
+    se_add_pair_to_slist("babystep_mode", strdup("check_consistency"),
+                         MODE_BSTEP_CHECK_CONSISTENCY);
+    se_add_pair_to_slist("babystep_mode", strdup("undo_set"),
+                         MODE_BSTEP_UNDO_SET);
+    se_add_pair_to_slist("babystep_mode", strdup("commit"),
+                         MODE_BSTEP_COMMIT);
+    se_add_pair_to_slist("babystep_mode", strdup("undo_commit"),
+                         MODE_BSTEP_UNDO_COMMIT);
+    se_add_pair_to_slist("babystep_mode", strdup("irreversible_commit"),
+                         MODE_BSTEP_IRREVERSIBLE_COMMIT);
+    se_add_pair_to_slist("babystep_mode", strdup("undo_cleanup"),
+                         MODE_BSTEP_UNDO_CLEANUP);
+    se_add_pair_to_slist("babystep_mode", strdup("post_request"),
+                         MODE_BSTEP_POST_REQUEST);
+    se_add_pair_to_slist("babystep_mode", strdup("original"), 0xffff);
+
+    /*
+     * xxx-rks: hmmm.. will this work for modes which are or'd together?
+     *          I'm betting not...
+     */
     se_add_pair_to_slist("handler_can_mode", strdup("GET/GETNEXT"),
                          HANDLER_CAN_GETANDGETNEXT);
     se_add_pair_to_slist("handler_can_mode", strdup("SET"),
