@@ -170,6 +170,7 @@ netsnmp_subtree *
 add_subtree(netsnmp_subtree *new_tree, const char *context_name)
 {
     subtree_context_cache *ptr = SNMP_MALLOC_TYPEDEF(subtree_context_cache);
+    
     if (!context_name) {
         context_name = "";
     }
@@ -177,13 +178,15 @@ add_subtree(netsnmp_subtree *new_tree, const char *context_name)
     if (!ptr) {
         return NULL;
     }
-
+    
     DEBUGMSGTL(("subtree", "adding subtree for context: \"%s\"\n",	
 		context_name));
+
     ptr->next = context_subtrees;
     ptr->first_subtree = new_tree;
     ptr->context_name = strdup(context_name);
     context_subtrees = ptr;
+
     return ptr->first_subtree;
 }
 
@@ -380,6 +383,23 @@ netsnmp_subtree_load(netsnmp_subtree *new_sub, const char *context_name)
         return MIB_REGISTERED_OK;       /* Degenerate case */
     }
 
+    if (!netsnmp_subtree_find_first(context_name)) {
+        static int inloop = 0;
+        if (!inloop) {
+            oid ccitt[1]           = { 0 };
+            oid iso[1]             = { 1 };
+            oid joint_ccitt_iso[1] = { 2 };
+            inloop = 1;
+            netsnmp_register_null_context(snmp_duplicate_objid(ccitt, 1), 1,
+                                          context_name);
+            netsnmp_register_null_context(snmp_duplicate_objid(iso, 1), 1,
+                                          context_name);
+            netsnmp_register_null_context(snmp_duplicate_objid(joint_ccitt_iso, 1),
+                                          1, context_name);
+            inloop = 0;
+        }
+    }
+
     /*  Find the subtree that contains the start of the new subtree (if
 	any)...*/
 
@@ -489,6 +509,7 @@ netsnmp_subtree_load(netsnmp_subtree *new_sub, const char *context_name)
 	
 	    if (next && (next->namelen  == new_sub->namelen) &&
 		(next->priority == new_sub->priority)) {
+                netsnmp_assert("registration" != "duplicate");
 		return MIB_DUPLICATE_REGISTRATION;
 	    }
 
@@ -577,7 +598,7 @@ netsnmp_register_mib(const char *moduleName,
     subtree->label_a = strdup(moduleName);
     if (subtree->name_a == NULL || subtree->start_a == NULL || 
 	subtree->end_a  == NULL || subtree->label_a == NULL) {
-	netsnmp_subtree_free(subtree);
+	netsnmp_subtree_free(subtree); /* also frees reginfo */
 	return MIB_REGISTRATION_FAILED;
     }
     subtree->namelen   = (u_char)mibloclen;
@@ -588,7 +609,7 @@ netsnmp_register_mib(const char *moduleName,
     if (var != NULL) {
 	subtree->variables = (struct variable *)malloc(varsize*numvars);
 	if (subtree->variables == NULL) {
-	    netsnmp_subtree_free(subtree);
+	    netsnmp_subtree_free(subtree); /* also frees reginfo */
 	    return MIB_REGISTRATION_FAILED;
 	}
 	memcpy(subtree->variables, var, numvars*varsize);
@@ -625,7 +646,7 @@ netsnmp_register_mib(const char *moduleName,
             sub2->name_a[range_subid - 1]  = i;
             sub2->start_a[range_subid - 1] = i;
             sub2->end_a[range_subid - 1]   = i;     /* XXX - ???? */
-            if (range_subid == mibloclen)
+            if (range_subid == (int)mibloclen)
                 ++sub2->end_a[range_subid - 1];
             res = netsnmp_subtree_load(sub2, context);
             sub2->flags |= SUBTREE_ATTACHED;
@@ -657,7 +678,7 @@ netsnmp_register_mib(const char *moduleName,
 	}
     }
 
-    if (perform_callback) {
+    if (res == MIB_REGISTERED_OK && perform_callback) {
         memset(&reg_parms, 0x0, sizeof(reg_parms));
         reg_parms.name = mibloc;
         reg_parms.namelen = mibloclen;
@@ -666,12 +687,7 @@ netsnmp_register_mib(const char *moduleName,
         reg_parms.range_ubound = range_ubound;
         reg_parms.timeout = timeout;
         reg_parms.flags = (u_char) flags;
-
-        /*
-         * Should this really be called if the registration hasn't actually 
-         * succeeded?  
-         */
-
+        reg_parms.contextName = context;
         snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                             SNMPD_CALLBACK_REGISTER_OID, &reg_parms);
     }
@@ -705,6 +721,8 @@ register_mib_reattach_node(netsnmp_subtree *s)
         reg_parms.range_ubound = s->range_ubound;
         reg_parms.timeout = s->timeout;
         reg_parms.flags = s->flags;
+        if ((NULL != s->reginfo) && (NULL != s->reginfo->contextName))
+            reg_parms.contextName = s->reginfo->contextName;
         snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                             SNMPD_CALLBACK_REGISTER_OID, &reg_parms);
         s->flags |= SUBTREE_ATTACHED;
@@ -969,6 +987,7 @@ unregister_mib_context(oid * name, size_t len, int priority,
     reg_parms.range_subid = range_subid;
     reg_parms.range_ubound = range_ubound;
     reg_parms.flags = 0x00;     /*  this is okay I think  */
+    reg_parms.contextName = context;
     snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                         SNMPD_CALLBACK_UNREGISTER_OID, &reg_parms);
 
@@ -1043,6 +1062,7 @@ netsnmp_unregister_mib_table_row(oid * name, size_t len, int priority,
     reg_parms.range_subid = var_subid;
     reg_parms.range_ubound = range_ubound;
     reg_parms.flags = 0x00;     /*  this is okay I think  */
+    reg_parms.contextName = context;
     snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                         SNMPD_CALLBACK_UNREGISTER_OID, &reg_parms);
 
@@ -1102,6 +1122,9 @@ unregister_mibs_by_session(netsnmp_session * ss)
                     rp.range_ubound = child->range_ubound;
                     rp.timeout = child->timeout;
                     rp.flags = child->flags;
+                    if ((NULL != child->reginfo) &&
+                        (NULL != child->reginfo->contextName))
+                        rp.contextName = child->reginfo->contextName;
 
                     if (child->reginfo != NULL) {
                         /*
@@ -1149,9 +1172,11 @@ in_a_view(oid *name, size_t *namelen, netsnmp_pdu *pdu, int type)
     /*
      * check for v1 and counter64s, since snmpv1 doesn't support it 
      */
+#ifndef DISABLE_SNMPV1
     if (pdu->version == SNMP_VERSION_1 && type == ASN_COUNTER64) {
         return VACM_NOTINVIEW;
     }
+#endif
 
     view_parms.pdu = pdu;
     view_parms.name = name;
@@ -1164,8 +1189,12 @@ in_a_view(oid *name, size_t *namelen, netsnmp_pdu *pdu, int type)
     view_parms.check_subtree = 0;
 
     switch (pdu->version) {
+#ifndef DISABLE_SNMPV1
     case SNMP_VERSION_1:
+#endif
+#ifndef DISABLE_SNMPV2C
     case SNMP_VERSION_2c:
+#endif
     case SNMP_VERSION_3:
         snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                             SNMPD_CALLBACK_ACM_CHECK, &view_parms);
@@ -1195,8 +1224,12 @@ check_access(netsnmp_pdu *pdu)
     }
 
     switch (pdu->version) {
+#ifndef DISABLE_SNMPV1
     case SNMP_VERSION_1:
+#endif
+#ifndef DISABLE_SNMPV2C
     case SNMP_VERSION_2c:
+#endif
     case SNMP_VERSION_3:
         snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                             SNMPD_CALLBACK_ACM_CHECK_INITIAL, &view_parms);
@@ -1228,8 +1261,12 @@ netsnmp_acm_check_subtree(netsnmp_pdu *pdu, oid *name, size_t namelen)
     }
 
     switch (pdu->version) {
+#ifndef DISABLE_SNMPV1
     case SNMP_VERSION_1:
+#endif
+#ifndef DISABLE_SNMPV2C
     case SNMP_VERSION_2c:
+#endif
     case SNMP_VERSION_3:
         snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                             SNMPD_CALLBACK_ACM_CHECK_SUBTREE, &view_parms);
