@@ -36,6 +36,11 @@ static void     table_helper_cleanup(netsnmp_agent_request_info *reqinfo,
                                      netsnmp_request_info *request,
                                      int status);
 static void     table_data_free_func(void *data);
+static int
+sparse_table_helper_handler(netsnmp_mib_handler *handler,
+                            netsnmp_handler_registration *reginfo,
+                            netsnmp_agent_request_info *reqinfo,
+                            netsnmp_request_info *requests);
 
 /** @defgroup table table: Helps you implement a table.
  *  @ingroup handler
@@ -173,10 +178,14 @@ table_helper_handler(netsnmp_mib_handler *handler,
         return SNMP_ERR_GENERR;
     }
 
-    DEBUGMSGTL(("helper:table", "Got request for handler %s: base oid:",
-                handler->handler_name));
-    DEBUGMSGOID(("helper:table", reginfo->rootoid, reginfo->rootoid_len));
-    DEBUGMSG(("helper:table", "\n"));
+    DEBUGIF("helper:table:req") {
+        DEBUGMSGTL(("helper:table:req",
+                    "Got request for handler %s: base oid:",
+                    handler->handler_name));
+        DEBUGMSGOID(("helper:table:req", reginfo->rootoid,
+                     reginfo->rootoid_len));
+        DEBUGMSG(("helper:table:req", "\n"));
+    }
     
     /*
      * if the agent request info has a state reference, then this is a 
@@ -227,7 +236,7 @@ table_helper_handler(netsnmp_mib_handler *handler,
         DEBUGMSG(("verbose:table", "\n"));
 
         if (request->processed) {
-            DEBUGMSG(("helper:table", "already processed\n"));
+            DEBUGMSG(("verbose:table", "already processed\n"));
             continue;
         }
         netsnmp_assert(request->status == SNMP_ERR_NOERROR);
@@ -247,23 +256,23 @@ table_helper_handler(netsnmp_mib_handler *handler,
         }
 
         if (reqinfo->mode == MODE_SET_RESERVE1) {
-            DEBUGIF("helper:table") {
+            DEBUGIF("helper:table:set") {
                 u_char         *buf = NULL;
                 size_t          buf_len = 0, out_len = 0;
-                DEBUGMSGTL(("helper:table", " SET_REQUEST for OID: "));
-                DEBUGMSGOID(("helper:table", var->name, var->name_length));
+                DEBUGMSGTL(("helper:table:set", " SET_REQUEST for OID: "));
+                DEBUGMSGOID(("helper:table:set", var->name, var->name_length));
                 out_len = 0;
                 if (sprint_realloc_by_type(&buf, &buf_len, &out_len, 1,
                                            var, 0, 0, 0)) {
-                    DEBUGMSG(("helper:table"," type=%d(%02x), value=%s\n",
+                    DEBUGMSG(("helper:table:set"," type=%d(%02x), value=%s\n",
                               var->type, var->type, buf));
                 } else {
                     if (buf != NULL) {
-                        DEBUGMSG(("helper:table",
+                        DEBUGMSG(("helper:table:set",
                                   " type=%d(%02x), value=%s [TRUNCATED]\n",
                                   var->type, var->type, buf));
                     } else {
-                        DEBUGMSG(("helper:table",
+                        DEBUGMSG(("helper:table:set",
                                   " type=%d(%02x), value=[NIL] [TRUNCATED]\n",
                                   var->type, var->type));
                     }
@@ -342,24 +351,37 @@ table_helper_handler(netsnmp_mib_handler *handler,
          */
 
         incomplete = 0;
-        tbl_req_info = SNMP_MALLOC_TYPEDEF(netsnmp_table_request_info);
-        tbl_req_info->reg_info = tbl_info;
-        tbl_req_info->indexes = snmp_clone_varbind(tbl_info->indexes);
-        tbl_req_info->number_indexes = 0;       /* none yet */
-        netsnmp_request_add_list_data(request,
-                                      netsnmp_create_data_list
-                                      (TABLE_HANDLER_NAME,
-                                       (void *) tbl_req_info,
-                                       table_data_free_func));
+        tbl_req_info = netsnmp_extract_table_info(request);
+        if (NULL == tbl_req_info) {
+            tbl_req_info = SNMP_MALLOC_TYPEDEF(netsnmp_table_request_info);
+            tbl_req_info->reg_info = tbl_info;
+            tbl_req_info->indexes = snmp_clone_varbind(tbl_info->indexes);
+            tbl_req_info->number_indexes = 0;       /* none yet */
+            netsnmp_request_add_list_data(request,
+                                          netsnmp_create_data_list
+                                          (TABLE_HANDLER_NAME,
+                                           (void *) tbl_req_info,
+                                           table_data_free_func));
+        } else {
+            DEBUGMSGTL(("helper:table", "  using existing tbl_req_info\n "));
+        }
 
+        /*
+         * do we have a column?
+         */
         if (var->name_length > oid_column_pos) {
             /*
-             * oid is long enough to contain index info
+             * oid is long enough to contain COLUMN info
              */
+            DEBUGMSGTL(("helper:table:col", "  have at least a column (%d)\n",
+                        var->name[oid_column_pos]));
             if (var->name[oid_column_pos] < tbl_info->min_column) {
+                DEBUGMSGTL(("helper:table:col",
+                            "    but it's less than min (%d)\n",
+                            tbl_info->min_column));
                 if (reqinfo->mode == MODE_GETNEXT) {
                     /*
-                     * fix column, truncate useless index info 
+                     * fix column, truncate useless column info 
                      */
                     var->name_length = oid_column_pos;
                     tbl_req_info->colnum = tbl_info->min_column;
@@ -367,6 +389,8 @@ table_helper_handler(netsnmp_mib_handler *handler,
                     out_of_range = 1;
             } else if (var->name[oid_column_pos] > tbl_info->max_column)
                 out_of_range = 1;
+            else
+                tbl_req_info->colnum = var->name[oid_column_pos];
 
             if (out_of_range) {
                 /*
@@ -374,7 +398,7 @@ table_helper_handler(netsnmp_mib_handler *handler,
                  * memory 
                  */
                 DEBUGMSGTL(("helper:table",
-                            "  oid is out of range. Not processed: "));
+                            "    oid is out of range. Not processed: "));
                 DEBUGMSGOID(("helper:table", var->name, var->name_length));
                 DEBUGMSG(("helper:table", "\n"));
 
@@ -394,24 +418,37 @@ table_helper_handler(netsnmp_mib_handler *handler,
                 tbl_req_info->colnum =
                     netsnmp_closest_column(var->name[oid_column_pos],
                                            tbl_info->valid_columns);
+                DEBUGMSGTL(("helper:table:col", "    closest column is %d\n",
+                            tbl_req_info->colnum));
+                /*
+                 * xxx-rks: document why the continue...
+                 */
                 if (tbl_req_info->colnum == 0)
                     continue;
                 if (tbl_req_info->colnum != var->name[oid_column_pos]) {
+                    DEBUGMSGTL(("helper:table:col",
+                                "    which doesn't match req %d - truncating index info\n",
+                                   var->name[oid_column_pos]));
                     /*
                      * different column! truncate useless index info 
                      */
-                    var->name_length = oid_column_pos;
+                    var->name_length = oid_column_pos + 1; /* pos is 0 based */
                 }
             }
             /*
              * var->name_length may have changed - check again 
              */
-            if (var->name_length <= oid_column_pos) { /** none available */
-                tbl_req_info->index_oid_len = 0;
+            if ((int)var->name_length <= oid_index_pos) { /* pos is 0 based */
+                DEBUGMSGTL(("helper:table", "    not enough for indexes\n"));
+                tbl_req_info->index_oid_len = 0; /** none available */
             } else {
-                tbl_req_info->colnum = var->name[oid_column_pos];
+                /*
+                 * oid is long enough to contain INDEX info
+                 */
                 tbl_req_info->index_oid_len =
                     var->name_length - oid_index_pos;
+                DEBUGMSGTL(("helper:table", "    have %d bytes of index\n",
+                            tbl_req_info->index_oid_len));
                 netsnmp_assert(tbl_req_info->index_oid_len < MAX_OID_LEN);
                 memcpy(tbl_req_info->index_oid, &var->name[oid_index_pos],
                        tbl_req_info->index_oid_len * sizeof(oid));
@@ -432,6 +469,7 @@ table_helper_handler(netsnmp_mib_handler *handler,
              * at the minimum column. Set index oid len to 0 because we don't
              * have any index info in the OID.
              */
+            DEBUGMSGTL(("helper:table", "  no column/index in request\n"));
             tbl_req_info->index_oid_len = 0;
             tbl_req_info->colnum = tbl_info->min_column;
         }
@@ -452,7 +490,8 @@ table_helper_handler(netsnmp_mib_handler *handler,
         /*
          * for each index type, try to extract the index from var->name
          */
-
+        DEBUGMSGTL(("helper:table", "  looking for %d indexes\n",
+                    tbl_info->number_indexes));
         for (tmp_idx = 0, vb = tbl_req_info->indexes;
              tmp_idx < tbl_info->number_indexes;
              ++tmp_idx, vb = vb->next_variable) {
@@ -491,6 +530,8 @@ table_helper_handler(netsnmp_mib_handler *handler,
                 /*
                  * do not count incomplete indexes 
                  */
+                DEBUGMSGTL(("helper:table", "  got 1 (incomplete=%d)\n",
+                            incomplete));
                 if (incomplete)
                     continue;
                 ++tbl_req_info->number_indexes; /** got one ok */
@@ -502,12 +543,15 @@ table_helper_handler(netsnmp_mib_handler *handler,
             }
         }                       /** for loop */
 
-        DEBUGIF("helper:table") {
+        DEBUGIF("helper:table:results") {
+            DEBUGMSGTL(("helper:table:results", "  found %d indexes\n",
+                        tbl_req_info->number_indexes));
             if (!cleaned_up) {
                 unsigned int    count;
                 u_char         *buf = NULL;
                 size_t          buf_len = 0, out_len = 0;
-                DEBUGMSGTL(("helper:table", "  column: %d, indexes: %d",
+                DEBUGMSGTL(("helper:table:results",
+                            "  column: %d, indexes: %d",
                             tbl_req_info->colnum,
                             tbl_req_info->number_indexes));
                 for (vb = tbl_req_info->indexes, count = 0;
@@ -516,16 +560,16 @@ table_helper_handler(netsnmp_mib_handler *handler,
                     out_len = 0;
                     if (sprint_realloc_by_type(&buf, &buf_len, &out_len, 1,
                                                vb, 0, 0, 0)) {
-                        DEBUGMSG(("helper:table",
+                        DEBUGMSG(("helper:table:results",
                                   "   index: type=%d(%02x), value=%s",
                                   vb->type, vb->type, buf));
                     } else {
                         if (buf != NULL) {
-                            DEBUGMSG(("helper:table",
+                            DEBUGMSG(("helper:table:results",
                                       "   index: type=%d(%02x), value=%s [TRUNCATED]",
                                       vb->type, vb->type, buf));
                         } else {
-                            DEBUGMSG(("helper:table",
+                            DEBUGMSG(("helper:table:results",
                                       "   index: type=%d(%02x), value=[NIL] [TRUNCATED]",
                                       vb->type, vb->type));
                         }
@@ -534,7 +578,7 @@ table_helper_handler(netsnmp_mib_handler *handler,
                 if (buf != NULL) {
                     free(buf);
                 }
-                DEBUGMSG(("helper:table", "\n"));
+                DEBUGMSG(("helper:table:results", "\n"));
             }
         }
 
@@ -559,15 +603,132 @@ table_helper_handler(netsnmp_mib_handler *handler,
     }
 
     /*
-     * * call our child access function 
+     * bail if there is nothing for our child handlers
      */
-    if (need_processing)
-        status =
-            netsnmp_call_next_handler(handler, reginfo, reqinfo, requests);
+    if (0 == need_processing)
+        return status;
+
+    /*
+     * call our child access function 
+     */
+    status =
+        netsnmp_call_next_handler(handler, reginfo, reqinfo, requests);
+
+    /*
+     * check for sparse tables
+     */
+    if (reqinfo->mode == MODE_GETNEXT)
+        sparse_table_helper_handler( handler, reginfo, reqinfo, requests );
 
     return status;
 }
 
+#define SPARSE_TABLE_HANDLER_NAME "sparse_table"
+
+/** implements the sparse table helper handler
+ * @internal
+ *
+ * @note
+ * This function is static to prevent others from calling it
+ * directly. It it automatically called by the table helper,
+ * 
+ */
+static int
+sparse_table_helper_handler(netsnmp_mib_handler *handler,
+                     netsnmp_handler_registration *reginfo,
+                     netsnmp_agent_request_info *reqinfo,
+                     netsnmp_request_info *requests)
+{
+    int             status = SNMP_ERR_NOERROR;
+    netsnmp_request_info *request;
+    oid             coloid[MAX_OID_LEN];
+    netsnmp_table_request_info *table_info;
+
+    /*
+     * since we don't call child handlers, warn if one was registered
+     * beneath us. A special exception for the table helper, which calls
+     * the handler directly. Use handle custom flag to only log once.
+     */
+    if((table_helper_handler != handler->access_method) &&
+       (NULL != handler->next)) {
+        /*
+         * always warn if called without our own handler. If we
+         * have our own handler, use custom bit 1 to only log once.
+         */
+        if((sparse_table_helper_handler != handler->access_method) ||
+           !(handler->flags & MIB_HANDLER_CUSTOM1)) {
+            snmp_log(LOG_WARNING, "handler (%s) registered after sparse table "
+                     "hander will not be called\n",
+                     handler->next->handler_name ?
+                     handler->next->handler_name : "" );
+            if(sparse_table_helper_handler == handler->access_method)
+                handler->flags |= MIB_HANDLER_CUSTOM1;
+        }
+    }
+
+    if (reqinfo->mode == MODE_GETNEXT) {
+        for(request = requests ; request; request = request->next) {
+            if (request->requestvb->type == ASN_NULL && request->processed)
+                continue;
+            if (request->requestvb->type == ASN_NULL ||
+                request->requestvb->type == SNMP_NOSUCHINSTANCE) {
+                /*
+                 * get next skipped this value for this column, we
+                 * need to keep searching forward 
+                 */
+                DEBUGMSGT(("sparse", "retry for NOSUCHINSTANCE\n"));
+                request->requestvb->type = ASN_PRIV_RETRY;
+            }
+            if (request->requestvb->type == SNMP_NOSUCHOBJECT ||
+                request->requestvb->type == SNMP_ENDOFMIBVIEW) {
+                /*
+                 * get next has completely finished with this column,
+                 * so we need to try with the next column (if any)
+                 */
+                DEBUGMSGT(("sparse", "retry for NOSUCHOBJECT\n"));
+                table_info = netsnmp_extract_table_info(request);
+                table_info->colnum = netsnmp_table_next_column(table_info);
+                if (0 != table_info->colnum) {
+                    memcpy(coloid, reginfo->rootoid,
+                           reginfo->rootoid_len * sizeof(oid));
+                    coloid[reginfo->rootoid_len]   = 1;   /* table.entry node */
+                    coloid[reginfo->rootoid_len+1] = table_info->colnum;
+                    snmp_set_var_objid(request->requestvb,
+                                       coloid, reginfo->rootoid_len + 2);
+                    
+                    request->requestvb->type = ASN_PRIV_RETRY;
+                }
+            }
+        }
+    }
+    return status;
+}
+
+/** create sparse table handler
+ */
+netsnmp_mib_handler *
+netsnmp_sparse_table_handler_get(void)
+{
+    return netsnmp_create_handler(SPARSE_TABLE_HANDLER_NAME,
+                                  sparse_table_helper_handler);
+}
+
+/** creates a table handler given the netsnmp_table_registration_info object,
+ *  inserts it into the request chain and then calls
+ *  netsnmp_register_handler() to register the table into the agent.
+ */
+int
+netsnmp_sparse_table_register(netsnmp_handler_registration *reginfo,
+                       netsnmp_table_registration_info *tabreq)
+{
+    netsnmp_inject_handler(reginfo,
+        netsnmp_create_handler(SPARSE_TABLE_HANDLER_NAME,
+                               sparse_table_helper_handler));
+    netsnmp_inject_handler(reginfo, netsnmp_get_table_handler(tabreq));
+    return netsnmp_register_handler(reginfo);
+}
+
+
 /** Builds the result to be returned to the agent given the table information.
  *  Use this function to return results from lowel level handlers to
  *  the agent.  It takes care of building the proper resulting oid
@@ -622,6 +783,9 @@ netsnmp_table_build_oid(netsnmp_handler_registration *reginfo,
      * xxx-rks: inefficent. we do a copy here, then build_oid does it
      *          again. either come up with a new utility routine, or
      *          do some hijinks here to eliminate extra copy.
+     *          Probably could make sure all callers have the
+     *          index & variable list updated, and use
+     *          netsnmp_table_build_oid_from_index() instead of all this.
      */
     memcpy(tmpoid, reginfo->rootoid, reginfo->rootoid_len * sizeof(oid));
     tmpoid[reginfo->rootoid_len] = 1;   /** .Entry */
@@ -932,49 +1096,29 @@ netsnmp_table_get_or_create_row_stash(netsnmp_agent_request_info *reqinfo,
     return stashp;
 }
 
-netsnmp_index *
-netsnmp_table_index_find_next_row(netsnmp_container *c,
-                                  netsnmp_table_request_info *tblreq)
+/*
+ * advance the table info colnum to the next column, or 0 if there are no more
+ *
+ * @return new column, or 0 if there are no more
+ */
+unsigned int
+netsnmp_table_next_column(netsnmp_table_request_info *table_info)
 {
-    netsnmp_index *row = NULL;
-
-    if (!c || !tblreq || !tblreq->reg_info) {
-        snmp_log(LOG_ERR,"netsnmp_table_index_find_next_row param error\n");
-        return NULL;
-    }
+    if (NULL == table_info)
+        return 0;
 
     /*
-     * no indexes, or below our minimum column? then use first row.
+     * try and validate next column
      */
-    if(tblreq->colnum < tblreq->reg_info->min_column) {
-        tblreq->colnum = tblreq->reg_info->min_column;
-        tblreq->number_indexes = 0;
-    }
-    if(tblreq->number_indexes == 0) {
-        row = CONTAINER_FIRST(c);
-    } else {
-        netsnmp_index index;
-        index.oids = tblreq->index_oid;
-        index.len = tblreq->index_oid_len;
-
-        row = CONTAINER_NEXT(c, &index);
-
-        /*
-         * we don't have a row, but we might be at the end of a
-         * column, so try the next column.
-         */
-        if (!row) {
-            ++tblreq->colnum;
-            if (tblreq->reg_info->valid_columns) {
-                tblreq->colnum = netsnmp_closest_column
-                    (tblreq->colnum,tblreq->reg_info->valid_columns);
-            } else if (tblreq->colnum > tblreq->reg_info->max_column)
-                tblreq->colnum = 0;
-
-            if (tblreq->colnum != 0)
-                row = CONTAINER_FIRST(c);
-        }
-    }
-
-    return row;
+    if (table_info->reg_info->valid_columns)
+        return netsnmp_closest_column(table_info->colnum + 1,
+                                      table_info->reg_info->valid_columns);
+    
+    /*
+     * can't validate. assume 1..max_column are valid
+     */
+    if (table_info->colnum < table_info->reg_info->max_column)
+        return table_info->colnum + 1;
+    
+    return 0; /* out of range */
 }
