@@ -1,3 +1,6 @@
+/*
+ * table_iterator.c 
+ */
 /* Portions of this file are subject to the following copyright(s).  See
  * the Net-SNMP's COPYING file for more details and other copyrights
  * that may apply:
@@ -327,7 +330,7 @@ netsnmp_table_iterator_helper_handler(netsnmp_mib_handler *handler,
     ti_cache_info  *ti_info = NULL;
     int             request_count = 0;
     netsnmp_oid_stash_node **cinfo = NULL;
-    netsnmp_variable_list *old_indexes, *vb;
+    netsnmp_variable_list *old_indexes = NULL, *vb;
     netsnmp_table_registration_info *table_reg_info = NULL;
     int i;
     netsnmp_data_list    *ldata;
@@ -696,20 +699,6 @@ netsnmp_table_iterator_helper_handler(netsnmp_mib_handler *handler,
 
     /* reverse the previously saved mode if we were a getnext */
     if (oldmode == MODE_GETNEXT) {
-        for(request = requests ; request; request = request->next) {
-            if (request->requestvb->type == ASN_NULL ||
-                request->requestvb->type == SNMP_NOSUCHOBJECT ||
-                request->requestvb->type == SNMP_NOSUCHINSTANCE) {
-                /*
-                 * get next skipped this value for this column, we
-                 * need to keep searching forward 
-                 */
-                if (request->processed != TABLE_ITERATOR_NOTAGAIN)
-                    request->requestvb->type = ASN_PRIV_RETRY;
-                else
-                    request->processed = 1;
-            }
-        }
         reqinfo->mode = oldmode;
     }
 
