@@ -18,10 +18,29 @@
  * SOFTWARE. 
  ******************************************************************/
 
-#include <stdlib.h>
-#include <sys/time.h>
-#include <unistd.h>
 #include <net-snmp/net-snmp-config.h>
+
+#if HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#if TIME_WITH_SYS_TIME
+# ifdef WIN32
+#  include <sys/timeb.h>
+# else
+#  include <sys/time.h>
+# endif
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include "util_funcs.h"
@@ -135,7 +154,6 @@ fetch_var_val(oid * name, size_t namelen, u_long * new_value)
     WriteMethod    *write_method;
     struct variable called_var;
     register struct variable *s_var_ptr = NULL;
-    register int    iii;
     register u_char *access;
 
 
