@@ -47,7 +47,6 @@
 #include "snmpd.h"
 #include "agentx/protocol.h"
 #include "agentx/client.h"
-#include "agentx/agentx_config.h"
 #include <net-snmp/agent/agent_callbacks.h>
 #include <net-snmp/agent/agent_trap.h>
 #ifdef USING_MIBII_SYSORTABLE_MODULE
@@ -98,10 +97,6 @@ init_subagent(void)
                "the callback transport is not available.\n");
     return;
 #else
-    snmpd_register_config_handler("agentxsocket",
-                                  agentx_parse_agentx_socket, NULL,
-                                  "AgentX bind address");
-
     if (agentx_callback_sess == NULL) {
         agentx_callback_sess = netsnmp_callback_open(callback_master_num,
                                                      handle_subagent_response,
@@ -411,6 +406,10 @@ handle_agentx_packet(int operation, netsnmp_session * session, int reqid,
      */
 
     internal_pdu = snmp_clone_pdu(pdu);
+    internal_pdu->contextName = internal_pdu->community;
+    internal_pdu->contextNameLen = internal_pdu->community_len;
+    internal_pdu->community = NULL;
+    internal_pdu->community_len = 0;
     snmp_async_send(agentx_callback_sess, internal_pdu, mycallback,
                     retmagic);
     return 1;
@@ -572,13 +571,15 @@ agentx_registration_callback(int majorID, int minorID, void *serverarg,
                                reg_parms->priority,
                                reg_parms->range_subid,
                                reg_parms->range_ubound, reg_parms->timeout,
-                               reg_parms->flags);
+                               reg_parms->flags,
+                               reg_parms->contextName);
     else
         return agentx_unregister(agentx_ss,
                                  reg_parms->name, reg_parms->namelen,
                                  reg_parms->priority,
                                  reg_parms->range_subid,
-                                 reg_parms->range_ubound);
+                                 reg_parms->range_ubound,
+                                 reg_parms->contextName);
 }
 
 
