@@ -314,8 +314,9 @@ parse_storage_config(const char *token, char *cptr)
 {
     char *val;
     int ival;
+    char *st;
 
-    val = strtok(cptr, " \t");
+    val = strtok_r(cptr, " \t", &st);
     if (!val) {
         config_perror("Missing FLAG parameter in storageUseNFS");
         return;
@@ -488,7 +489,7 @@ var_hrstore(struct variable *vp,
             return NULL;
     } else {
 
-try_next:
+really_try_next:
 	store_idx = header_hrstoreEntry(vp, name, length, exact, var_len,
 					write_method);
 	if (store_idx == MATCH_FAILED)
@@ -606,6 +607,8 @@ try_next:
             case HRS_TYPE_MBUF:
 #ifdef MSIZE
                 long_return = MSIZE;
+#elif defined(linux)
+                long_return = 1024;
 #else
                 long_return = 256;
 #endif
@@ -624,6 +627,7 @@ try_next:
         else
             switch (store_idx) {
 #if defined(linux)
+            case HRS_TYPE_MBUF:
             case HRS_TYPE_MEM:
             case HRS_TYPE_SWAP:
                 long_return = linux_mem(store_idx, HRSTORE_SIZE);
@@ -691,6 +695,7 @@ try_next:
         else
             switch (store_idx) {
 #if defined(linux)
+            case HRS_TYPE_MBUF:
             case HRS_TYPE_MEM:
             case HRS_TYPE_SWAP:
                 long_return = linux_mem(store_idx, HRSTORE_USED);
@@ -726,7 +731,7 @@ try_next:
                 long_return = (mbpool.pr_nget - mbpool.pr_nput)
                     * mbpool.pr_size + (mclpool.pr_nget - mclpool.pr_nput)
                     * mclpool.pr_size;
-#elif defined(MBSTAT_SYMBOL)
+#elif defined(MBSTAT_SYMBOL) && defined(STRUCT_MBSTAT_HAS_M_CLUSTERS)
                 long_return = mbstat.m_clusters - mbstat.m_clfree;      /* unlikely, but... */
 #elif defined(NO_DUMMY_VALUES)
                 goto try_next;
@@ -777,6 +782,12 @@ try_next:
                     vp->magic));
     }
     return NULL;
+
+  try_next:
+    if (!exact)
+        goto really_try_next;
+
+    return NULL;
 }
 
 
@@ -792,7 +803,7 @@ static int      HRS_index;
 void
 Init_HR_Store(void)
 {
-#if !defined(solaris2) && !defined(hpux10) && !defined(hpux11) && !defined(linux)
+#if !defined(solaris2) && !defined(hpux10) && !defined(hpux11)
     HRS_index = 0;
 #else
     HRS_index = HRS_TYPE_MBUF;
@@ -836,23 +847,32 @@ linux_mem(int mem_type, int size_or_used)
 {
     FILE           *fp;
     char            buf[1024];
-    int             size = -1, free = -1;
+    int             size = -1, free = -1, buffers = -1;
 
     if ((fp = fopen("/proc/meminfo", "r")) == NULL)
         return -1;
 
     while (fgets(buf, sizeof(buf), fp) != NULL) {
-       if ((!strncmp(buf, "MemTotal:", 9) && mem_type == HRS_TYPE_MEM) ||
+       if ((!strncmp(buf, "MemTotal:", 9) && (mem_type == HRS_TYPE_MEM || mem_type == HRS_TYPE_MBUF)) ||
             (!strncmp(buf, "SwapTotal:", 10) && mem_type == HRS_TYPE_SWAP))
             sscanf(buf, "%*s %d", &size);
 
        if ((!strncmp(buf, "MemFree:", 8) && mem_type == HRS_TYPE_MEM) ||
             (!strncmp(buf, "SwapFree:", 9) && mem_type == HRS_TYPE_SWAP))
             sscanf(buf, "%*s %d", &free);
+
+       if ((!strncmp(buf, "Buffers:", 8) && mem_type == HRS_TYPE_MBUF))
+		sscanf(buf, "%*s %d", &buffers);
     }
 
     fclose(fp);
-    return (size_or_used == HRSTORE_SIZE ? size : (size - free));
+
+    switch (mem_type) {
+        case HRS_TYPE_MBUF:
+            return (size_or_used == HRSTORE_SIZE ? size : buffers);
+        default:
+            return (size_or_used == HRSTORE_SIZE ? size : (size - free));
+    } /* switch */
 
 }
 #endif
