@@ -149,12 +149,6 @@
 #if HAVE_IOCTLS_H
 #include <ioctls.h>
 #endif
-#if HAVE_LINUX_ETHTOOLS_H
-#   if HAVE_PCI_PCI_H
-#      include <pci/pci.h>
-#   endif
-#include <linux/ethtools.h>
-#endif
 
 #if HAVE_DMALLOC_H
 #include <dmalloc.h>
@@ -205,6 +199,7 @@
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include <net-snmp/agent/auto_nlist.h>
+#include <net-snmp/data_access/interface.h>
 
 #include "interfaces.h"
 #include "struct.h"
@@ -219,11 +214,9 @@ static unsigned long LastLoad = 0;        /* ET in secs at last table load */
 
 extern struct timeval starttime;
 
-static void     parse_interface_config(const char *, char *);
-static void     free_interface_config(void);
-
 struct variable3 interfaces_variables[] = {
     {IFNUMBER, ASN_INTEGER, RONLY, var_interfaces, 1, {1}},
+#if !defined(NETSNMP_ENABLE_MFD_REWRITES)
     {IFINDEX, ASN_INTEGER, RONLY, var_ifEntry, 3, {2, 1, 1}},
     {IFDESCR, ASN_OCTET_STR, RONLY, var_ifEntry, 3, {2, 1, 2}},
     {IFTYPE, ASN_INTEGER, RONLY, var_ifEntry, 3, {2, 1, 3}},
@@ -250,6 +243,7 @@ struct variable3 interfaces_variables[] = {
     {IFOUTERRORS, ASN_COUNTER, RONLY, var_ifEntry, 3, {2, 1, 20}},
     {IFOUTQLEN, ASN_GAUGE, RONLY, var_ifEntry, 3, {2, 1, 21}},
     {IFSPECIFIC, ASN_OBJECT_ID, RONLY, var_ifEntry, 3, {2, 1, 22}}
+#endif /* !def NETSNMP_ENABLE_MFD_REWRITES */
 };
 
 /*
@@ -270,10 +264,6 @@ init_interfaces(void)
     REGISTER_SYSOR_ENTRY(interfaces_module_oid,
                          "The MIB module to describe generic objects for network interface sub-layers");
 
-    snmpd_register_config_handler("interface", parse_interface_config,
-                                  free_interface_config,
-                                  "name type speed");
-
 #ifndef USE_SYSCTL_IFLIST
 #if HAVE_NET_IF_MIB_H
     init_interfaces_setup();
@@ -321,89 +311,10 @@ if_type_from_name(const char *pcch)
 #endif
 
 
-typedef struct _conf_if_list {
-    char           *name;
-    int             type;
-    u_long          speed;
-    struct _conf_if_list *next;
-} conf_if_list;
-
-static conf_if_list *conf_list = NULL;
 #ifdef linux
 static struct ifnet *ifnetaddr_list;
 #endif
 
-static void
-parse_interface_config(const char *token, char *cptr)
-{
-    conf_if_list   *if_ptr, *if_new;
-    char           *name, *type, *speed, *ecp;
-    char           *st;
-
-    name = strtok_r(cptr, " \t", &st);
-    if (!name) {
-        config_perror("Missing NAME parameter");
-        return;
-    }
-    type = strtok_r(NULL, " \t", &st);
-    if (!type) {
-        config_perror("Missing TYPE parameter");
-        return;
-    }
-    speed = strtok_r(NULL, " \t", &st);
-    if (!speed) {
-        config_perror("Missing SPEED parameter");
-        return;
-    }
-    if_ptr = conf_list;
-    while (if_ptr)
-        if (strcmp(if_ptr->name, name))
-            if_ptr = if_ptr->next;
-        else
-            break;
-    if (if_ptr)
-        config_pwarn("Duplicate interface specification");
-    if_new = (conf_if_list *) malloc(sizeof(conf_if_list));
-    if (!if_new) {
-        config_perror("Out of memory");
-        return;
-    }
-    if_new->speed = strtoul(speed, &ecp, 0);
-    if (*ecp) {
-        config_perror("Bad SPEED value");
-        free(if_new);
-        return;
-    }
-    if_new->type = strtol(type, &ecp, 0);
-    if (*ecp || if_new->type < 0) {
-        config_perror("Bad TYPE");
-        free(if_new);
-        return;
-    }
-    if_new->name = strdup(name);
-    if (!if_new->name) {
-        config_perror("Out of memory");
-        free(if_new);
-        return;
-    }
-    if_new->next = conf_list;
-    conf_list = if_new;
-}
-
-static void
-free_interface_config(void)
-{
-    conf_if_list   *if_ptr = conf_list, *if_next;
-    while (if_ptr) {
-        if_next = if_ptr->next;
-        free(if_ptr->name);
-        free(if_ptr);
-        if_ptr = if_next;
-    }
-    conf_list = NULL;
-}
-
-
 
 /*
  * header_ifEntry(...
@@ -654,7 +565,7 @@ var_ifEntry(struct variable *vp,
     int             interface;
     struct if_msghdr if_msg;
     static char     if_name[100];
-    conf_if_list   *if_ptr = conf_list;
+    conf_if_list   *if_ptr;
     char           *cp;
 
     interface =
@@ -664,8 +575,7 @@ var_ifEntry(struct variable *vp,
 
     if (Interface_Scan_By_Index(interface, &if_msg, if_name, NULL) != 0)
         return NULL;
-    while (if_ptr && strcmp(if_name, if_ptr->name))
-        if_ptr = if_ptr->next;
+    if_ptr = netsnmp_access_interface_entry_overrides_get(if_name);
 
     switch (vp->magic) {
     case IFINDEX:
@@ -829,7 +739,7 @@ var_ifEntry(struct variable *vp,
     static struct in_ifaddr in_ifaddr;
     static char     Name[16];
     char           *cp;
-    conf_if_list   *if_ptr = conf_list;
+    conf_if_list   *if_ptr;
 #if STRUCT_IFNET_HAS_IF_LASTCHANGE_TV_SEC
     struct timeval  now;
 #endif
@@ -840,8 +750,7 @@ var_ifEntry(struct variable *vp,
         return NULL;
 
     Interface_Scan_By_Index(interface, Name, &ifnet, &in_ifaddr);
-    while (if_ptr && strcmp(Name, if_ptr->name))
-        if_ptr = if_ptr->next;
+    if_ptr = netsnmp_access_interface_entry_overrides_get(Name);
 
     switch (vp->magic) {
     case IFINDEX:
@@ -895,11 +804,12 @@ var_ifEntry(struct variable *vp,
         return (u_char *) & long_return;
     case IFPHYSADDRESS:
         Interface_Get_Ether_By_Index(interface, return_buf);
-        *var_len = 6;
         if ((return_buf[0] == 0) && (return_buf[1] == 0) &&
             (return_buf[2] == 0) && (return_buf[3] == 0) &&
             (return_buf[4] == 0) && (return_buf[5] == 0))
             *var_len = 0;
+        else
+            *var_len = 6;
         return (u_char *) return_buf;
     case IFADMINSTATUS:
         long_return = ifnet.if_flags & IFF_UP ? 1 : 2;
@@ -1067,7 +977,7 @@ var_ifEntry(struct variable *vp,
     int             hp_fd;
     int             hp_len = sizeof(hp_ifEntry);
 #endif
-    conf_if_list   *if_ptr = conf_list;;
+    conf_if_list   *if_ptr;
 
     interface =
         header_ifEntry(vp, name, length, exact, var_len, write_method);
@@ -1098,8 +1008,7 @@ var_ifEntry(struct variable *vp,
         }
     }
 #endif
-    while (if_ptr && strcmp(Name, if_ptr->name))
-        if_ptr = if_ptr->next;
+    if_ptr = netsnmp_access_interface_entry_overrides_get(Name);
 
     switch (vp->magic) {
     case IFINDEX:
@@ -1158,11 +1067,12 @@ var_ifEntry(struct variable *vp,
         return (u_char *) ifnet.if_entry.ifPhysAddress.o_bytes;
 #else
         Interface_Get_Ether_By_Index(interface, return_buf);
-        *var_len = 6;
         if ((return_buf[0] == 0) && (return_buf[1] == 0) &&
             (return_buf[2] == 0) && (return_buf[3] == 0) &&
             (return_buf[4] == 0) && (return_buf[5] == 0))
             *var_len = 0;
+        else
+            *var_len = 6;
         return (u_char *) return_buf;
 #endif
     case IFADMINSTATUS:
@@ -1331,7 +1241,7 @@ var_ifEntry(struct variable * vp,
 {
     int             interface;
     mib2_ifEntry_t  ifstat;
-    conf_if_list   *if_ptr = conf_list;
+    conf_if_list   *if_ptr = NULL;
 
     interface =
         header_ifEntry(vp, name, length, exact, var_len, write_method);
@@ -1344,13 +1254,10 @@ var_ifEntry(struct variable * vp,
         return NULL;
     }
     /*
-     * where to get name to check overrides?
+     * hmmm.. where to get the interface name to check overrides?
      *
-     * while (if_ptr && strcmp(Name, if_ptr->name))
-     *    if_ptr = if_ptr->next;
+     * if_ptr = netsnmp_access_interface_entry_overrides_get(Name);
      */
-    if_ptr = NULL; /* XXX until we have name to check overrides */
-
     switch (vp->magic) {
     case IFINDEX:
         long_return = ifstat.ifIndex;
@@ -1455,137 +1362,22 @@ static char     saveName[16];
 #endif
 static int      saveIndex = 0;
 
-#ifdef linux
-static unsigned int _mii_if_speed(int fd, struct ifreq ifr){
-	unsigned int retspeed = 10000000;
-/* the code is based on mii-diag utility by Donald Becker
-* see ftp://ftp.scyld.com/pub/diag/mii-diag.c
-*/
-	ushort *data = (ushort *)(&ifr.ifr_data);
-	unsigned phy_id;
-	unsigned char new_ioctl_nums = 0;
-	int mii_reg, i;
-	ushort mii_val[32];
-	ushort bmcr, bmsr, nway_advert, lkpar;
-	const unsigned int media_speeds[] = {10000000, 10000000, 100000000, 100000000, 10000000, 0};	
-/* It corresponds to "10baseT", "10baseT-FD", "100baseTx", "100baseTx-FD", "100baseT4", "Flow-control", 0, */
-
-
-	data[0] = 0;
-
-	if (ioctl(fd, 0x8947, &ifr) >= 0) {
-		new_ioctl_nums = 1;
-	} else if (ioctl(fd, SIOCDEVPRIVATE, &ifr) >= 0) {
-		new_ioctl_nums = 0;
-	} else {
-		DEBUGMSGTL(("mibII/interfaces", "SIOCGMIIPHY on %s failed\n", ifr.ifr_name));
-		return retspeed;
-	}
-/* Begin getting mii register values */
-	phy_id = data[0];
-	for (mii_reg = 0; mii_reg < 8; mii_reg++){
-		data[0] = phy_id;
-		data[1] = mii_reg;
-		if(ioctl(fd, new_ioctl_nums ? 0x8948 : SIOCDEVPRIVATE+1, &ifr) <0){
-			DEBUGMSGTL(("mibII/interfaces", "SIOCGMIIREG on %s failed\n", ifr.ifr_name));
-		}
-		mii_val[mii_reg] = data[3];		
-	}
-/*Parsing of mii values*/
-/*Invalid basic mode control register*/
-	if (mii_val[0] == 0xffff  ||  mii_val[1] == 0x0000) {
-		DEBUGMSGTL(("mibII/interfaces", "No MII transceiver present!.\n"));
-		return retspeed;
-	}
-	/* Descriptive rename. */
-	bmcr = mii_val[0]; 	  /*basic mode control register*/
-	bmsr = mii_val[1]; 	  /* basic mode status register*/
-	nway_advert = mii_val[4]; /* autonegotiation advertisement*/
-	lkpar = mii_val[5]; 	  /*link partner ability*/
-	
-/*Check for link existence, returns 0 if link is absent*/
-	if ((bmsr & 0x0016) != 0x0004){
-		DEBUGMSGTL(("mibII/interfaces", "No link...\n"));
-		retspeed = 0;
-		return retspeed;
-	}
-	
-	if(!(bmcr & 0x1000) ){
-		DEBUGMSGTL(("mibII/interfaces", "Auto-negotiation disabled.\n"));
-		retspeed = bmcr & 0x2000 ? 100000000 : 10000000;
-		return retspeed;
-	}
-/* Link partner got our advertised abilities */	
-	if (lkpar & 0x4000) {
-		int negotiated = nway_advert & lkpar & 0x3e0;
-		int max_capability = 0;
-		/* Scan for the highest negotiated capability, highest priority
-		   (100baseTx-FDX) to lowest (10baseT-HDX). */
-		int media_priority[] = {8, 9, 7, 6, 5}; 	/* media_names[i-5] */
-		for (i = 0; media_priority[i]; i++){
-			if (negotiated & (1 << media_priority[i])) {
-				max_capability = media_priority[i];
-				break;
-			}
-		}
-		if (max_capability)
-			retspeed = media_speeds[max_capability - 5];
-		else
-			DEBUGMSGTL(("mibII/interfaces", "No common media type was autonegotiated!\n"));
-	}else if(lkpar & 0x00A0){
-		retspeed = (lkpar & 0x0080) ? 100000000 : 10000000;
-	}
-	return retspeed;
-}
-#endif 
-
-#if HAVE_LINUX_ETHTOOLS_H
-static u_int
-_ethtools_if_speed(int fd, struct ifreq ifr)
-{
-    struct ethtool_cmd edata;
-
-    edata.cmd = ETHTOOL_GSET;
-    ifr.ifr_data = (char *) &edata;
-    
-    if (ioctl(fd, SIOCETHTOOL, &ifr) == -1) {
-        DEBUGMSGTL(("mibII/interfaces", "ETHTOOL_GSET on %s failed\n",
-                    ifr.ifr_name));
-        return _mii_if_speed(fd,ifr);
-    }
-    
-    if (edata.speed != SPEED_10 && edata.speed != SPEED_100 &&
-        edata.speed != SPEED_1000) {
-        DEBUGMSGTL(("mibII/interfaces", "fallback to mii for %s\n",
-                    ifr.ifr_name));
-        /* try MII */
-        return _mii_get_if_speed(fd,ifr);
-    }
-
-    /* return in bps */
-    DEBUGMSGTL(("mibII/interfaces", "ETHTOOL_GSET on %s speed = %d\n",
-                ifr.ifr_name, edata.speed));
-    return edata.speed*1000*1000;
-}
-#endif /* HAVE_LINUX_ETHTOOLS_H */
-
 /**
 * Determines network interface speed. It is system specific. Only linux
 * realization is made. 
 */
-unsigned int getIfSpeed(int fd, struct ifreq ifr){
-	unsigned int retspeed = 10000000;
+unsigned int getIfSpeed(int fd, struct ifreq ifr)
+{
 #ifdef linux
-#   if HAVE_LINUX_ETHTOOLS_H
-   return _ethtool_if_speed(fd, ifr);
-#   else
-   return _mii_if_speed(fd, ifr);
-#   endif /* HAVE_LINUX_ETHTOOLS_H */
-#else /*!linux*/			   
-   return retspeed;
-#endif
-}
+    /** temporary expose internal until this module can be re-written */
+    extern unsigned int
+        netsnmp_arch_interface_get_if_speed(int fd, const char *name);
 
+    return netsnmp_arch_interface_get_if_speed(fd, ifr.ifr_name);
+#else /*!linux*/			   
+    return 10000000;
+#endif 
+}
 
 void
 Interface_Scan_Init(void)
@@ -1597,6 +1389,13 @@ Interface_Scan_Init(void)
     FILE           *devin;
     int             i, fd;
     conf_if_list   *if_ptr;
+    /*
+     * scanline_2_2:
+     *  [               IN                        ]
+     *   byte pkts errs drop fifo frame cmprs mcst |
+     *  [               OUT                               ]
+     *   byte pkts errs drop fifo colls carrier compressed
+     */
 #ifdef SCNuMAX
     uintmax_t       rec_pkt, rec_oct, rec_err, rec_drop;
     uintmax_t       snd_pkt, snd_oct, snd_err, snd_drop, coll;
@@ -1751,19 +1550,19 @@ Interface_Scan_Init(void)
         if (!strcmp(ifname_buf, "lo") && rec_pkt > 0 && !snd_pkt)
             snd_pkt = rec_pkt;
 
-        nnew->if_ipackets = rec_pkt & 0xffffffff;
+        nnew->if_ipackets = rec_pkt;
         nnew->if_ierrors = rec_err;
-        nnew->if_opackets = snd_pkt & 0xffffffff;
+        nnew->if_opackets = snd_pkt;
         nnew->if_oerrors = snd_err;
         nnew->if_collisions = coll;
         if (scan_line_to_use == scan_line_2_2) {
-            nnew->if_ibytes = rec_oct & 0xffffffff;
-            nnew->if_obytes = snd_oct & 0xffffffff;
+            nnew->if_ibytes = rec_oct;
+            nnew->if_obytes = snd_oct;
             nnew->if_iqdrops = rec_drop;
             nnew->if_snd.ifq_drops = snd_drop;
         } else {
-            nnew->if_ibytes = (rec_pkt * 308) & 0xffffffff;
-            nnew->if_obytes = (snd_pkt * 308) & 0xffffffff;
+            nnew->if_ibytes = rec_pkt * 308;
+            nnew->if_obytes = snd_pkt * 308;
         }
 
         /*
@@ -1810,12 +1609,18 @@ Interface_Scan_Init(void)
 
         nnew->if_type = 0;
 
+        /*
+         * NOTE: this ioctl does not guarantee 6 bytes of a physaddr.
+         * In particular, a 'sit0' interface only appears to get back
+         * 4 bytes of sa_data.
+         */
+        memset(ifrq.ifr_hwaddr.sa_data, (0), IFHWADDRLEN);
         strncpy(ifrq.ifr_name, ifname, sizeof(ifrq.ifr_name));
         ifrq.ifr_name[ sizeof(ifrq.ifr_name)-1 ] = 0;
         if (ioctl(fd, SIOCGIFHWADDR, &ifrq) < 0)
-            memset(nnew->if_hwaddr, (0), 6);
+            memset(nnew->if_hwaddr, (0), IFHWADDRLEN);
         else {
-            memcpy(nnew->if_hwaddr, ifrq.ifr_hwaddr.sa_data, 6);
+            memcpy(nnew->if_hwaddr, ifrq.ifr_hwaddr.sa_data, IFHWADDRLEN);
 
 #ifdef ARPHRD_LOOPBACK
             switch (ifrq.ifr_hwaddr.sa_family) {
@@ -1882,10 +1687,7 @@ Interface_Scan_Init(void)
         nnew->if_mtu = 0;
 #endif
 
-        for (if_ptr = conf_list; if_ptr; if_ptr = if_ptr->next)
-            if (!strcmp(if_ptr->name, ifname))
-                break;
-
+        if_ptr = netsnmp_access_interface_entry_overrides_get(ifname);
         if (if_ptr) {
             nnew->if_type = if_ptr->type;
             nnew->if_speed = if_ptr->speed;
@@ -2084,10 +1886,9 @@ Interface_Scan_Next(short *Index,
              *  Try to find an address for this interface
              */
 
+            auto_nlist(IFADDR_SYMBOL, (char *) &ia, sizeof(ia));
 #ifdef netbsd1
             ia = (struct in_ifaddr *) ifnet.if_addrlist.tqh_first;
-#else
-            auto_nlist(IFADDR_SYMBOL, (char *) &ia, sizeof(ia));
 #endif
             while (ia) {
                 klookup((unsigned long) ia, (char *) &in_ifaddr,
@@ -2603,7 +2404,7 @@ var_ifEntry(struct variable * vp,
     static struct ifmibdata ifmd;
     size_t          len;
     char           *cp;
-    conf_if_list   *if_ptr = conf_list;;
+    conf_if_list   *if_ptr = NULL;
 
     interface = header_ifEntry(vp, name, length, exact, var_len,
                                write_method);
@@ -2615,12 +2416,10 @@ var_ifEntry(struct variable * vp,
     if (sysctl(sname, 6, &ifmd, &len, 0, 0) < 0)
         return NULL;
     /*
-     * where to get name to check overrides?
+     * hmmm.. where to get the interface name to check overrides?
      *
-     * while (if_ptr && strcmp(Name, if_ptr->name))
-     *     if_ptr = if_ptr->next;
+     * if_ptr = netsnmp_access_interface_entry_overrides_get(Name);
      */
-    if_ptr = NULL; /* XXX until we find name to check overrides */
 
     switch (vp->magic) {
     case IFINDEX:
@@ -2836,12 +2635,17 @@ var_ifEntry(struct variable * vp,
 {
     int             ifIndex;
     static MIB_IFROW ifRow;
-    conf_if_list   *if_ptr = conf_list;
+    conf_if_list   *if_ptr;
     
     ifIndex =
         header_ifEntry(vp, name, length, exact, var_len, write_method);
     if (ifIndex == MATCH_FAILED)
         return NULL;
+    /*
+     * hmmm.. where to get the interface name to check overrides?
+     *
+     * if_ptr = netsnmp_access_interface_entry_overrides_get(Name);
+     */
 
     /*
      * Get the If Table Row by passing index as argument 
@@ -2849,14 +2653,6 @@ var_ifEntry(struct variable * vp,
     ifRow.dwIndex = ifIndex;
     if (GetIfEntry(&ifRow) != NO_ERROR)
         return NULL;
-    /*
-     * where to get name to check overrides?
-     *
-     * while (if_ptr && strcmp(Name, if_ptr->name))
-     *     if_ptr = if_ptr->next;
-     */
-    if_ptr = NULL; /* XXX until we find name to check overrides */
-
     switch (vp->magic) {
     case IFINDEX:
         long_return = ifIndex;
