@@ -114,6 +114,7 @@
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include <net-snmp/agent/auto_nlist.h>
+#include <net-snmp/data_access/interface.h>
 
 #include "ip.h"
 #include "interfaces.h"
@@ -199,9 +200,6 @@ var_ipAddrEntry(struct variable *vp,
     static struct in_ifaddr in_ifaddr, lowin_ifaddr;
 #else
     static struct ifnet lowin_ifnet;
-#if defined(linux)
-    static struct in_ifaddr in_ifaddr;
-#endif
 #endif
     static struct ifnet ifnet;
 #endif                          /* hpux11 */
@@ -213,7 +211,7 @@ var_ipAddrEntry(struct variable *vp,
     memcpy((char *) current, (char *) vp->name,
            (int) vp->namelen * sizeof(oid));
 
-#if !defined(freebsd2) && !defined(hpux11)
+#if !defined(freebsd2) && !defined(hpux11) && !defined(linux)
     Interface_Scan_Init();
 #else
     Address_Scan_Init();
@@ -229,7 +227,7 @@ var_ipAddrEntry(struct variable *vp,
 #endif
 #else                           /* !freebsd2 && !hpux11 */
 #if defined(linux)
-        if (Interface_Scan_Next(&interface, NULL, &ifnet, &in_ifaddr) == 0)
+        if (Address_Scan_Next(&interface, &ifnet) == 0)
             break;
 #else
         if (Address_Scan_Next(&interface, &in_ifaddr) == 0)
@@ -242,9 +240,6 @@ var_ipAddrEntry(struct variable *vp,
 #elif defined(linux) || defined(sunV3)
         cp = (u_char *) & (((struct sockaddr_in *) &(ifnet.if_addr))->
                            sin_addr.s_addr);
-
-        if (*cp == 0)		/* first octet is zero? 0.x.x.x is not a */
-            continue;		/* legal address for an interface */
 #else
         cp = (u_char *) & (((struct sockaddr_in *) &(in_ifaddr.ia_addr))->
                            sin_addr.s_addr);
@@ -371,7 +366,9 @@ Address_Scan_Init(void)
  * NB: Index is the number of the corresponding interface, not of the address 
  */
 static int
-Address_Scan_Next(short *Index, struct in_ifaddr *Retin_ifaddr)
+Address_Scan_Next(Index, Retin_ifaddr)
+     short          *Index;
+     struct in_ifaddr *Retin_ifaddr;
 {
     struct in_ifaddr in_ifaddr;
     struct ifnet    ifnet, *ifnetaddr;  /* NOTA: same name as another one */
@@ -460,7 +457,9 @@ Address_Scan_Init(void)
  * NB: Index is the number of the corresponding interface, not of the address 
  */
 static int
-Address_Scan_Next(short *Index, mib_ipAdEnt *Retin_ifaddr)
+Address_Scan_Next(Index, Retin_ifaddr)
+     short          *Index;
+     mib_ipAdEnt    *Retin_ifaddr;
 {
     if (iptab_current < iptab_size) {
         /*
@@ -530,7 +529,10 @@ Address_Scan_Init(void)
  * NB: Index is the number of the corresponding interface, not of the address 
  */
 static int
-Address_Scan_Next(short *Index, struct ifnet *Retifnet)
+Address_Scan_Next(Index, Retifnet)
+    short          *Index;
+    struct ifnet   *Retifnet;
+
 {
     struct ifnet   ifnet_store;
     int fd;
@@ -568,12 +570,7 @@ Address_Scan_Next(short *Index, struct ifnet *Retifnet)
         if (Index)
 	{
 	    ifr->ifr_addr = ifnet_store.if_addr;
-	    if (ioctl(fd, SIOCGIFINDEX, ifr) < 0)
-	    {
-	        *Index = -1;
-	    }
-	    else
-        	*Index = ifr->ifr_ifindex;
+            *Index = netsnmp_access_interface_index_find(ifr->ifr_name);
 	}
 	
 	ifr++;
@@ -625,6 +622,7 @@ var_ipAddrEntry(struct variable * vp,
     static mib2_ipAddrEntry_t Lowentry;
     int             Found = 0;
     req_e           req_type;
+    unsigned int    ipaddr_return;
 
     /*
      * fill in object part of name for current (less sizeof instance part) 
@@ -691,16 +689,18 @@ var_ipAddrEntry(struct variable * vp,
     *var_len = sizeof(long_return);
     switch (vp->magic) {
     case IPADADDR:
-	*var_len = sizeof(Lowentry.ipAdEntAddr);
-        return (u_char *) &Lowentry.ipAdEntAddr;
+	*var_len = sizeof(uint32_t);
+        ipaddr_return = Lowentry.ipAdEntAddr;
+        return (u_char *) & ipaddr_return;
     case IPADIFINDEX:
         long_return =
             Interface_Index_By_Name(Lowentry.ipAdEntIfIndex.o_bytes,
                                     Lowentry.ipAdEntIfIndex.o_length);
         return (u_char *) & long_return;
     case IPADNETMASK:
-	*var_len = sizeof(Lowentry.ipAdEntNetMask);
-        return (u_char *) &Lowentry.ipAdEntNetMask;
+	*var_len = sizeof(uint32_t);
+        ipaddr_return = Lowentry.ipAdEntNetMask;
+        return (u_char *) & ipaddr_return;
     case IPADBCASTADDR:
 	*var_len = sizeof(Lowentry.ipAdEntBcastAddr);
         return (u_char *)&Lowentry.ipAdEntBcastAddr;
