@@ -7,6 +7,13 @@
 #include <net-snmp/net-snmp-config.h>
 #include "mibII_common.h"
 
+#if HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #if HAVE_SYS_PROTOSW_H
 #include <sys/protosw.h>
 #endif
@@ -193,7 +200,8 @@ tcp_handler(netsnmp_mib_handler          *handler,
      * But just to be safe, check this and load it manually if necessary
      */
 #ifndef hpux11
-    if (!netsnmp_is_cache_valid(reqinfo)) {
+    if (!netsnmp_cache_is_valid(reqinfo, reginfo->handlerName)) {
+        netsnmp_assert("cache" == "valid"); /* always false */
         tcp_load( NULL, NULL );	/* XXX - check for failure */
     }
 #endif
