@@ -224,6 +224,7 @@ extensible_parse_config(const char *token, char *cptr)
         /*
          * its a relocatable extensible mib 
          */
+        config_perror("Warning: relocatable 'exec' format will change in a future release - See 'NET-SNMP-EXTEND-MIB' for an alternative" );
         for (pp = &relocs, numrelocs++; *pp; pp = &((*pp)->next));
         (*pp) = ptmp;
         pp = &relocs; scount = numrelocs;
@@ -358,18 +359,6 @@ extensible_free_config(void)
 }
 
 
-struct extensible *
-get_exten_instance(struct extensible *exten, size_t inst)
-{
-    int             i;
-
-    if (exten == NULL)
-        return (NULL);
-    for (i = 1; i != (int) inst && exten != NULL; i++)
-        exten = exten->next;
-    return (exten);
-}
-
 #define MAXMSGLINES 1000
 
 struct extensible *extens = NULL;       /* In exec.c */
