@@ -1,3 +1,13 @@
+/* Portions of this file are subject to the following copyright(s).  See
+ * the Net-SNMP's COPYING file for more details and other copyrights
+ * that may apply:
+ */
+/*
+ * Portions of this file are copyrighted by:
+ * Copyright @ 2003 Sun Microsystems, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
+ */
 #include <net-snmp/net-snmp-config.h>
 
 #include <sys/types.h>
@@ -42,6 +52,10 @@ proxyOptProc(int argc, char *const *argv, int opt)
                     config_perror("No context name passed to -Cn");
                 }
                 break;
+            case 'c':
+                netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID,
+                                       NETSNMP_DS_LIB_IGNORE_NO_COMMUNITY, 1);
+                break;
             default:
                 config_perror("unknown argument passed to -C");
                 break;
@@ -90,7 +104,13 @@ proxy_parse_config(const char *token, char *line)
     }
 
     DEBUGMSGTL(("proxy_config", "parsing args: %d\n", argn));
+    /* Call special parse_args that allows for no specified community string */
     arg = snmp_parse_args(argn, argv, &session, "C:", proxyOptProc);
+
+    /* reset this in case we modified it */
+    netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID,
+                           NETSNMP_DS_LIB_IGNORE_NO_COMMUNITY, 0);
+    
     if (arg < 0) {
         config_perror("failed to parse proxy args");
         return;
@@ -216,6 +236,83 @@ proxy_free_config(void)
     }
 }
 
+/*
+ * Configure special parameters on the session.
+ * Currently takes the parameter configured and changes it if something 
+ * was configured.  It becomes "-c" if the community string from the pdu
+ * is placed on the session.
+ */
+int
+proxy_fill_in_session(netsnmp_mib_handler *handler,
+                      netsnmp_agent_request_info *reqinfo,
+                      void **configured)
+{
+    netsnmp_session *session;
+    struct simple_proxy *sp;
+
+    sp = (struct simple_proxy *) handler->myvoid;
+    if (!sp) {
+        return 0;
+    }
+    session = sp->sess;
+    if (!session) {
+        return 0;
+    }
+
+#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+    if (session->version == SNMP_VERSION_1 ||
+        session->version == SNMP_VERSION_2c) {
+
+        /*
+         * Check if session has community string defined for it.
+         * If not, need to extract community string from the pdu.
+         * Copy to session and set 'configured' to indicate this.
+         */
+        if (session->community_len == 0) {
+            DEBUGMSGTL(("proxy", "session has no community string\n"));
+            if (reqinfo->asp == NULL || reqinfo->asp->pdu == NULL ||
+                reqinfo->asp->pdu->community_len == 0) {
+                return 0;
+            }
+
+            *configured = malloc(strlen("-c") + 1);
+            strcpy(*configured, "-c");
+            DEBUGMSGTL(("proxy", "pdu has community string\n"));
+            session->community_len = reqinfo->asp->pdu->community_len;
+            session->community = malloc(session->community_len + 1);
+            strncpy((char *)session->community,
+                    (const char *)reqinfo->asp->pdu->community,
+                    session->community_len);
+        }
+    }
+#endif
+
+    return 1;
+}
+
+/*
+ * Free any specially configured parameters used on the session.
+ */
+void
+proxy_free_filled_in_session_args(netsnmp_session *session, void **configured)
+{
+
+    /* Only do comparisions, etc., if something was configured */
+    if (*configured == NULL) {
+        return;
+    }
+
+    /* If used community string from pdu, release it from session now */
+    if (strcmp((const char *)(*configured), "-c") == 0) {
+        free(session->community);
+        session->community = NULL;
+        session->community_len = 0;
+    }
+
+    free((u_char *)(*configured));
+    *configured = NULL;
+}
+
 void
 init_proxy(void)
 {
@@ -242,6 +339,7 @@ proxy_handler(netsnmp_mib_handler *handler,
     oid            *ourname;
     size_t          ourlength;
     netsnmp_request_info *request = requests;
+    u_char         *configured = NULL;
 
     DEBUGMSGTL(("proxy", "proxy handler starting, mode = %d\n",
                 reqinfo->mode));
@@ -327,6 +425,14 @@ proxy_handler(netsnmp_mib_handler *handler,
         request = request->next;
     }
 
+    /*
+     * Customize session parameters based on request information
+     */
+    if (!proxy_fill_in_session(handler, reqinfo, (void **)&configured)) {
+        netsnmp_set_request_error(reqinfo, requests, SNMP_ERR_GENERR);
+        return SNMP_ERR_NOERROR;
+    }
+
     /*
      * send the request out 
      */
@@ -335,6 +441,10 @@ proxy_handler(netsnmp_mib_handler *handler,
                     netsnmp_create_delegated_cache(handler, reginfo,
                                                    reqinfo, requests,
                                                    (void *) sp));
+
+    /* Free any special parameters generated on the session */
+    proxy_free_filled_in_session_args(sp->sess, (void **)&configured);
+
     return SNMP_ERR_NOERROR;
 }
 
@@ -389,24 +499,9 @@ proxy_got_response(int operation, netsnmp_session * sess, int reqid,
             /*
              *  If we receive an error from the proxy agent, pass it on up.
              *  The higher-level processing seems to Do The Right Thing.
-             *
-             * 2005/06 rks: actually, it doesn't do the right thing for
-             * a get-next request that returns NOSUCHNAME. If we do nothing,
-             * it passes that error back to the comman initiator. What it should
-             * do is ignore the error and move on to the next tree. To
-             * accomplish that, all we need to do is clear the delegated flag.
-             * Not sure if any other error codes need the same treatment. Left
-             * as an exercise to the reader...
              */
             DEBUGMSGTL(("proxy", "got error response (%d)\n", pdu->errstat));
-            if((cache->reqinfo->mode == MODE_GETNEXT) &&
-               (SNMP_ERR_NOSUCHNAME == pdu->errstat)) {
-                DEBUGMSGTL(("proxy", "  ignoring error response\n"));
-                for(request = requests; request; request = request->next)
-                    request->delegated = 0;
-            }
-            else
-                netsnmp_set_request_error(cache->reqinfo, requests, pdu->errstat);
+            netsnmp_set_request_error(cache->reqinfo, requests, pdu->errstat);
 
         /*
          * update the original request varbinds with the results 
