@@ -203,8 +203,6 @@ getstat(unsigned long *cuse, unsigned long *cice, unsigned long *csys,
     static int      bsize = 0, vmbsize = 0;
     char           *b;
     time_t          now;
-    unsigned long long cusell = 0, cicell = 0, csysll = 0, cidell = 0,
-        ciowll = 0, cirqll = 0, csoftll = 0;
 
     time(&now);
     if (cache_time + CACHE_TIMEOUT < now) {
@@ -243,25 +241,16 @@ getstat(unsigned long *cuse, unsigned long *cice, unsigned long *csys,
     }
 
     *itot = 0;
-    *i1 = 1;
+    *i1 = 1;                /* ensure assert below will fail if the sscanf bombs */
     b = strstr(buff, "cpu ");
     if (b) {
 	if (!has_cpu_26 ||
-            sscanf(b, "cpu  %llu %llu %llu %llu %llu %llu %llu", &cusell,
-                   &cicell, &csysll, &cidell, &ciowll, &cirqll, &csoftll) != 7) {
+		sscanf(b, "cpu  %lu %lu %lu %lu %lu %lu %lu", cuse, cice, csys,
+		                        cide, ciow, cirq, csoft) != 7) {
 	    has_cpu_26 = 0;
-	    sscanf(b, "cpu  %llu %llu %llu %llu", &cusell, &cicell, &csysll,
-                   &cidell);
+	    sscanf(b, "cpu  %lu %lu %lu %lu", cuse, cice, csys, cide);
 	    *ciow = *cirq = *csoft = 0;
-	} else {
-  	    *ciow = (unsigned long)ciowll;
-  	    *cirq = (unsigned long)cirqll;
-  	    *csoft = (unsigned long)csoftll;
- 	}
-	*cuse = (unsigned long)cusell;
-	*cice = (unsigned long)cicell;
-	*csys = (unsigned long)csysll;
-	*cide = (unsigned long)cidell;
+	}
     }
     else {
 	if (first)
@@ -414,9 +403,9 @@ vmstat(int iindex)
     case cpurawsoft:
 	return cpu_softirq;
     case rawiosent:
-	return pgpgout*2;
-    case rawioreceive:
 	return pgpgin*2;
+    case rawioreceive:
+	return pgpgout*2;
     case rawswapin:
 	return pswpin;
     case rawswapout:
