@@ -93,6 +93,9 @@ PERFORMANCE OF THIS SOFTWARE.
 # include <sys/socket.h>
 #endif
 #if HAVE_SYS_STREAM_H
+#   ifdef sysv5UnixWare7
+#      define _KMEMUSER 1 /* <sys/stream.h> needs this for queue_t */
+#   endif
 #include <sys/stream.h>
 #endif
 #if HAVE_SYS_SOCKETVAR_H
@@ -139,11 +142,13 @@ PERFORMANCE OF THIS SOFTWARE.
 
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
+#include <net-snmp/agent/mib_modules.h>
 #include "kernel.h"
 
 #include "mibgroup/struct.h"
 #include "snmpd.h"
 #include "net-snmp/agent/all_helpers.h"
+#include "agent_module_includes.h"
 #include "mib_module_includes.h"
 #include "net-snmp/library/container.h"
 
@@ -151,6 +156,8 @@ PERFORMANCE OF THIS SOFTWARE.
 #define  MIN(a,b)                     (((a) < (b)) ? (a) : (b))
 #endif
 
+static char     done_init_agent = 0;
+
 /*
  * mib clients are passed a pointer to a oid buffer.  Some mib clients
  * * (namely, those first noticed in mibII/vacm.c) modify this oid buffer
@@ -217,7 +224,6 @@ extern netsnmp_subtree *subtrees;
  */
 
 long            long_return;
-uint32_t	ipaddr_return;
 #ifndef ibm032
 u_char          return_buf[258];
 #else
@@ -264,6 +270,12 @@ init_agent(const char *app)
 {
     int             r = 0;
 
+    if(++done_init_agent > 1) {
+        snmp_log(LOG_WARNING, "ignoring extra call to init_agent (%d)\n", 
+                 done_init_agent);
+        return r;
+    }
+
     /*
      * get current time (ie, the time the agent started) 
      */
@@ -323,6 +335,8 @@ init_agent(const char *app)
     init_perl();
 #endif
 
+#  include "agent_module_inits.h"
+
     return r;
 }                               /* end init_agent() */
 
@@ -343,5 +357,7 @@ shutdown_agent(void) {
     clear_snmp_enum();
     clear_callback();
     clear_user_list();
+
+    done_init_agent = 0;
 }
 
