@@ -191,7 +191,7 @@ int             Facility = LOG_DAEMON;
 int             agent_status = AGENT_STOPPED;
 LPTSTR          app_name = _T("Net-SNMP Agent");     /* Application Name */
 #else
-const char     *app_name = "snmpd";
+char           *app_name = "snmpd";
 #endif
 
 extern char   **argvrestartp;
@@ -395,17 +395,13 @@ SnmpTrapNodeDown(void)
      */
 }
 
-/*
- * dummy1 used to be stderr_log, but that was never set. So, ignore the
- * value here, instead of always disabling stderr logging (which might
- * have been set up using the new log handler methods).
- */
 static void
-setup_log(int restart, int dont_zero, int dummy1, int syslog_log, 
+setup_log(int restart, int dont_zero, int stderr_log, int syslog_log, 
 	  char *logfile)
 {
     static char logfile_s[PATH_MAX + 1] = { 0 };
     static int dont_zero_s  = 0;
+    static int stderr_log_s = 0;
     static int syslog_log_s = 0;
 
     if (restart == 0) {
@@ -413,9 +409,16 @@ setup_log(int restart, int dont_zero, int dummy1, int syslog_log,
 	    strncpy(logfile_s, logfile, PATH_MAX);
 	}
 	dont_zero_s  = dont_zero;
+	stderr_log_s = stderr_log;
 	syslog_log_s = syslog_log;
     }
 
+    if (stderr_log_s) {
+	snmp_enable_stderrlog();
+    } else {
+	snmp_disable_stderrlog();
+    }
+
     if (logfile_s[0]) {
 	snmp_enable_filelog(logfile_s, dont_zero_s);
     }
@@ -452,7 +455,7 @@ main(int argc, char *argv[])
     int             arg, i, ret;
     int             dont_fork = 0;
     int             dont_zero_log = 0;
-    int             syslog_log = 0;
+    int             stderr_log = 0, syslog_log = 0;
     int             uid = 0, gid = 0;
     int             agent_mode = -1;
     char            logfile[PATH_MAX + 1] = { 0 };
@@ -464,30 +467,6 @@ main(int argc, char *argv[])
     FILE           *PID;
 #endif
 
-#ifdef SIGTERM
-    DEBUGMSGTL(("signal", "registering SIGTERM signal handler\n"));
-    signal(SIGTERM, SnmpdShutDown);
-#endif
-#ifdef SIGINT
-    DEBUGMSGTL(("signal", "registering SIGINT signal handler\n"));
-    signal(SIGINT, SnmpdShutDown);
-#endif
-#ifdef SIGHUP
-    DEBUGMSGTL(("signal", "registering SIGHUP signal handler\n"));
-    signal(SIGHUP, SnmpdReconfig);
-#endif
-#ifdef SIGUSR1
-    DEBUGMSGTL(("signal", "registering SIGUSR1 signal handler\n"));
-    signal(SIGUSR1, SnmpdDump);
-#endif
-#ifdef SIGPIPE
-    DEBUGMSGTL(("signal", "registering SIGPIPE signal handler\n"));
-    signal(SIGPIPE, SIG_IGN);   /* 'Inline' failure of wayward readers */
-#endif
-#ifdef SIGXFSZ
-    signal(SIGXFSZ, SnmpdCatchRandomSignal);
-#endif
-
 #ifdef LOGFILE
     strncpy(logfile, LOGFILE, PATH_MAX);
 #endif
@@ -509,6 +488,8 @@ main(int argc, char *argv[])
     netsnmp_ds_set_int(NETSNMP_DS_APPLICATION_ID,
                        NETSNMP_DS_AGENT_AGENTX_RETRIES, -1);
 
+    netsnmp_ds_set_int(NETSNMP_DS_APPLICATION_ID,
+                       NETSNMP_DS_AGENT_CACHE_TIMEOUT, 5);
     /*
      * Add some options if they are available.  
      */
@@ -635,6 +616,9 @@ main(int argc, char *argv[])
             break;
 
         case 'L':
+	    /*
+            stderr_log = 1;
+	     */
 	    if  (snmp_log_options( optarg, argc, argv ) < 0 ) {
                 usage(argv[0]);
             }
@@ -807,7 +791,7 @@ main(int argc, char *argv[])
             char *c, *astring;
             if ((c = netsnmp_ds_get_string(NETSNMP_DS_APPLICATION_ID, 
 					   NETSNMP_DS_AGENT_PORTS))) {
-                astring = (char*)malloc(strlen(c) + 2 + strlen(argv[i]));
+                astring = malloc(strlen(c) + 2 + strlen(argv[i]));
                 if (astring == NULL) {
                     fprintf(stderr, "malloc failure processing argv[%d]\n", i);
                     exit(1);
@@ -826,7 +810,7 @@ main(int argc, char *argv[])
 					  NETSNMP_DS_AGENT_PORTS)));
     }
 
-    setup_log(0, dont_zero_log, 0, syslog_log, logfile);
+    setup_log(0, dont_zero_log, stderr_log, syslog_log, logfile);
 
     /*
      * Initialize a argv set to the current for restarting the agent.   
@@ -873,7 +857,7 @@ main(int argc, char *argv[])
     if(!dont_fork) {
         int quit = ! netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID,
                                             NETSNMP_DS_AGENT_QUIT_IMMEDIATELY);
-        ret = netsnmp_daemonize(quit, snmp_stderrlog_status());
+        ret = netsnmp_daemonize(quit, stderr_log);
         /*
          * xxx-rks: do we care if fork fails? I think we should...
          */
@@ -896,6 +880,29 @@ main(int argc, char *argv[])
          */
         Exit(1);                /*  Exit logs exit val for us  */
     }
+#ifdef SIGTERM
+    DEBUGMSGTL(("signal", "registering SIGTERM signal handler\n"));
+    signal(SIGTERM, SnmpdShutDown);
+#endif
+#ifdef SIGINT
+    DEBUGMSGTL(("signal", "registering SIGINT signal handler\n"));
+    signal(SIGINT, SnmpdShutDown);
+#endif
+#ifdef SIGHUP
+    DEBUGMSGTL(("signal", "registering SIGHUP signal handler\n"));
+    signal(SIGHUP, SnmpdReconfig);
+#endif
+#ifdef SIGUSR1
+    DEBUGMSGTL(("signal", "registering SIGUSR1 signal handler\n"));
+    signal(SIGUSR1, SnmpdDump);
+#endif
+#ifdef SIGPIPE
+    DEBUGMSGTL(("signal", "registering SIGPIPE signal handler\n"));
+    signal(SIGPIPE, SIG_IGN);   /* 'Inline' failure of wayward readers */
+#endif
+#ifdef SIGXFSZ
+    signal(SIGXFSZ, SnmpdCatchRandomSignal);
+#endif
 
     /*
      * Store persistent data immediately in case we crash later.  
@@ -1029,33 +1036,13 @@ receive(void)
     int             numfds;
     fd_set          readfds, writefds, exceptfds;
     struct timeval  timeout, *tvp = &timeout;
-    struct timeval  sched, *svp = &sched, now, *nvp = &now;
     int             count, block, i;
 #ifdef	USING_SMUX_MODULE
     int             sd;
 #endif                          /* USING_SMUX_MODULE */
 
-
     /*
-     * Set the 'sched'uled timeout to the current time + one TIMETICK.
-     */
-    gettimeofday(nvp, (struct timezone *) NULL);
-    svp->tv_usec = nvp->tv_usec + TIMETICK;
-    svp->tv_sec = nvp->tv_sec;
-
-    while (svp->tv_usec >= ONE_SEC) {
-        svp->tv_usec -= ONE_SEC;
-        svp->tv_sec++;
-    }
-
-    /*
-     * ignore early sighup during startup
-     */
-    reconfig = 0;
-
-    /*
-     * Loop-forever: execute message handlers for sockets with data,
-     * reset the 'sched'uler.
+     * Loop-forever: execute message handlers for sockets with data
      */
     while (running) {
         if (reconfig) {
@@ -1078,9 +1065,12 @@ receive(void)
             }
         }
 
+        /*
+         * default to sleeping for a really long time
+         */
         tvp = &timeout;
-        tvp->tv_sec = 0;
-        tvp->tv_usec = TIMETICK;
+        tvp->tv_sec = LONG_MAX;
+        tvp->tv_usec = 0;
 
         numfds = 0;
         FD_ZERO(&readfds);
@@ -1123,6 +1113,8 @@ receive(void)
     reselect:
         DEBUGMSGTL(("snmpd/select", "select( numfds=%d, ..., tvp=%p)\n",
                     numfds, tvp));
+        if(tvp)
+            DEBUGMSGTL(("timer", "tvp %d.%d\n", tvp->tv_sec, tvp->tv_usec));
         count = select(numfds, &readfds, &writefds, &exceptfds, tvp);
         DEBUGMSGTL(("snmpd/select", "returned, count = %d\n", count));
 
@@ -1193,6 +1185,7 @@ receive(void)
                 snmp_timeout();
                 break;
             case -1:
+                DEBUGMSGTL(("snmpd/select", "  errno = %d\n", errno));
                 if (errno == EINTR) {
                     /*
                      * likely that we got a signal. Check our special signal
@@ -1211,33 +1204,6 @@ receive(void)
                 return -1;
             }                   /* endif -- count>0 */
 
-
-
-
-        /*
-         * If the time 'now' is greater than the 'sched'uled time, then:
-         *
-         *    Check alarm and event timers.
-         *    Reset the 'sched'uled time to current time + one TIMETICK.
-         *    Age the cache network addresses (from whom messges have
-         *        been received).
-         */
-        gettimeofday(nvp, (struct timezone *)NULL);
-
-        if (nvp->tv_sec > svp->tv_sec || (nvp->tv_sec == svp->tv_sec &&
-					  nvp->tv_usec > svp->tv_usec)) {
-            svp->tv_usec = nvp->tv_usec + TIMETICK;
-            svp->tv_sec = nvp->tv_sec;
-
-            while (svp->tv_usec >= ONE_SEC) {
-                svp->tv_usec -= ONE_SEC;
-                svp->tv_sec++;
-            }
-        }
-
-        /*
-         * endif -- now>sched 
-         */
         /*
          * run requested alarms 
          */
