@@ -10,9 +10,6 @@
 
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
-
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
-
 #include <net-snmp/agent/instance.h>
 #include <net-snmp/agent/table.h>
 #include <net-snmp/agent/table_data.h>
@@ -21,6 +18,8 @@
 #include "snmptrapd_log.h"
 #include "notification_log.h"
 
+#define SNMPTRAPD_CONTEXT "snmptrapd"
+
 extern u_long   num_received;
 u_long          num_deleted = 0;
 
@@ -42,6 +41,10 @@ check_log_size(unsigned int clientreg, void *clientarg)
     gettimeofday(&now, NULL);
     tmpl = netsnmp_timeval_uptime(&now);
 
+    if (!nlmLogTable || !nlmLogTable->table )  {
+        DEBUGMSGTL(("notification_log", "missing log table\n"));
+        return;
+    }
     for (row = nlmLogTable->table->first_row; row; row = row->next) {
         /*
          * check max allowed count 
@@ -100,13 +103,13 @@ check_log_size(unsigned int clientreg, void *clientarg)
         deleterow = tmprow;
         num_deleted++;
         /*
-         * XXX: delete vars from it's table 
+         * XXX: delete vars from its table 
          */
     }
 }
 
 
-/** Initialize the nlmLogVariableTable table by defining it's contents and how it's structured */
+/** Initialize the nlmLogVariableTable table by defining its contents and how it's structured */
 void
 initialize_table_nlmLogVariableTable(void)
 {
@@ -115,6 +118,7 @@ initialize_table_nlmLogVariableTable(void)
     size_t          nlmLogVariableTable_oid_len =
         OID_LENGTH(nlmLogVariableTable_oid);
     netsnmp_table_data_set *table_set;
+    netsnmp_handler_registration *reginfo;
 
     /*
      * create the table structure itself 
@@ -260,20 +264,23 @@ initialize_table_nlmLogVariableTable(void)
      * note: if you don't need a subhandler to deal with any aspects of
      * the request, change nlmLogVariableTable_handler to "NULL" 
      */
-    netsnmp_register_table_data_set(netsnmp_create_handler_registration
-                                    ("nlmLogVariableTable",
-                                     nlmLogVariableTable_handler,
-                                     nlmLogVariableTable_oid,
-                                     nlmLogVariableTable_oid_len,
-                                     HANDLER_CAN_RWRITE), table_set, NULL);
+    reginfo =
+        netsnmp_create_handler_registration ("nlmLogVariableTable",
+                                             nlmLogVariableTable_handler,
+                                             nlmLogVariableTable_oid,
+                                             nlmLogVariableTable_oid_len,
+                                             HANDLER_CAN_RWRITE);
+    reginfo->contextName = strdup(SNMPTRAPD_CONTEXT);
+    netsnmp_register_table_data_set(reginfo, table_set, NULL);
 }
 
-/** Initialize the nlmLogTable table by defining it's contents and how it's structured */
+/** Initialize the nlmLogTable table by defining its contents and how it's structured */
 void
 initialize_table_nlmLogTable(void)
 {
     static oid      nlmLogTable_oid[] = { 1, 3, 6, 1, 2, 1, 92, 1, 3, 1 };
     size_t          nlmLogTable_oid_len = OID_LENGTH(nlmLogTable_oid);
+    netsnmp_handler_registration *reginfo;
 
     /*
      * create the table structure itself 
@@ -372,11 +379,13 @@ initialize_table_nlmLogTable(void)
      * note: if you don't need a subhandler to deal with any aspects of
      * the request, change nlmLogTable_handler to "NULL" 
      */
-    netsnmp_register_table_data_set(netsnmp_create_handler_registration
-                                    ("nlmLogTable", nlmLogTable_handler,
-                                     nlmLogTable_oid, nlmLogTable_oid_len,
-                                     HANDLER_CAN_RWRITE), nlmLogTable,
-                                    NULL);
+    reginfo =
+        netsnmp_create_handler_registration("nlmLogTable", nlmLogTable_handler,
+                                            nlmLogTable_oid,
+                                            nlmLogTable_oid_len,
+                                            HANDLER_CAN_RWRITE);
+    reginfo->contextName = strdup(SNMPTRAPD_CONTEXT);
+    netsnmp_register_table_data_set(reginfo, nlmLogTable, NULL);
 
     /*
      * hmm...  5 minutes seems like a reasonable time to check for out
@@ -416,43 +425,47 @@ init_notification_log(void)
     /*
      * static variables 
      */
-    netsnmp_register_read_only_counter32_instance
+#ifdef USING_AGENTX_SUBAGENT_MODULE
+    netsnmp_register_read_only_counter32_instance_context
         ("nlmStatsGlobalNotificationsLogged",
          my_nlmStatsGlobalNotificationsLogged_oid,
          OID_LENGTH(my_nlmStatsGlobalNotificationsLogged_oid),
-         &num_received, NULL);
+         &num_received, NULL, SNMPTRAPD_CONTEXT);
 
-    netsnmp_register_read_only_counter32_instance
+    netsnmp_register_read_only_counter32_instance_context
         ("nlmStatsGlobalNotificationsBumped",
          my_nlmStatsGlobalNotificationsBumped_oid,
          OID_LENGTH(my_nlmStatsGlobalNotificationsBumped_oid),
-         &num_deleted, NULL);
+         &num_deleted, NULL, SNMPTRAPD_CONTEXT);
 
-    netsnmp_register_ulong_instance("nlmConfigGlobalEntryLimit",
-                                    my_nlmConfigGlobalEntryLimit_oid,
-                                    OID_LENGTH
-                                    (my_nlmConfigGlobalEntryLimit_oid),
-                                    &max_logged,
-                                    notification_log_config_handler);
+    netsnmp_register_ulong_instance_context("nlmConfigGlobalEntryLimit",
+                                            my_nlmConfigGlobalEntryLimit_oid,
+                                            OID_LENGTH
+                                            (my_nlmConfigGlobalEntryLimit_oid),
+                                            &max_logged,
+                                            notification_log_config_handler,
+                                            SNMPTRAPD_CONTEXT);
 
-    netsnmp_register_ulong_instance("nlmConfigGlobalAgeOut",
-                                    my_nlmConfigGlobalAgeOut_oid,
-                                    OID_LENGTH
-                                    (my_nlmConfigGlobalAgeOut_oid),
-                                    &max_age,
-                                    notification_log_config_handler);
+    netsnmp_register_ulong_instance_context("nlmConfigGlobalAgeOut",
+                                            my_nlmConfigGlobalAgeOut_oid,
+                                            OID_LENGTH
+                                            (my_nlmConfigGlobalAgeOut_oid),
+                                            &max_age,
+                                            notification_log_config_handler,
+                                            SNMPTRAPD_CONTEXT);
 
     /*
      * tables 
      */
-    initialize_table_nlmLogTable();
     initialize_table_nlmLogVariableTable();
+    initialize_table_nlmLogTable();
 
     /*
      * disable flag 
      */
     netsnmp_ds_register_config(ASN_BOOLEAN, "snmptrapd", "dontRetainLogs",
 			   NETSNMP_DS_APPLICATION_ID, NETSNMP_DS_APP_DONT_LOG);
+#endif /* USING_AGENTX_SUBAGENT_MODULE */
 }
 
 u_long          default_num = 0;
@@ -706,4 +719,3 @@ int   notification_handler(netsnmp_pdu           *pdu,
     log_notification(host, pdu, transport);
     return NETSNMPTRAPD_HANDLER_OK;
 }
-#endif /* USING_AGENTX_SUBAGENT_MODULE && !SNMPTRAPD_DISABLE_AGENTX */
