@@ -192,7 +192,7 @@ optProc(int argc, char *const *argv, int opt)
 			max_getbulk = atoi(argv[optind]);
 			if (max_getbulk == 0) {
 			    usage();
-			    fprintf(stderr, "Bad -Cr option: %s\n", 
+			    fprintf(stderr, "Bad -Cc option: %s\n", 
 				    argv[optind]);
 			    exit(1);
 			}
@@ -309,8 +309,10 @@ main(int argc, char *argv[])
         exit(1);
     }
 
+#ifndef DISABLE_SNMPV1
     if (ss->version == SNMP_VERSION_1)
         use_getbulk = 0;
+#endif
 
     do {
         entries = 0;
