@@ -148,6 +148,12 @@ int             reconfig = 0;
 u_long          num_received = 0;
 char            ddefault_port[] = "udp:162";	/* Default default port */
 char           *default_port = ddefault_port;
+#if HAVE_GETPID
+    FILE           *PID;
+    char           *pid_file = NULL;
+#endif
+char           *trap1_fmt_str_remember = NULL;
+int             dofork = 1;
 
 /*
  * These definitions handle 4.2 systems without additional syslog facilities.
@@ -201,7 +207,7 @@ int Facility = LOG_DAEMON;
 int             trapd_status = SNMPTRAPD_STOPPED;
 LPTSTR          app_name = _T("Net-SNMP Trap Handler");     /* Application Name */
 #else
-const char     *app_name = "snmptrapd";
+char           *app_name = "snmptrapd";
 #endif
 
 struct timeval  Now;
@@ -403,26 +409,31 @@ pre_parse(netsnmp_session * session, netsnmp_transport *transport,
     }
 
     if (addr_string != NULL) {
-        if( addr_string[0] == '[' ) { /* fix up ipv6 addr */
-            int i;
-            for( i = 1; addr_string[i] != ']'; i++ ) {
-                addr_string[i-1] =  addr_string[i];
-            }
-            addr_string[i-1] = '\0';
-        }
+      /* Catch udp,udp6,tcp,tcp6 transports using "[" */
+      char *tcpudpaddr = strstr(addr_string, "[");
+      if ( tcpudpaddr != 0 ) {
+	char sbuf[64];
+	char *xp;
+	strncpy(sbuf, tcpudpaddr + 1, sizeof(sbuf));
+        sbuf[sizeof(sbuf)-1] = '\0';
+        xp = strstr(sbuf, "]");
+        if (xp)
+            *xp = '\0';
+
         if (hosts_ctl("snmptrapd", STRING_UNKNOWN, 
-		      addr_string, STRING_UNKNOWN) == 0) {
-            free(addr_string);
+		      sbuf, STRING_UNKNOWN) == 0) {
+            SNMP_FREE(addr_string);
             return 0;
         }
-        free(addr_string);
+      }
+      SNMP_FREE(addr_string);
     } else {
         if (hosts_ctl("snmptrapd", STRING_UNKNOWN,
                       STRING_UNKNOWN, STRING_UNKNOWN) == 0) {
             return 0;
         }
     }
-#endif	/*  USE_LIBWRAP  */
+#endif/*  USE_LIBWRAP  */
     return 1;
 }
 
@@ -484,6 +495,73 @@ free_trapd_address(void)
     }
 }
 
+void
+parse_config_doNotLogTraps(const char *token, char *cptr)
+{
+  if (atoi(cptr) > 0)
+    SyslogTrap++;
+}
+
+void
+free_config_pidFile(void)
+{
+  if (pid_file)
+    free(pid_file);
+  pid_file = NULL;
+}
+
+void
+parse_config_pidFile(const char *token, char *cptr)
+{
+  free_config_pidFile();
+  pid_file = strdup (cptr);
+}
+
+void
+parse_config_logOption(const char *token, char *cptr)
+{
+  int my_argc = 0 ;
+  char **my_argv = NULL;
+
+  if  (snmp_log_options( cptr, my_argc, my_argv ) >= 0 ) {
+    Log++;
+  }
+}
+
+void
+parse_config_doNotFork(const char *token, char *cptr)
+{
+  if (atoi(cptr) == 1)
+    dofork = 0;
+}
+
+void
+parse_config_printEventNumbers(const char *token, char *cptr)
+{
+  if (atoi(cptr) == 1)
+    Event++;
+}
+
+void
+parse_config_ignoreAuthFailure(const char *token, char *cptr)
+{
+  if (atoi(cptr) == 1)
+    dropauth = 1;
+}
+
+void
+parse_config_outputOption(const char *token, char *cptr)
+{
+  char *cp;
+
+  cp = snmp_out_toggle_options(cptr);
+  if (cp != NULL) {
+    fprintf(stderr, "Unknown output option passed to -O: %c\n",
+        *cp);
+  }
+}
+
+
 /*******************************************************************-o-******
  * main - Non Windows
  * SnmpTrapdMain - Windows to support windows service
@@ -507,55 +585,46 @@ SnmpTrapdMain(int argc, TCHAR * argv[])
 main(int argc, char *argv[])
 #endif
 {
-    char            options[128] = "ac:CdD::efF:hHI:l:L:m:M:no:PqsS:tvO:-:";
+    char            options[128] = "ac:CdD::efF:hHl:L:m:M:no:PqsS:tvO:-:";
     netsnmp_session *sess_list = NULL, *ss = NULL;
     netsnmp_transport *transport = NULL;
-    int             arg, i = 0, depmsg = 0;
+    int             arg, i = 0;
     int             count, numfds, block;
     fd_set          fdset;
     struct timeval  timeout, *tvp;
-    int             dofork = 1;
     char           *cp, *listen_ports = NULL;
-    char           *trap1_fmt_str_remember = NULL;
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
-    int             agentx_subagent = 1;
-#endif
-#if HAVE_GETPID
-    FILE           *PID;
-    char           *pid_file = NULL;
-#endif
-
-#ifdef SIGTERM
-    signal(SIGTERM, term_handler);
-#endif
-#ifdef SIGHUP
-    signal(SIGHUP, hup_handler);
-#endif
-#ifdef SIGINT
-    signal(SIGINT, term_handler);
-#endif
-#ifdef SIGPIPE
-    signal(SIGPIPE, SIG_IGN);   /* 'Inline' failure of wayward readers */
-#endif
+    int             agentx_subagent = 1, depmsg = 0;
 
     /*
      * register our configuration handlers now so -H properly displays them 
      */
     snmptrapd_register_configs( );
-    register_config_handler("snmptrapd", "createUser",
-                            usm_parse_create_usmUser, NULL,
-                           "username (MD5|SHA) passphrase [DES [passphrase]]");
-    register_config_handler("snmptrapd", "usmUser",
-                            usm_parse_config_usmUser, NULL, NULL);
+    init_usm_conf( "snmptrapd" );
     register_config_handler("snmptrapd", "snmptrapdaddr",
-                            parse_trapd_address, free_trapd_address, NULL);
+                            parse_trapd_address, free_trapd_address, "string");
 
-    /*
-     * we need to be called back later 
-     */
-    snmp_register_callback(SNMP_CALLBACK_LIBRARY, SNMP_CALLBACK_STORE_DATA,
-                           usm_store_users, NULL);
+    register_config_handler("snmptrapd", "doNotLogTraps",
+                            parse_config_doNotLogTraps, NULL, "(1|yes|true|0|no|false)");
+#if HAVE_GETPID
+    register_config_handler("snmptrapd", "pidFile",
+                            parse_config_pidFile, NULL, "string");
+#endif
+    
+    register_config_handler("snmptrapd", "logOption",
+                            parse_config_logOption, NULL, "string");
 
+    register_config_handler("snmptrapd", "doNotFork",
+                            parse_config_doNotFork, NULL, "(1|yes|true|0|no|false)");
+
+    register_config_handler("snmptrapd", "printEventNumbers",
+                            parse_config_printEventNumbers, NULL, "(1|yes|true|0|no|false)");
+
+    register_config_handler("snmptrapd", "ignoreAuthFailure",
+                            parse_config_ignoreAuthFailure, NULL, "(1|yes|true|0|no|false)");
+
+    register_config_handler("snmptrapd", "outputOption",
+                            parse_config_outputOption, NULL, "string");
+    
 #ifdef WIN32
     setvbuf(stdout, NULL, _IONBF, BUFSIZ);
 #else
@@ -640,22 +709,11 @@ main(int argc, char *argv[])
             exit(0);
 
         case 'H':
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
             init_notification_log();
-#endif
             init_snmp("snmptrapd");
             fprintf(stderr, "Configuration directives understood:\n");
             read_config_print_usage("  ");
             exit(0);
-            break;
-
-        case 'I':
-            if (optarg != NULL) {
-                add_to_init_list(optarg);
-            } else {
-                usage();
-            }
-            break;
 
 	case 'S':
             fprintf(stderr,
@@ -787,7 +845,7 @@ main(int argc, char *argv[])
                     "Warning: -u option is deprecated; use -p instead\n");
         case 'p':
             if (optarg != NULL) {
-                pid_file = optarg;
+                parse_config_pidFile(NULL, optarg);
             } else {
                 usage();
                 exit(1);
@@ -844,6 +902,7 @@ main(int argc, char *argv[])
     } else {
         netsnmp_add_global_traphandler(NETSNMPTRAPD_PRE_HANDLER, print_handler);
     }
+    netsnmp_add_global_traphandler(NETSNMPTRAPD_POST_HANDLER, notification_handler);
 
     if (Event) {
         netsnmp_add_traphandler(event_handler, risingAlarm,
@@ -858,7 +917,7 @@ main(int argc, char *argv[])
 	 */
     }
 
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+#ifdef USING_AGENTX_SUBAGENT_MODULE
     /*
      * we're an agentx subagent? 
      */
@@ -887,25 +946,40 @@ main(int argc, char *argv[])
      */
     init_agent("snmptrapd");
 
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
     /*
      * initialize local modules 
      */
     if (agentx_subagent) {
+        extern void init_register_usmUser_context(const char *);
+#ifdef USING_AGENTX_SUBAGENT_MODULE
 	void  init_subagent(void);
         init_subagent();
-        if (should_init("notificationLogMib")) {
-            netsnmp_add_global_traphandler(NETSNMPTRAPD_POST_HANDLER,
-                                           notification_handler);
-            init_notification_log();
-        }
+#endif
+        /* register the notification log table */
+        init_notification_log();
+
+        /* register ourselves as having a USM user database */
+        init_register_usmUser_context("snmptrapd");
+    }
+
+#ifdef NETSNMP_EMBEDDED_PERL
+    init_perl();
+    {
+        /* set the default path to load */
+        char            init_file[SNMP_MAXBUF];
+        snprintf(init_file, sizeof(init_file) - 1,
+                 "%s/%s", SNMPSHAREPATH, "snmp_perl_trapd.pl");
+        netsnmp_ds_set_string(NETSNMP_DS_APPLICATION_ID,
+                              NETSNMP_DS_AGENT_PERL_INIT_FILE,
+                              init_file);
     }
 #endif
 
     /*
-     * Initialize the world. Create initial user 
+     * Initialize the world.
      */
     init_snmp("snmptrapd");
+
     if (trap1_fmt_str_remember) {
         free_trap1_fmt();
         free_trap2_fmt();
@@ -965,6 +1039,7 @@ main(int argc, char *argv[])
         }
         fprintf(PID, "%d\n", (int) getpid());
         fclose(PID);
+        free_config_pidFile();
     }
 #endif
 
@@ -1051,10 +1126,11 @@ main(int argc, char *argv[])
         }
     }
 
-    /*
-     * ignore early sighup during startup
-     */
-    reconfig = 0;
+    signal(SIGTERM, term_handler);
+#ifdef SIGHUP
+    signal(SIGHUP, hup_handler);
+#endif
+    signal(SIGINT, term_handler);
 
 #ifdef WIN32SERVICE
     trapd_status = SNMPTRAPD_RUNNING;
