@@ -62,6 +62,10 @@
 #include <arpa/inet.h>
 #endif
 
+#ifdef HAVE_OPENSSL_DH_H
+#include <openssl/dh.h>
+#endif
+
 #include <net-snmp/net-snmp-includes.h>
 
 int             main(int, char **);
@@ -74,32 +78,51 @@ int             main(int, char **);
 #define CMD_DELETE         3
 #define CMD_CLONEFROM_NAME "cloneFrom"
 #define CMD_CLONEFROM      4
+#define CMD_ACTIVATE_NAME  "activate"
+#define CMD_ACTIVATE       5
+#define CMD_DEACTIVATE_NAME "deactivate"
+#define CMD_DEACTIVATE     6
+#define CMD_CHANGEKEY_NAME  "changekey"
+#define CMD_CHANGEKEY      7
 
-#define CMD_NUM    4
+#define CMD_NUM    7
 
 static const char *successNotes[CMD_NUM] = {
     "SNMPv3 Key(s) successfully changed.",
     "User successfully created.",
     "User successfully deleted.",
-    "User successfully cloned."
+    "User successfully cloned.",
+    "User successfully activated.",
+    "User successfully deactivated.",
+    "SNMPv3 Key(s) successfully changed."
 };
 
 #define                   USM_OID_LEN    12
+#define                DH_USM_OID_LEN    11
 
-static oid      authKeyOid[MAX_OID_LEN] =
-    { 1, 3, 6, 1, 6, 3, 15, 1, 2, 2, 1, 6 }, ownAuthKeyOid[MAX_OID_LEN] = {
-1, 3, 6, 1, 6, 3, 15, 1, 2, 2, 1, 7}, privKeyOid[MAX_OID_LEN] = {
-1, 3, 6, 1, 6, 3, 15, 1, 2, 2, 1, 9}, ownPrivKeyOid[MAX_OID_LEN] = {
-1, 3, 6, 1, 6, 3, 15, 1, 2, 2, 1, 10}, usmUserCloneFrom[MAX_OID_LEN] = {
-1, 3, 6, 1, 6, 3, 15, 1, 2, 2, 1, 4}, usmUserSecurityName[MAX_OID_LEN] = {
-1, 3, 6, 1, 6, 3, 15, 1, 2, 2, 1, 3}, usmUserStatus[MAX_OID_LEN] = {
-1, 3, 6, 1, 6, 3, 15, 1, 2, 2, 1, 13}
+static oid
 
+authKeyOid[MAX_OID_LEN] = { 1, 3, 6, 1, 6, 3, 15, 1, 2, 2, 1, 6 },
+ownAuthKeyOid[MAX_OID_LEN] = {1, 3, 6, 1, 6, 3, 15, 1, 2, 2, 1, 7},
+privKeyOid[MAX_OID_LEN] = {1, 3, 6, 1, 6, 3, 15, 1, 2, 2, 1, 9},
+ownPrivKeyOid[MAX_OID_LEN] = {1, 3, 6, 1, 6, 3, 15, 1, 2, 2, 1, 10},
+usmUserCloneFrom[MAX_OID_LEN] = {1, 3, 6, 1, 6, 3, 15, 1, 2, 2, 1, 4},
+usmUserSecurityName[MAX_OID_LEN] = {1, 3, 6, 1, 6, 3, 15, 1, 2, 2, 1, 3},
+usmUserStatus[MAX_OID_LEN] = {1, 3, 6, 1, 6, 3, 15, 1, 2, 2, 1, 13},
+/* diffie helman change key objects */
+usmDHUserAuthKeyChange[MAX_OID_LEN] = {1, 3, 6, 1, 3, 101, 1, 1, 2, 1, 1 },
+usmDHUserOwnAuthKeyChange[MAX_OID_LEN] = {1, 3, 6, 1, 3, 101, 1, 1, 2, 1, 2 },
+usmDHUserPrivKeyChange[MAX_OID_LEN] = {1, 3, 6, 1, 3, 101, 1, 1, 2, 1, 3 },
+usmDHUserOwnPrivKeyChange[MAX_OID_LEN] = {1, 3, 6, 1, 3, 101, 1, 1, 2, 1, 4 },
+usmDHParameters[] = { 1,3,6,1,3,101,1,1,1,0 }
 ;
+size_t usmDHParameters_len = OID_LENGTH(usmDHParameters);
 
 static
 oid            *authKeyChange = authKeyOid, *privKeyChange = privKeyOid;
-int             doauthkey = 0, doprivkey = 0;
+oid            *dhauthKeyChange = usmDHUserAuthKeyChange,
+               *dhprivKeyChange = usmDHUserPrivKeyChange;
+int             doauthkey = 0, doprivkey = 0, uselocalizedkey = 0;
 
 void
 usage(void)
@@ -109,14 +132,19 @@ usage(void)
     fprintf(stderr, " COMMAND\n\n");
     snmp_parse_args_descriptions(stderr);
     fprintf(stderr, "\nsnmpusm commands:\n");
-    fprintf(stderr, "  create    USER [CLONEFROM-USER]\n");
-    fprintf(stderr, "  delete    USER\n");
-    fprintf(stderr, "  cloneFrom USER CLONEFROM-USER\n");
+    fprintf(stderr, "  create     USER [CLONEFROM-USER]\n");
+    fprintf(stderr, "  delete     USER\n");
+    fprintf(stderr, "  cloneFrom  USER CLONEFROM-USER\n");
+    fprintf(stderr, "  activate   USER\n");
+    fprintf(stderr, "  deactivate USER\n");
+    fprintf(stderr, "  [-Ca] [-Cx] changekey [USER]\n");
     fprintf(stderr,
-            "  [-Co] [-Ca] [-Cx] passwd OLD-PASSPHRASE NEW-PASSPHRASE\n");
-    fprintf(stderr, "\t\t-Co\t\tUse the ownKeyChange objects.\n");
+            "  [-Ca] [-Cx] passwd OLD-PASSPHRASE NEW-PASSPHRASE [USER]\n");
+    fprintf(stderr,
+            "  <-Ca | -Cx> -Ck passwd OLD-LOCALIZED-KEY NEW-PASSPHRASE [USER]\n");
     fprintf(stderr, "\t\t-Cx\t\tChange the privacy key.\n");
     fprintf(stderr, "\t\t-Ca\t\tChange the authentication key.\n");
+    fprintf(stderr, "\t\t-Ck\t\tUse old localized key instead of old passphrase.\n");
 }
 
 /*
@@ -126,7 +154,7 @@ void
 setup_oid(oid * it, size_t * len, u_char * id, size_t idlen,
           const char *user)
 {
-    int             i, itIndex = 12;
+    int             i, itIndex = *len;
 
     *len = itIndex + 1 + idlen + 1 + strlen(user);
 
@@ -151,6 +179,68 @@ setup_oid(oid * it, size_t * len, u_char * id, size_t idlen,
      */
 }
 
+#if defined(HAVE_OPENSSL_DH_H) && defined(HAVE_LIBCRYPTO)
+int
+get_USM_DH_key(netsnmp_variable_list *vars, netsnmp_variable_list *dhvar,
+               size_t outkey_len,
+               netsnmp_pdu *pdu, const char *keyname,
+               oid *keyoid, size_t keyoid_len) {
+    char *dhkeychange;
+    DH *dh;
+    BIGNUM *other_pub;
+    u_char *key;
+    size_t key_len;
+    unsigned char *cp;
+            
+    dhkeychange = (char *) malloc(2 * vars->val_len * sizeof(char));
+    memcpy(dhkeychange, vars->val.string, vars->val_len);
+
+    cp = dhvar->val.string;
+    dh = d2i_DHparams(NULL, (const unsigned char **) &cp,
+                      dhvar->val_len);
+
+    if (!dh || !dh->g || !dh->p)
+        return SNMPERR_GENERR;
+    DH_generate_key(dh);
+    if (!dh->pub_key)
+        return SNMPERR_GENERR;
+            
+    if (vars->val_len != BN_num_bytes(dh->pub_key)) {
+        fprintf(stderr,"incorrect diffie-helman lengths (%d != %d)\n",
+                vars->val_len, BN_num_bytes(dh->pub_key));
+        return SNMPERR_GENERR;
+    }
+
+    BN_bn2bin(dh->pub_key, dhkeychange + vars->val_len);
+
+    key_len = DH_size(dh);
+    if (!key_len)
+        return SNMPERR_GENERR;
+    key = (u_char *) malloc(key_len * sizeof(u_char));
+
+    other_pub = BN_bin2bn(vars->val.string, vars->val_len, NULL);
+    if (!other_pub)
+        return SNMPERR_GENERR;
+
+    if (DH_compute_key(key, other_pub, dh)) {
+        u_char *kp;
+
+        printf("new %s key: 0x", keyname);
+        for(kp = key + key_len - outkey_len;
+            kp - key < key_len;  kp++) {
+            printf("%02x", (unsigned char) *kp);
+        }
+        printf("\n");
+    }
+
+    snmp_pdu_add_variable(pdu, keyoid, keyoid_len,
+                          ASN_OCTET_STR, dhkeychange,
+                          2 * vars->val_len);
+
+    return SNMPERR_SUCCESS;
+}
+#endif /* HAVE_OPENSSL_DH_H */
+
 static void
 optProc(int argc, char *const *argv, int opt)
 {
@@ -158,11 +248,6 @@ optProc(int argc, char *const *argv, int opt)
     case 'C':
         while (*optarg) {
             switch (*optarg++) {
-            case 'o':
-                authKeyChange = ownAuthKeyOid;
-                privKeyChange = ownPrivKeyOid;
-                break;
-
             case 'a':
                 doauthkey = 1;
                 break;
@@ -171,6 +256,10 @@ optProc(int argc, char *const *argv, int opt)
                 doprivkey = 1;
                 break;
 
+	    case 'k':
+	        uselocalizedkey = 1;
+		break;
+
             default:
                 fprintf(stderr, "Unknown flag passed to -C: %c\n",
                         optarg[-1]);
@@ -212,13 +301,20 @@ main(int argc, char *argv[])
     size_t          oldKu_len = SNMP_MAXBUF_SMALL,
         newKu_len = SNMP_MAXBUF_SMALL,
         oldkul_len = SNMP_MAXBUF_SMALL,
-        newkul_len = SNMP_MAXBUF_SMALL, keychange_len = SNMP_MAXBUF_SMALL;
+        oldkulpriv_len = SNMP_MAXBUF_SMALL,
+        newkulpriv_len = SNMP_MAXBUF_SMALL,
+        newkul_len = SNMP_MAXBUF_SMALL,
+        keychange_len = SNMP_MAXBUF_SMALL,
+        keychangepriv_len = SNMP_MAXBUF_SMALL;
 
     char           *newpass = NULL, *oldpass = NULL;
     u_char          oldKu[SNMP_MAXBUF_SMALL],
         newKu[SNMP_MAXBUF_SMALL],
         oldkul[SNMP_MAXBUF_SMALL],
-        newkul[SNMP_MAXBUF_SMALL], keychange[SNMP_MAXBUF_SMALL];
+        oldkulpriv[SNMP_MAXBUF_SMALL],
+        newkulpriv[SNMP_MAXBUF_SMALL],
+        newkul[SNMP_MAXBUF_SMALL], keychange[SNMP_MAXBUF_SMALL],
+        keychangepriv[SNMP_MAXBUF_SMALL];
 
     authKeyChange = authKeyOid;
     privKeyChange = privKeyOid;
@@ -272,9 +368,12 @@ main(int argc, char *argv[])
          * XXX:  Uses the auth type of the calling user, a MD5 user can't
          *       change a SHA user's key.
          */
+        char *passwd_user;
+
         command = CMD_PASSWD;
         oldpass = argv[++arg];
         newpass = argv[++arg];
+        passwd_user = argv[++arg];
 
         if (doprivkey == 0 && doauthkey == 0)
             doprivkey = doauthkey = 1;
@@ -293,6 +392,19 @@ main(int argc, char *argv[])
             exit(1);
         }
 
+        /* 
+         * Change the user supplied on command line.
+         */
+        if ((passwd_user != NULL) && (strlen(passwd_user) > 0)) {
+            session.securityName = passwd_user;
+        } else {
+            /*
+             * Use own key object if no user was supplied.
+             */
+            authKeyChange = ownAuthKeyOid;
+            privKeyChange = ownPrivKeyOid;
+        }
+
         /*
          * do we have a securityName?  If not, copy the default 
          */
@@ -318,11 +430,20 @@ main(int argc, char *argv[])
             /*
              * assume MD5 
              */
+#ifndef DISABLE_MD5
             session.securityAuthProtoLen =
                 sizeof(usmHMACMD5AuthProtocol) / sizeof(oid);
             session.securityAuthProto =
                 snmp_duplicate_objid(usmHMACMD5AuthProtocol,
                                      session.securityAuthProtoLen);
+#else
+            session.securityAuthProtoLen =
+                sizeof(usmHMACSHA1AuthProtocol) / sizeof(oid);
+            session.securityAuthProto =
+                snmp_duplicate_objid(usmHMACSHA1AuthProtocol,
+                                     session.securityAuthProtoLen);
+#endif
+
         }
         rval = generate_Ku(session.securityAuthProto,
                            session.securityAuthProtoLen,
@@ -335,33 +456,53 @@ main(int argc, char *argv[])
             exit(1);
         }
 
-        /*
-         * the old Ku is in the session, but we need the new one 
-         */
-        rval = generate_Ku(session.securityAuthProto,
-                           session.securityAuthProtoLen,
-                           (u_char *) oldpass, strlen(oldpass),
-                           oldKu, &oldKu_len);
+	if (uselocalizedkey) {
+	    /*
+	     * use the localized key from the command line
+	     */
+	    u_char *buf;
+	    size_t buf_len = SNMP_MAXBUF_SMALL;
+	    buf = (u_char *) malloc (buf_len * sizeof(u_char));
 
-        if (rval != SNMPERR_SUCCESS) {
-            snmp_perror(argv[0]);
-            fprintf(stderr, "generating the new Ku failed\n");
-            exit(1);
-        }
+	    oldkul_len = 0; /* initialize the offset */
+	    if (!snmp_hex_to_binary((u_char **) (&buf), &buf_len, &oldkul_len, 0, oldpass)) {
+	      snmp_perror(argv[0]);
+	      fprintf(stderr, "generating the old Kul from localized key failed\n");
+	      exit(1);
+	    }
+	    
+	    memcpy(oldkul, buf, oldkul_len);
+	    SNMP_FREE(buf);
+	}
+	else {
+	    /*
+	     * the old Ku is in the session, but we need the new one 
+	     */
+	    rval = generate_Ku(session.securityAuthProto,
+			       session.securityAuthProtoLen,
+			       (u_char *) oldpass, strlen(oldpass),
+			       oldKu, &oldKu_len);
+	    
+	    if (rval != SNMPERR_SUCCESS) {
+	        snmp_perror(argv[0]);
+	        fprintf(stderr, "generating the new Ku failed\n");
+	        exit(1);
+	    }
 
-        /*
-         * generate the two Kul's 
-         */
-        rval = generate_kul(session.securityAuthProto,
-                            session.securityAuthProtoLen,
-                            ss->contextEngineID, ss->contextEngineIDLen,
-                            oldKu, oldKu_len, oldkul, &oldkul_len);
-
-        if (rval != SNMPERR_SUCCESS) {
-            snmp_perror(argv[0]);
-            fprintf(stderr, "generating the old Kul failed\n");
-            exit(1);
-        }
+	    /*
+	     * generate the two Kul's 
+	     */
+	    rval = generate_kul(session.securityAuthProto,
+				session.securityAuthProtoLen,
+				ss->contextEngineID, ss->contextEngineIDLen,
+				oldKu, oldKu_len, oldkul, &oldkul_len);
+	    
+	    if (rval != SNMPERR_SUCCESS) {
+	        snmp_perror(argv[0]);
+		fprintf(stderr, "generating the old Kul failed\n");
+		exit(1);
+	    }
+	}
 
         rval = generate_kul(session.securityAuthProto,
                             session.securityAuthProtoLen,
@@ -374,21 +515,67 @@ main(int argc, char *argv[])
             exit(1);
         }
 
+        /*
+         * for encryption, we may need to truncate the key to the proper length
+         * so we need two copies.  For simplicity, we always just copy even if
+         * they're the same lengths.
+         */
+        if (doprivkey) {
+            if (!session.securityPrivProto) {
+                snmp_log(LOG_ERR, "no encryption type specified, which I need in order to know to change the key\n");
+                exit(1);
+            }
+                
+#ifndef DISABLE_DES
+            if (ISTRANSFORM(session.securityPrivProto, DESPriv)) {
+                /* DES uses a 128 bit key, 64 bits of which is a salt */
+                oldkulpriv_len = newkulpriv_len = 16;
+            }
+#endif
+#ifdef HAVE_AES
+            if (ISTRANSFORM(session.securityPrivProto, AESPriv)) {
+                oldkulpriv_len = newkulpriv_len = 16;
+            }
+#endif
+            memcpy(oldkulpriv, oldkul, oldkulpriv_len);
+            memcpy(newkulpriv, newkul, newkulpriv_len);
+        }
+            
+
         /*
          * create the keychange string 
          */
-        rval = encode_keychange(session.securityAuthProto,
-                                session.securityAuthProtoLen,
-                                oldkul, oldkul_len,
-                                newkul, newkul_len,
-                                keychange, &keychange_len);
+	if (doauthkey) {
+	  rval = encode_keychange(session.securityAuthProto,
+				  session.securityAuthProtoLen,
+				  oldkul, oldkul_len,
+				  newkul, newkul_len,
+				  keychange, &keychange_len);
 
-        if (rval != SNMPERR_SUCCESS) {
+	  if (rval != SNMPERR_SUCCESS) {
+	    snmp_perror(argv[0]);
+            fprintf(stderr, "encoding the keychange failed\n");
+            usage();
+            exit(1);
+	  }
+	}
+
+        /* which is slightly different for encryption if lengths are
+           different */
+	if (doprivkey) {
+	  rval = encode_keychange(session.securityAuthProto,
+                                session.securityAuthProtoLen,
+                                oldkulpriv, oldkulpriv_len,
+                                newkulpriv, newkulpriv_len,
+                                keychangepriv, &keychangepriv_len);
+
+	  if (rval != SNMPERR_SUCCESS) {
             snmp_perror(argv[0]);
             fprintf(stderr, "encoding the keychange failed\n");
             usage();
             exit(1);
-        }
+	  }
+	}
 
         /*
          * add the keychange string to the outgoing packet 
@@ -401,11 +588,12 @@ main(int argc, char *argv[])
                                   ASN_OCTET_STR, keychange, keychange_len);
         }
         if (doprivkey) {
-            setup_oid(privKeyChange, &name_length,
+            setup_oid(privKeyChange, &name_length2,
                       ss->contextEngineID, ss->contextEngineIDLen,
                       session.securityName);
-            snmp_pdu_add_variable(pdu, privKeyChange, name_length,
-                                  ASN_OCTET_STR, keychange, keychange_len);
+            snmp_pdu_add_variable(pdu, privKeyChange, name_length2,
+                                  ASN_OCTET_STR,
+                                  keychangepriv, keychangepriv_len);
         }
 
     } else if (strcmp(argv[arg], CMD_CREATE_NAME) == 0) {
@@ -434,6 +622,7 @@ main(int argc, char *argv[])
                                   ASN_INTEGER, (u_char *) & longvar,
                                   sizeof(longvar));
 
+            name_length = USM_OID_LEN;
             setup_oid(usmUserCloneFrom, &name_length,
                       ss->contextEngineID, ss->contextEngineIDLen,
                       argv[arg - 1]);
@@ -477,6 +666,7 @@ main(int argc, char *argv[])
         snmp_pdu_add_variable(pdu, usmUserStatus, name_length,
                               ASN_INTEGER, (u_char *) & longvar,
                               sizeof(longvar));
+        name_length = USM_OID_LEN;
         setup_oid(usmUserCloneFrom, &name_length,
                   ss->contextEngineID, ss->contextEngineIDLen, argv[arg]);
 
@@ -512,6 +702,158 @@ main(int argc, char *argv[])
         snmp_pdu_add_variable(pdu, usmUserStatus, name_length,
                               ASN_INTEGER, (u_char *) & longvar,
                               sizeof(longvar));
+    } else if (strcmp(argv[arg], CMD_ACTIVATE_NAME) == 0) {
+        /*
+         * activate:  activate a user
+         *
+         * activate USER
+         */
+        if (++arg >= argc) {
+            fprintf(stderr, "You must specify the user name to activate\n");
+            exit(1);
+        }
+
+        command = CMD_ACTIVATE;
+        setup_oid(usmUserStatus, &name_length,
+                  ss->contextEngineID, ss->contextEngineIDLen, argv[arg]);
+        longvar = RS_ACTIVE;
+        snmp_pdu_add_variable(pdu, usmUserStatus, name_length,
+                              ASN_INTEGER, (u_char *) & longvar,
+                              sizeof(longvar));
+    } else if (strcmp(argv[arg], CMD_DEACTIVATE_NAME) == 0) {
+        /*
+         * deactivate:  deactivate a user
+         *
+         * deactivate USER
+         */
+        if (++arg >= argc) {
+            fprintf(stderr, "You must specify the user name to deactivate\n");
+            exit(1);
+        }
+
+        command = CMD_DEACTIVATE;
+        setup_oid(usmUserStatus, &name_length,
+                  ss->contextEngineID, ss->contextEngineIDLen, argv[arg]);
+        longvar = RS_NOTINSERVICE;
+        snmp_pdu_add_variable(pdu, usmUserStatus, name_length,
+                              ASN_INTEGER, (u_char *) & longvar,
+                              sizeof(longvar));
+#if defined(HAVE_OPENSSL_DH_H) && defined(HAVE_LIBCRYPTO)
+    } else if (strcmp(argv[arg], CMD_CHANGEKEY_NAME) == 0) {
+        /*
+         * change the key of a user if DH is available
+         */
+
+        char *passwd_user;
+        netsnmp_pdu *dhpdu, *dhresponse = NULL;
+        netsnmp_variable_list *vars, *dhvar;
+        
+        command = CMD_CHANGEKEY;
+        name_length = DH_USM_OID_LEN;
+        name_length2 = DH_USM_OID_LEN;
+
+        passwd_user = argv[++arg];
+
+        if (doprivkey == 0 && doauthkey == 0)
+            doprivkey = doauthkey = 1;
+
+        /* 
+         * Change the user supplied on command line.
+         */
+        if ((passwd_user != NULL) && (strlen(passwd_user) > 0)) {
+            session.securityName = passwd_user;
+        } else {
+            /*
+             * Use own key object if no user was supplied.
+             */
+            dhauthKeyChange = usmDHUserOwnAuthKeyChange;
+            dhprivKeyChange = usmDHUserOwnPrivKeyChange;
+        }
+
+        /*
+         * do we have a securityName?  If not, copy the default 
+         */
+        if (session.securityName == NULL) {
+            session.securityName = 
+	      strdup(netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
+					   NETSNMP_DS_LIB_SECNAME));
+        }
+
+        /* fetch the needed diffie helman parameters */
+        dhpdu = snmp_pdu_create(SNMP_MSG_GET);
+
+        /* get the current DH parameters */
+        snmp_add_null_var(dhpdu, usmDHParameters, usmDHParameters_len);
+        
+        /* maybe the auth key public value */
+        if (doauthkey) {
+            setup_oid(dhauthKeyChange, &name_length,
+                      ss->contextEngineID, ss->contextEngineIDLen,
+                      session.securityName);
+            snmp_add_null_var(dhpdu, dhauthKeyChange, name_length);
+        }
+            
+        /* maybe the priv key public value */
+        if (doprivkey) {
+            setup_oid(dhprivKeyChange, &name_length2,
+                      ss->contextEngineID, ss->contextEngineIDLen,
+                      session.securityName);
+            snmp_add_null_var(dhpdu, dhprivKeyChange, name_length2);
+        }
+
+        /* fetch the values */
+        status = snmp_synch_response(ss, dhpdu, &dhresponse);
+
+        if (status != SNMPERR_SUCCESS || dhresponse == NULL ||
+            dhresponse->errstat != SNMP_ERR_NOERROR ||
+            dhresponse->variables->type != ASN_OCTET_STR) {
+            snmp_sess_perror("snmpusm", ss);
+            if (dhresponse && dhresponse->variables &&
+                dhresponse->variables->type != ASN_OCTET_STR) {
+                fprintf(stderr,
+                        "Can't get diffie-helman exchange from the agent\n");
+                fprintf(stderr,
+                        "  (maybe it doesn't support the SNMP-USM-DH-OBJECTS-MIB MIB)\n");
+            }
+            exitval = 1;
+            goto begone;
+        }
+        
+        dhvar = dhresponse->variables;
+        vars = dhvar->next_variable;
+        /* complete the DH equation & print resulting keys */
+        if (doauthkey) {
+            if (get_USM_DH_key(vars, dhvar,
+                               sc_get_properlength(ss->securityAuthProto,
+                                                   ss->securityAuthProtoLen),
+                               pdu, "auth",
+                               dhauthKeyChange, name_length) != SNMPERR_SUCCESS)
+                goto begone;
+            vars = vars->next_variable;
+        }
+        if (doprivkey) {
+	    size_t dhprivKeyLen = 0;
+#ifndef DISABLE_DES
+	    if (ISTRANSFORM(ss->securityPrivProto, DESPriv)) {
+                /* DES uses a 128 bit key, 64 bits of which is a salt */
+	        dhprivKeyLen = 16;
+	    }
+#endif
+#ifdef HAVE_AES
+	    if (ISTRANSFORM(ss->securityPrivProto, AESPriv)) {
+	        dhprivKeyLen = 16;
+	    }
+#endif
+            if (get_USM_DH_key(vars, dhvar,
+                               dhprivKeyLen,
+                               pdu, "priv",
+                               dhprivKeyChange, name_length2)
+                != SNMPERR_SUCCESS)
+                goto begone;
+            vars = vars->next_variable;
+        }
+        /* snmp_free_pdu(dhresponse); */ /* parts still in use somewhere */
+#endif /* HAVE_OPENSSL_DH_H */
     } else {
         fprintf(stderr, "Unknown command\n");
         usage();
@@ -555,6 +897,7 @@ main(int argc, char *argv[])
         exitval = 1;
     }
 
+  begone:
     if (response)
         snmp_free_pdu(response);
     snmp_close(ss);
