@@ -74,7 +74,6 @@ PERFORMANCE OF THIS SOFTWARE.
     struct nlist;
 
     extern long     long_return;
-    extern uint32_t ipaddr_return;
     extern u_char   return_buf[];
 
     extern oid      nullOid[];
@@ -97,9 +96,6 @@ PERFORMANCE OF THIS SOFTWARE.
     int             init_agent(const char *);
     void            shutdown_agent(void);
 
-    int             should_init(const char *module_name);
-    void            add_to_init_list(char *module_list);
-
 #ifdef USING_AGENTX_SUBAGENT_MODULE
     void            netsnmp_enable_subagent(void);
 #endif
