@@ -39,7 +39,8 @@ void            netsnmp_udp_parse_security(const char *token, char *param);
 int             netsnmp_udp_getSecName(void *opaque, int olength,
                                        const char *community,
                                        size_t community_len,
-                                       char **secname);
+                                       char **secname,
+                                       char **contextName);
 
 /*
  * "Constructor" for transport domain object.  
