@@ -1,7 +1,5 @@
 #include "bsd.h"
 
-#include <sys/param.h>
-
 #define PCB_TABLE 1
 #undef TCP_SYMBOL
 #define TCP_SYMBOL "tcbtable"
@@ -24,12 +22,6 @@
 #define INP_PREV_SYMBOL inp_queue.cqe_prev
 #define HAVE_INPCBTABLE 1
 
-#if __NetBSD_Version__ >= 106300000       /* NetBSD 1.6ZD */            
-#undef IFADDR_SYMBOL
-#define IFADDR_SYMBOL "in_ifaddrhead"
-#undef TOTAL_MEMORY_SYMBOL
-#endif
-
 #define UTMP_FILE _PATH_UTMP
 
 #ifndef netbsd1
@@ -40,7 +32,7 @@
  * fixups for broken AC_CHECK_IFNET_FOR() 
  */
 /*
- * Remove when fixed in configure.in 
+ * XXX - Remove when fixed in configure.in 
  */
 #undef STRUCT_IFNET_HAS_IF_BAUDRATE
 #define STRUCT_IFNET_HAS_IF_BAUDRATE 1
