@@ -7,7 +7,7 @@
 #     modify it under the same terms as Perl itself.
 
 package SNMP;
-$VERSION = '5.1.3';   # current release version number
+$VERSION = '5.2.1.2';   # current release version number
 
 require Exporter;
 require DynaLoader;
@@ -80,6 +80,7 @@ tie $SNMP::dump_packet, SNMP::DUMP_PACKET;
 tie %SNMP::MIB, SNMP::MIB;
 tie $SNMP::save_descriptions, SNMP::MIB::SAVE_DESCR;
 tie $SNMP::replace_newer, SNMP::MIB::REPLACE_NEWER;
+tie $SNMP::mib_options, SNMP::MIB::MIB_OPTIONS;
 
 %SNMP::V3_SEC_LEVEL_MAP = (noAuthNoPriv => 1, authNoPriv => 2, authPriv =>3);
 
@@ -378,37 +379,6 @@ sub _tie {
     tie($_[0],$_[1],$_[2],$_[3]);
 }
 
-sub split_vars {
-    # This sub holds the regex that is used throughout this module  
-    #  to parse the base part of an OID from the IID.
-    #  eg: portName.9.30 -> ['portName','9.30'] 
-    my $vars = shift;
-
-    # The regex was changed to this simple form by patch 722075 for some reason.
-    # Testing shows now (2/05) that it is not needed, and that the long expression 
-    # works fine.  AB
-    # my ($tag, $iid) = ($vars =~ /^(.*?)\.?(\d+)+$/);
-    
-    # These following two are the same.  Broken down for easier maintenance
-    # my ($tag, $iid) = ($vars =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
-    my ($tag, $iid) =
-        ($vars =~ /^(               # Capture $1
-                    # 1. either this 5.5.5.5
-                     (?:\.\d+)+     # for grouping, won't increment $1
-                    |
-                    # 2. or asdf-asdf-asdf-asdf
-                     (?:            # grouping again
-                        \w+         # needs some letters followed by
-                        (?:\-*\w+)+ #  zero or more dashes, one or more letters
-                     )
-                    )
-                    \.?             # optionally match a dot
-                    (.*)            # whatever is left in the string is our iid ($2)
-                   $/x
-    );
-    return [$tag,$iid];
-}
-
 package SNMP::Session;
 
 sub new {
@@ -489,6 +459,12 @@ sub new {
 		     NetSNMP::default_store::NETSNMP_DS_LIB_AUTHPASSPHRASE()) ||
        NetSNMP::default_store::netsnmp_ds_get_string(NetSNMP::default_store::NETSNMP_DS_LIBRARY_ID(), 
 		     NetSNMP::default_store::NETSNMP_DS_LIB_PASSPHRASE()) || '';
+
+       $this->{AuthMasterKey} ||= '';
+       $this->{PrivMasterKey} ||= '';
+       $this->{AuthLocalizedKey} ||= '';
+       $this->{PrivLocalizedKey} ||= '';
+
        $this->{PrivProto} ||= 'DEFAULT';  # defaults to hte library's default
        $this->{PrivPass} ||=
        NetSNMP::default_store::netsnmp_ds_get_string(NetSNMP::default_store::NETSNMP_DS_LIBRARY_ID(), 
@@ -513,7 +489,15 @@ sub new {
 						$this->{PrivPass},
 						$this->{EngineBoots},
 						$this->{EngineTime},
-						);
+						$this->{AuthMasterKey},
+						length($this->{AuthMasterKey}),
+						$this->{PrivMasterKey},
+						length($this->{PrivMasterKey}),
+						$this->{AuthLocalizedKey},
+						length($this->{AuthLocalizedKey}),
+						$this->{PrivLocalizedKey},
+						length($this->{PrivLocalizedKey}),
+					       );
    }
    unless ($this->{SessPtr}) {
        warn("unable to create session") if $SNMP::verbose;
@@ -583,11 +567,10 @@ sub set {
      $varbind_list_ref = [$vars];
      $varbind_list_ref = $vars if ref($$vars[0]) =~ /ARRAY/;
    } else {
-     #$varbind_list_ref = [[$tag, $iid, $val]];
-     my $split_vars = SNMP::split_vars($vars);
+     # my ($tag, $iid) = ($vars =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
+     my ($tag, $iid) = ($vars =~ /^(.*?)\.?(\d+)+$/);     
      my $val = shift;
-     push @$split_vars,$val;
-     $varbind_list_ref = [$split_vars];
+     $varbind_list_ref = [[$tag, $iid, $val]];
    }
    my $cb = shift;
 
@@ -607,7 +590,9 @@ sub get {
      $varbind_list_ref = [$vars];
      $varbind_list_ref = $vars if ref($$vars[0]) =~ /ARRAY/;
    } else {
-     $varbind_list_ref = [SNMP::split_vars($vars)];
+     # my ($tag, $iid) = ($vars =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
+     my ($tag, $iid) = ($vars =~ /^(.*?)\.?(\d+)+$/);
+     $varbind_list_ref = [[$tag, $iid]];
    }
 
    my $cb = shift;
@@ -618,90 +603,181 @@ sub get {
 }
 
 
-
+$have_netsnmp_oid = eval { require NetSNMP::OID; };
 sub gettable {
 
-	#
-	# getTable
-	# --------
-	#
-	# Get OIDs starting at $table_oid, and continue down the tree
-	# until we get to an OID which does not start with $table_oid,
-	# i.e. we have reached the end of this table.
-	#
+    #
+    # getTable
+    # --------
+    #
+    # Get OIDs starting at $table_oid, and continue down the tree
+    # until we get to an OID which does not start with $table_oid,
+    # i.e. we have reached the end of this table.
+    #
 
-	my $this = shift;
-	my $root_oid = shift;
+    my ($this, $root_oid, $options) = @_;
+    my ($textnode, $stopconds, $varbinds, $vbl, $res, %result_hash, $repeat);
 
+    # translate the OID into numeric form if its not
+    if ($root_oid !~ /^[\.0-9]+$/) {
+	$textnode = $root_oid;
+	$root_oid = SNMP::translateObj($root_oid);
+    } else {
+	$textnode = SNMP::translateObj($root_oid);
+    }
 
-	# Check if the root OID is a VarList
+    # bail if we don't have a valid oid.
+    return if (!$root_oid);
 
-	if(ref($root_oid) =~ /SNMP::VarList/) {
+    # deficed if we're going to parse indexes
+    my $parse_indexes = (defined($options->{'noindexes'})) ? 
+      0 : $have_netsnmp_oid;
 
-		$table_root_oid = $root_oid;
+    # get the list of columns we should look at.
+    my @columns;
+    if (!$options->{'columns'}) {
+	if ($textnode) {
+	    my %indexes;
 
-	} elsif(ref($root_oid) =~ /SNMP::Varbind/) {
+	    if ($parse_indexes) {
+		# get indexes
+		my @indexes =
+		  @{$SNMP::MIB{$textnode}{'children'}[0]{'indexes'}};
+		# quick translate into a hash
+		map { $indexes{$_} = 1; } @indexes;
+	    }
 
-		$table_root_oid = [$root_oid];
+	    # calculate the list of accessible columns that aren't indexes
+	    my $children = $SNMP::MIB{$textnode}{'children'}[0]{'children'};
+	    foreach my $c (@$children) {
+		push @columns,
+		  $root_oid . ".1." . $c->{'subID'}
+		    if (!$indexes{$c->{'label'}});
+	    }
+	    if ($#columns == -1) {
+		# some tables are only indexes, and we need to walk at
+		# least one column.  We pick the last.
+		push @columns, $root_oid . ".1." .
+		  $children->[$#$children]{'subID'};
+	    }
+	}
+    } else {
+	# XXX: requires specification in numeric OID...  ack.!
+	@columns = @{$options->{'columns'}};
+    }
 
-	} elsif(ref($root_oid) =~ /ARRAY/) {
+    # create the initial walking info.
+    foreach my $c (@columns) {
+	push @$varbinds, [$c];
+	push @$stopconds, $c;
+    }
+    $vbl = $varbinds;
+	
+    my $repeatcount;
+    if ($opts->{nogetbulk}) {
+	$repeatcount = 1;
+    } elsif ($options->{'repeat'}) {
+	$repeatcount = $options->{'repeat'};
+    } else {
+	# experimentally determined maybe guess at a best repeat value
+	# 1000 bytes max (safe), 30 bytes average for encoding of the
+	# varbind (experimentally determined to be closer to
+	# 26.  Again, being safe.  Then devide by the number of
+	# varbinds.
+	$repeatcount = int(1000 / 36 / ($#$varbinds + 1));
+    }
 
-		$table_root_oid = [$root_oid];
-		$table_root_oid = $root_oid if ref($$root_oid[0]) =~ /ARRAY/;
+    if ($this->{Version} > 1 && !$options->{'nogetbulk'}) {
+	$res = $this->getbulk(0, $repeatcount, $vbl);
+    } else {
+	$res = $this->getnext($vbl);
+    }
 
-	} else {
+    while ($#$vbl > -1 && !$this->{ErrorNum}) {
+	if ($#$vbl + 1 != ($#$stopconds + 1) * $repeatcount) {
+	    print STDERR "ack: gettable results not appropriate\n";
+	    my @k = keys(%result_hash);
+	    last if ($#k > -1);  # bail with what we have
+	    return;
+	}
 
-		my($tag, $iid) = ($root_oid =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
-		$table_root_oid = [[$tag, $iid]];
+	$varbinds = [];
+	my $newstopconds;
 
-	};
+	my $lastsetstart = ($repeatcount-1) * ($#$stopconds+1);
 
+	for (my $i = 0; $i <= $#$vbl; $i++) {
+	    my $row_oid = SNMP::translateObj($vbl->[$i][0]);
+	    my $row_text = $vbl->[$i][0];
+	    my $row_index = $vbl->[$i][1];
+	    my $row_value = $vbl->[$i][2];
+	    my $row_type = $vbl->[$i][3];
 
-	# Translate the supplied OID in to numeric format for parsing
+	    if ($row_oid =~ /^$stopconds->[$i % ($#$stopconds+1)]/) {
 
-	$table_root_oid = SNMP::translateObj(@{@{$table_root_oid}[0]}[0]);
-
-
-	my %result_hash;
-	my $end_of_table = 0;
-
-
-	my $table_varbind = new SNMP::Varbind([$table_root_oid]);
-
-	while($this->getnext($table_varbind) && !$this->{ErrorNum}) {
-
-		my $row_oid = SNMP::translateObj(@{$table_varbind}[0]);
-		my $row_text = @{$table_varbind}[0];
-		my $row_index = @{$table_varbind}[1];
-		my $row_value = @{$table_varbind}[2];
-		my $row_type = @{$table_varbind}[3];
-
-		if($row_oid =~ m/($table_root_oid)/) {
-
-  		        if ($row_type eq "OBJECTID") {
+		if ($row_type eq "OBJECTID") {
 
 				# If the value returned is an OID, translate this
 				# back in to a textual OID
 
-				$row_value = SNMP::translateObj($row_value);
+		    $row_value = SNMP::translateObj($row_value);
 
-			};
+		}
 
+		# Place the results in a hash
 
-			# Place the results in a hash
+		$result_hash{$row_index}{$row_text} = $row_value;
 
-			$result_hash{$row_index}->{$row_text} = $row_value;
+		# continue past this next time
+		if ($i >= $lastsetstart) {
+		    push @$newstopconds, $stopconds->[$i%($#$stopconds+1)];
+		    push @$varbinds,[$vbl->[$i][0],$vbl->[$i][1]];
+		}
+	    }
+	}
+	if ($#$newstopconds == -1) {
+	    last;
+	}
+	if ($#$varbinds == -1) {
+	    print "gettable ack.  shouldn't get here\n";
+	}
+	$vbl = $varbinds;
+	$stopconds = $newstopconds;
 
-		} else {
+	if ($this->{Version} > 1 && !$options->{'nogetbulk'}) {
+	    $res = $this->getbulk(0, $repeatcount, $vbl);
+	} else {
+	    $res = $this->getnext($vbl);
+	}
+    }
 
-			last;
+    # calculate indexes
+    if ($parse_indexes) {
+	my @indexes = @{$SNMP::MIB{$textnode}{'children'}[0]{'indexes'}};
+	my $i;
+	foreach my $trow (keys(%result_hash)) {
+	    my $noid = new NetSNMP::OID($columns[0] . "." . $trow);
+	    if (!$noid) {
+		print STDERR "***** ERROR parsing $columns[0].$trow MIB OID\n";
+		next;
+	    }
+	    my $nindexes = $noid->get_indexes();
+	    if (!$nindexes || ref($nindexes) ne 'ARRAY' ||
+		$#indexes != $#$nindexes) {
+		print STDERR "***** ERROR parsing $columns[0].$trow MIB indexes: $noid => " . ref($nindexes) . " [should be an ARRAY], expended # indexes = $#indexes\n";
+		if (ref($nindexes) eq 'ARRAY') {
+		    print STDERR "***** ERROR parsing $columns[0].$trow MIB indexes: " . ref($nindexes) . " $#indexes $#$nindexes\n";
+		}
+		next;
+	    }
 
-		};
-
-	};
-
-	return(\%result_hash);
+	    for ($i = 0; $i <= $#indexes; $i++) {
+		$result_hash{$trow}{$indexes[$i]} = $nindexes->[$i];
+	    }
+	}
+    }
 
+    return(\%result_hash);
 }
 
 sub fget {
@@ -717,7 +793,9 @@ sub fget {
      $varbind_list_ref = [$vars];
      $varbind_list_ref = $vars if ref($$vars[0]) =~ /ARRAY/;
    } else {
-     $varbind_list_ref = [SNMP::split_vars($vars)];
+     # my ($tag, $iid) = ($vars =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
+     my ($tag, $iid) = ($vars =~ /^(.*?)\.?(\d+)+$/);
+     $varbind_list_ref = [[$tag, $iid]];
    }
 
    my $cb = shift;
@@ -747,7 +825,9 @@ sub getnext {
      $varbind_list_ref = [$vars];
      $varbind_list_ref = $vars if ref($$vars[0]) =~ /ARRAY/;
    } else {
-     $varbind_list_ref = [SNMP::split_vars($vars)];
+     # my ($tag, $iid) = ($vars =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
+     my ($tag, $iid) = ($vars =~ /^(.*?)\.?(\d+)+$/);
+     $varbind_list_ref = [[$tag, $iid]];
    }
 
    my $cb = shift;
@@ -770,7 +850,9 @@ sub fgetnext {
      $varbind_list_ref = [$vars];
      $varbind_list_ref = $vars if ref($$vars[0]) =~ /ARRAY/;
    } else {
-     $varbind_list_ref = [SNMP::split_vars($vars)];
+     # my ($tag, $iid) = ($vars =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
+     my ($tag, $iid) = ($vars =~ /^(.*?)\.?(\d+)+$/);
+     $varbind_list_ref = [[$tag, $iid]];
    }
 
    my $cb = shift;
@@ -802,7 +884,9 @@ sub getbulk {
      $varbind_list_ref = [$vars];
      $varbind_list_ref = $vars if ref($$vars[0]) =~ /ARRAY/;
    } else {
-     $varbind_list_ref = [SNMP::split_vars($vars)];
+     # my ($tag, $iid) = ($vars =~ /^((?:\.\d+)+|(?:\w+(?:\-*\w+)+))\.?(.*)$/);
+     my ($tag, $iid) = ($vars =~ /^(.*?)\.?(\d+)+$/);
+     $varbind_list_ref = [[$tag, $iid]];
    }
 
    my $cb = shift;
@@ -1192,6 +1276,16 @@ sub DELETE {
     ${$_[0]} = 0;
 }
 
+package SNMP::MIB::MIB_OPTIONS;
+
+sub TIESCALAR { my $class = shift; my $val; bless \$val, $class; }
+
+sub FETCH { ${$_[0]}; }
+
+sub STORE { SNMP::_mib_toggle_options($_[1]); ${$_[0]} = $_[1]; }
+
+sub DELETE { SNMP::_mib_toggle_options(0); ${$_[0]} = ''; }
+
 package SNMP;
 END{SNMP::_sock_cleanup() if defined &SNMP::_sock_cleanup;}
 # Autoload methods go after __END__, and are processed by the autosplit prog.
@@ -1201,7 +1295,7 @@ __END__
 
 =head1 NAME
 
-SNMP - The Perl5 'SNMP' Extension Module v3.1.0 for the UCD SNMPv3 Library
+SNMP - The Perl5 'SNMP' Extension Module for the Net-SNMP SNMP package.
 
 =head1 SYNOPSIS
 
@@ -1321,12 +1415,23 @@ default <none>, authentication passphrase
 
 =item PrivProto
 
-default 'DES', privacy protocol [DES] (v3)
+default 'DES', privacy protocol [DES, AES] (v3)
 
 =item PrivPass
 
 default <none>, privacy passphrase (v3)
 
+=item authMasterKey
+
+=item privMasterKey
+
+=item authLocalizedKey
+
+=item privLocalizedKey
+
+Directly specified SNMPv3 USM user keys (used if you want to specify
+the keys instead of deriving them from a password as above).
+
 =item VarFormats
 
 default 'undef', used by 'fget[next]', holds an hash
@@ -1520,6 +1625,82 @@ agent to include one additional (unrelated) variable that signals
 the end of the sub-tree, allowing bulkwalk() to determine that
 the request is complete.
 
+=item $results = $sess->gettable(E<lt>TABLE OIDE<gt>, E<lt>OPTIONS<gt>)
+
+This will retrieve an entire table of data and return a hash reference
+to that data.  The returned hash reference will have indexes of the
+OID suffixes for the index data as the key.  The value for each entry
+will be another hash containing the data for a given row.  The keys to
+that hash will be the column names, and the values will be the data.
+
+Example:
+
+  #!/usr/bin/perl
+
+  use SNMP;
+  use Data::Dumper;
+
+  my $s = new SNMP::Session(DestHost => 'localhost');
+
+  print Dumper($s->gettable('ifTable'));
+
+On my machine produces:
+
+  $VAR1 = {
+            '6' => {
+                     'ifMtu' => '1500',
+                     'ifPhysAddress' => 'PV',
+                     # ...
+                     'ifInUnknownProtos' => '0'
+                   },
+            '4' => {
+                     'ifMtu' => '1480',
+                     'ifPhysAddress' => '',
+                     # ...
+                     'ifInUnknownProtos' => '0'
+                   },
+            # ...
+           };
+
+By default, it will try to do as optimized retrieval as possible.
+It'll request multiple columns at once, and use GETBULK if possible.
+A few options may be specified by passing in an I<OPTIONS> hash
+containing various parameters:
+
+=over
+
+=item noindexes => 1
+
+Instructs the code not to parse the indexes and place the results in
+the second hash.  If you don't need the index data, this will be
+faster.
+
+=item columns => [ colname1, ... ]
+
+This specifies which columns to collect.  By default, it will try to
+collect all the columns defined in the MIB table.
+
+=item repeat => I<COUNT>
+
+Specifies a GETBULK repeat I<COUNT>.  IE, it will request this many
+varbinds back per column when using the GETBULK operation.  Shortening
+this will mean smaller packets which may help going through some
+systems.  By default, this value is calculated and attepmts to guess
+at what will fit all the results into 1000 bytes.  This calculation is
+fairly safe, hopefully, but you can either raise or lower the number
+using this option if desired.  In lossy networks, you want to make
+sure that the packets don't get fragmented and lowering this value is
+one way to help that.
+
+=item nogetbulk => 1
+
+Force the use of GETNEXT rather than GETBULK.  (always true for
+SNMPv1, as it doesn't have GETBULK anyway).  Some agents are great
+implementers of GETBULK and this allows you to force the use of
+GETNEXT oprations instead.
+
+=back
+
 =back
 
 =head1 SNMP::TrapSession
@@ -1904,6 +2085,10 @@ returns 'textualConvention' if defined else 'type'
 
 returns TEXTUAL-CONVENTION
 
+=item TCDescription
+
+returns the TEXTUAL-CONVENTION's DESCRIPTION field.
+
 =item units
 
 returns UNITS
@@ -1925,6 +2110,10 @@ returns array ref of hash ref [{low => num, high => num}, ...]
 returns DESCRIPTION ($SNMP::save_descriptions must
 be set prior to MIB initialization/parsing)
 
+=item reference
+
+returns the REFERENCE clause
+
 =back
 
 =head1 MIB Functions
