@@ -28,7 +28,7 @@ _factory_free(container_type *data, void *context)
     if (data->name != NULL) {
         DEBUGMSGTL(("container", "  _factory_free_list() called for %s\n",
                     data->name));
-	free((const void*)data->name); /* SNMP_FREE wasted on object about to be freed */
+	free((void *)data->name); /* SNMP_FREE wasted on object about to be freed */
     }
     free(data); /* SNMP_FREE wasted on param */
 }
@@ -129,17 +129,18 @@ netsnmp_container_find_factory(const char *type_list)
 {
     netsnmp_factory   *f = NULL;
     char              *list, *entry;
+    char              *st;
 
     if (NULL==type_list)
         return NULL;
 
     list = strdup(type_list);
-    entry = strtok(list, ":");
+    entry = strtok_r(list, ":", &st);
     while(entry) {
         f = netsnmp_container_get_factory(entry);
         if (NULL != f)
             break;
-        entry = strtok(NULL, ":");
+        entry = strtok_r(NULL, ":", &st);
     }
 
     free(list);
@@ -164,10 +165,15 @@ netsnmp_container *
 netsnmp_container_find(const char *type)
 {
     netsnmp_factory *f = netsnmp_container_find_factory(type);
-    if (f)
-        return f->produce();
+    netsnmp_container *c = f ? f->produce() : NULL;
 
-    return NULL;
+    /*
+     * provide default compare
+     */
+    if (c && (NULL == c->compare))
+        c->compare = netsnmp_compare_netsnmp_index;
+
+    return c;
 }
 
 /*------------------------------------------------------------------
@@ -251,6 +257,8 @@ int CONTAINER_FREE(netsnmp_container *x)
     while(x) {
         netsnmp_container *tmp;
         tmp = x->prev;
+        if (NULL != x->container_name)
+            SNMP_FREE(x->container_name);
         rc2 = x->cfree(x);
         if (rc2) {
             snmp_log(LOG_ERR,"error on subcontainer cfree (%d)\n", rc2);
@@ -260,6 +268,52 @@ int CONTAINER_FREE(netsnmp_container *x)
     }
     return rc;
 }
+
+/*------------------------------------------------------------------
+ * These functions should EXACTLY match the function version in
+ * container.c. If you change one, change them both.
+ */
+/*
+ * clear all containers. When clearing the *first* container, and
+ * *only* the first container, call the function f for each item.
+ * After calling this function, all containers should be empty.
+ */
+void CONTAINER_CLEAR(netsnmp_container *x, netsnmp_container_obj_func *f,
+                    void *c)
+{
+    /** start at last container */
+    while(x->next)
+        x = x->next;
+    while(x->prev) {
+        x->clear(x, NULL, c);
+        x = x->prev;
+    }
+    x->clear(x, f, c);
+}
+
+/*------------------------------------------------------------------
+ * These functions should EXACTLY match the function version in
+ * container.c. If you change one, change them both.
+ */
+/*
+ * Find a sub-container with the given name
+ */
+netsnmp_container *SUBCONTAINER_FIND(netsnmp_container *x,
+                                     const char* name)
+{
+    if ((NULL == x) || (NULL == name))
+        return NULL;
+    
+    /** start at first container */
+    while(x->prev)
+        x = x->prev;
+    while(x) {
+        if ((NULL != x->container_name) && (0 == strcmp(name,x->container_name)))
+            break;
+        x = x->next;
+    }
+    return x;
+}
 #endif
 
 
@@ -375,3 +429,21 @@ netsnmp_compare_mem(const char * lhs, size_t lhs_len,
 
     return rc;
 }
+
+/*------------------------------------------------------------------
+ * netsnmp_container_simple_free
+ *
+ * useful function to pass to CONTAINER_FOR_EACH, when a simple
+ * free is needed for every item.
+ */
+void 
+netsnmp_container_simple_free(void *data, void *context)
+{
+    if (data == NULL)
+	return;
+    
+    DEBUGMSGTL(("verbose:container",
+                "netsnmp_container_simple_free) called for %p/%p\n",
+                data, context));
+    free((void*)data); /* SNMP_FREE wasted on param */
+}
