@@ -87,12 +87,18 @@
        - #define NETSNMP_DS_LIB_DONT_PRINT_UNITS    29 don't print UNITS suffix
        - #define NETSNMP_DS_LIB_NO_DISPLAY_HINT     30 don't apply DISPLAY-HINTs
        - #define NETSNMP_DS_LIB_16BIT_IDS           31 restrict requestIDs, etc to 16-bit values
+       - #define NETSNMP_DS_LIB_DONT_PERSIST_STATE  32 don't save/load any persistant state
+       - #define NETSNMP_DS_LIB_2DIGIT_HEX_OUTPUT   33 print a leading 0 on hex values <= 'f'
 
 
        library integers
 
        - #define NETSNMP_DS_LIB_MIB_WARNINGS  0
        - #define NETSNMP_DS_LIB_SECLEVEL      1
+       - #define NETSNMP_DS_LIB_SNMPVERSION   2
+       - #define NETSNMP_DS_LIB_DEFAULT_PORT  3
+       - #define NETSNMP_DS_LIB_OID_OUTPUT_FORMAT  4
+       - #define NETSNMP_DS_LIB_STRING_OUTPUT_FORMAT 5
 
        library strings
 
@@ -108,6 +114,14 @@
        - #define NETSNMP_DS_LIB_CONFIGURATION_DIR 9
        - #define NETSNMP_DS_LIB_SECMODEL          10
        - #define NETSNMP_DS_LIB_MIBDIRS           11
+       - #define NETSNMP_DS_LIB_OIDSUFFIX         12
+       - #define NETSNMP_DS_LIB_OIDPREFIX         13
+       - #define NETSNMP_DS_LIB_CLIENT_ADDR       14
+       - #define NETSNMP_DS_LIB_TEMP_FILE_PATTERN 15
+       - #define NETSNMP_DS_LIB_AUTHMASTERKEY     16
+       - #define NETSNMP_DS_LIB_PRIVMASTERKEY     17
+       - #define NETSNMP_DS_LIB_AUTHLOCALIZEDKEY  18
+       - #define NETSNMP_DS_LIB_PRIVLOCALIZEDKEY  19
 
  *  @{
  */
@@ -333,6 +347,7 @@ netsnmp_ds_handle_config(const char *token, char *line)
     char            buf[SNMP_MAXBUF];
     char           *value, *endptr;
     int             itmp;
+    char           *st;
 
     DEBUGMSGTL(("netsnmp_ds_handle_config", "handling %s\n", token));
 
@@ -348,7 +363,7 @@ netsnmp_ds_handle_config(const char *token, char *line)
 
         switch (drsp->type) {
         case ASN_BOOLEAN:
-            value = strtok(line, " \t\n");
+            value = strtok_r(line, " \t\n", &st);
             if (strcasecmp(value, "yes") == 0 || 
 		strcasecmp(value, "true") == 0) {
                 itmp = 1;
@@ -366,7 +381,7 @@ netsnmp_ds_handle_config(const char *token, char *line)
             break;
 
         case ASN_INTEGER:
-            value = strtok(line, " \t\n");
+            value = strtok_r(line, " \t\n", &st);
             itmp = strtol(value, &endptr, 10);
             if (*endptr != 0) {
                 config_perror("Bad integer value");
