@@ -199,8 +199,9 @@ register_config_handler(const char *type_param,
                         void (*releaser) (void), const char *help)
 {
     struct config_files **ctmp = &config_files;
-    struct config_line **ltmp;
+    struct config_line **ltmp, *ltmp2;
     const char     *type = type_param;
+    char           *cptr, buf[STRINGMAX];
     char           *st;
 
     if (type == NULL) {
@@ -208,6 +209,22 @@ register_config_handler(const char *type_param,
 				     NETSNMP_DS_LIB_APPTYPE);
     }
 
+    /*
+     * Handle multiple types (recursively)
+     */
+    cptr = strchr( type, ':' );
+    if (cptr) {
+        strncpy(buf, type, STRINGMAX);
+        buf[STRINGMAX - 1] = '\0';
+        ltmp2 = NULL;
+        cptr = strtok_r(buf, ":", &st);
+        while (cptr) {
+            ltmp2 = register_config_handler(cptr, token, parser, releaser, help);
+            cptr  = strtok_r(NULL, ":", &st);
+        }
+        return ltmp2;
+    }
+    
     /*
      * Find type in current list  -OR-  create a new file type.
      */
@@ -286,6 +303,7 @@ unregister_config_handler(const char *type_param, const char *token)
     struct config_files **ctmp = &config_files;
     struct config_line **ltmp, *ltmp2;
     const char     *type = type_param;
+    char           *cptr, buf[STRINGMAX];
     char           *st;
 
     if (type == NULL) {
@@ -293,6 +311,21 @@ unregister_config_handler(const char *type_param, const char *token)
 				     NETSNMP_DS_LIB_APPTYPE);
     }
 
+    /*
+     * Handle multiple types (recursively)
+     */
+    cptr = strchr( type, ':' );
+    if (cptr) {
+        strncpy(buf, type, STRINGMAX);
+        buf[STRINGMAX - 1] = '\0';
+        cptr = strtok_r(buf, ":", &st);
+        while (cptr) {
+            unregister_config_handler(cptr, token);
+            cptr  = strtok_r(NULL, ":", &st);
+        }
+        return;
+    }
+    
     /*
      * find type in current list 
      */
@@ -399,23 +432,17 @@ read_config_get_handlers(const char *type)
 }
 
 void
-read_config_with_type_when(const char *filename, const char *type, int when)
+read_config_with_type(const char *filename, const char *type)
 {
     struct config_line *ctmp = read_config_get_handlers(type);
     if (ctmp)
-        read_config(filename, ctmp, when);
+        read_config(filename, ctmp, EITHER_CONFIG);
     else
         DEBUGMSGTL(("read_config",
                     "read_config: I have no registrations for type:%s,file:%s\n",
                     type, filename));
 }
 
-void
-read_config_with_type(const char *filename, const char *type)
-{
-    read_config_with_type_when(filename, type, EITHER_CONFIG);
-}
-
 
 struct config_line *
 read_config_find_handler(struct config_line *line_handlers,
@@ -469,7 +496,7 @@ run_config_handler(struct config_line *lptr,
 
 /*
  * takens an arbitrary string and tries to intepret it based on the
- * known configruation handlers for all registered types.  May produce
+ * known configuration handlers for all registered types.  May produce
  * inconsistent results when multiple tokens of the same name are
  * registered under different file types. 
  */
@@ -483,9 +510,9 @@ int
 snmp_config_when(char *line, int when)
 {
     char           *cptr, buf[STRINGMAX], tmpbuf[STRINGMAX];
-    char           *st;
     struct config_line *lptr = NULL;
     struct config_files *ctmp = config_files;
+    char           *st;
 
     if (line == NULL) {
         config_perror("snmp_config() called with a null string.");
@@ -768,43 +795,14 @@ free_config(void)
                 (*(ltmp->free_func)) ();
 }
 
-void
-read_configs_optional(const char *optional_config, int when)
-{
-    char *newp, *cp, *st;
-    char *type = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
-				       NETSNMP_DS_LIB_APPTYPE);
-
-    if ((NULL == optional_config) || (NULL == type))
-        return;
-
-    DEBUGMSGTL(("read_configs_optional",
-                "reading optional configuration tokens for %s\n", type));
-    
-    newp = strdup(optional_config);      /* strtok_r messes it up */
-    cp = strtok_r(newp, ",", &st);
-    while (cp) {
-        struct stat     statbuf;
-        if (stat(cp, &statbuf)) {
-            DEBUGMSGTL(("read_config",
-                        "Optional File \"%s\" does not exist.\n", cp));
-            snmp_log_perror(cp);
-        } else {
-            DEBUGMSGTL(("read_config",
-                        "Reading optional config file: \"%s\"\n", cp));
-            read_config_with_type_when(cp, type, when);
-        }
-        cp = strtok_r(NULL, ",", &st);
-    }
-    free(newp);
-    
-}
-
 void
 read_configs(void)
 {
     char *optional_config = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
 					       NETSNMP_DS_LIB_OPTIONALCONFIG);
+    char *type = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
+				       NETSNMP_DS_LIB_APPTYPE);
+    char *st;
 
     DEBUGMSGTL(("read_config", "reading normal configuration tokens\n"));
 
@@ -816,8 +814,27 @@ read_configs(void)
     /*
      * do this even when the normal above wasn't done 
      */
-    if (NULL != optional_config)
-        read_configs_optional(optional_config, NORMAL_CONFIG);
+    if (optional_config && type) {
+      char           *newp, *cp;
+      newp = strdup(optional_config);      /* strtok_r messes it up */
+      cp = strtok_r(newp, ",", &st);
+      while (cp) {
+        struct stat     statbuf;
+        if (stat(cp, &statbuf)) {
+            DEBUGMSGTL(("read_config",
+                        "Optional File \"%s\" does not exist.\n",
+                        cp));
+            snmp_log_perror(cp);
+        } else {
+            DEBUGMSGTL(("read_config",
+                        "Reading optional config file: \"%s\"\n",
+                        cp));
+            read_config_with_type(cp, type);
+        }
+        cp = strtok_r(NULL, ",", &st);
+      }
+      free(newp);
+    }
 
     netsnmp_config_process_memories_when(NORMAL_CONFIG, 1);
 
@@ -830,9 +847,6 @@ read_configs(void)
 void
 read_premib_configs(void)
 {
-    char *optional_config = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
-					       NETSNMP_DS_LIB_OPTIONALCONFIG);
-
     DEBUGMSGTL(("read_config", "reading premib configuration tokens\n"));
 
     if (!netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID, 
@@ -840,9 +854,6 @@ read_premib_configs(void)
         read_config_files(PREMIB_CONFIG);
     }
 
-    if (NULL != optional_config)
-        read_configs_optional(optional_config, PREMIB_CONFIG);
-
     netsnmp_config_process_memories_when(PREMIB_CONFIG, 0);
 
     netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID, 
@@ -886,7 +897,7 @@ get_configuration_directory()
 
     if (NULL == netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
 				      NETSNMP_DS_LIB_CONFIGURATION_DIR)) {
-        homepath = getenv("HOME");
+        homepath = netsnmp_getenv("HOME");
         snprintf(defaultPath, sizeof(defaultPath), "%s%c%s%c%s%s%s%s",
                 SNMPCONFPATH, ENV_SEPARATOR_CHAR,
                 SNMPSHAREPATH, ENV_SEPARATOR_CHAR, SNMPLIBPATH,
@@ -1017,13 +1028,15 @@ read_config_files(int when)
     struct config_line *ltmp;
     struct stat     statbuf;
 
+    if (netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
+                               NETSNMP_DS_LIB_DONT_PERSIST_STATE)) return;
+
     config_errors = 0;
 
     if (when == PREMIB_CONFIG)
         free_config();
 
     confpath = get_configuration_directory();
-    perspath = get_persistent_directory();
 
     /*
      * read all config file types 
@@ -1032,10 +1045,16 @@ read_config_files(int when)
 
         ltmp = ctmp->start;
 
+        /*
+         * persistent path can change via conf file, so make sure
+         * we have a valid pointer.
+         */
+        perspath = get_persistent_directory();
+
         /*
          * read the config files 
          */
-        if ((envconfpath = getenv("SNMPCONFPATH")) == NULL) {
+        if ((envconfpath = netsnmp_getenv("SNMPCONFPATH")) == NULL) {
             snprintf(defaultPath, sizeof(defaultPath), "%s%s%s",
                     ((confpath == NULL) ? "" : confpath),
                     ((perspath == NULL) ? "" : ENV_SEPARATOR),
@@ -1063,7 +1082,7 @@ read_config_files(int when)
              * then we read all the configuration files we can, starting with
              * the oldest first.
              */
-	    persfile = getenv("SNMP_PERSISTENT_FILE");
+	    persfile = netsnmp_getenv("SNMP_PERSISTENT_FILE");
             if (strncmp(cptr2, perspath, strlen(perspath)) == 0 ||
                 (persfile != NULL &&
                  strncmp(cptr2, persfile, strlen(persfile)) == 0)) {
@@ -1168,12 +1187,15 @@ read_config_store(const char *type, const char *line)
     mode_t          oldmask;
 #endif
 
+    if (netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
+                               NETSNMP_DS_LIB_DONT_PERSIST_STATE)) return;
+
     /*
      * store configuration directives in the following order of preference:
      * 1. ENV variable SNMP_PERSISTENT_FILE
      * 2. configured <PERSISTENT_DIRECTORY>/<type>.conf
      */
-    if ((filep = getenv("SNMP_PERSISTENT_FILE")) == NULL) {
+    if ((filep = netsnmp_getenv("SNMP_PERSISTENT_FILE")) == NULL) {
         snprintf(file, sizeof(file),
                  "%s/%s.conf", get_persistent_directory(), type);
         file[ sizeof(file)-1 ] = 0;
@@ -1239,6 +1261,9 @@ snmp_save_persistent(const char *type)
     struct stat     statbuf;
     int             j;
 
+    if (netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
+                               NETSNMP_DS_LIB_DONT_PERSIST_STATE)) return;
+
     DEBUGMSGTL(("snmp_save_persistent", "saving %s files...\n", type));
     snprintf(file, sizeof(file),
              "%s/%s.conf", get_persistent_directory(), type);
@@ -1298,6 +1323,9 @@ snmp_clean_persistent(const char *type)
     struct stat     statbuf;
     int             j;
 
+    if (netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
+                               NETSNMP_DS_LIB_DONT_PERSIST_STATE)) return;
+
     DEBUGMSGTL(("snmp_clean_persistent", "cleaning %s files...\n", type));
     snprintf(file, sizeof(file),
              "%s/%s.conf", get_persistent_directory(), type);
@@ -1394,7 +1422,7 @@ char           *
 copy_nword(char *from, char *to, int len)
 {
     char            quote;
-    if ( !from || !to )
+    if (!from || !to)
         return NULL;
     if ((*from == '\"') || (*from == '\'')) {
         quote = *(from++);
@@ -1518,6 +1546,7 @@ read_config_read_octet_string(char *readfrom, u_char ** str, size_t * len)
     char           *cptr1;
     u_int           tmp;
     int             i;
+    size_t          ilen;
 
     if (readfrom == NULL || str == NULL)
         return NULL;
@@ -1529,29 +1558,32 @@ read_config_read_octet_string(char *readfrom, u_char ** str, size_t * len)
         readfrom += 2;
         cptr1 = skip_not_white(readfrom);
         if (cptr1)
-            *len = (cptr1 - readfrom);
+            ilen = (cptr1 - readfrom);
         else
-            *len = strlen(readfrom);
+            ilen = strlen(readfrom);
 
-        if (*len % 2) {
+        if (ilen % 2) {
             snmp_log(LOG_WARNING,"invalid hex string: wrong length\n");
             DEBUGMSGTL(("read_config_read_octet_string",
                         "invalid hex string: wrong length"));
             return NULL;
         }
-        *len = *len / 2;
+        ilen = ilen / 2;
 
         /*
          * malloc data space if needed (+1 for good measure) 
          */
         if (*str == NULL) {
-            if ((cptr = (u_char *) malloc(*len + 1)) == NULL) {
+            if ((cptr = (u_char *) malloc(ilen + 1)) == NULL) {
                 return NULL;
             }
             *str = cptr;
         } else {
+            if (ilen >= *len)
+                ilen = *len-1;
             cptr = *str;
         }
+        *len = ilen;
 
         /*
          * copy validated data 
@@ -1742,6 +1774,7 @@ read_config_read_memory(int type, char *readfrom,
 {
     int            *intp;
     unsigned int   *uintp;
+    char            buf[SPRINT_MAX_LEN];
 
     if (!dataptr || !readfrom)
         return NULL;
@@ -1751,9 +1784,9 @@ read_config_read_memory(int type, char *readfrom,
         if (*len < sizeof(int))
             return NULL;
         intp = (int *) dataptr;
-        *intp = atoi(readfrom);
+        readfrom = copy_nword(readfrom, buf, sizeof(buf));
+        *intp = atoi(buf);
         *len = sizeof(int);
-        readfrom = skip_token(readfrom);
         return readfrom;
 
     case ASN_TIMETICKS:
@@ -1761,9 +1794,9 @@ read_config_read_memory(int type, char *readfrom,
         if (*len < sizeof(unsigned int))
             return NULL;
         uintp = (unsigned int *) dataptr;
-        *uintp = strtoul(readfrom, NULL, 0);
+        readfrom = copy_nword(readfrom, buf, sizeof(buf));
+        *uintp = strtoul(buf, NULL, 0);
         *len = sizeof(unsigned int);
-        readfrom = skip_token(readfrom);
         return readfrom;
 
     case ASN_OCTET_STR:
