@@ -75,6 +75,7 @@
 #include <openssl/aes.h>
 #endif
 
+#ifndef DISABLE_DES
 #ifdef STRUCT_DES_KS_STRUCT_HAS_WEAK_KEY
 /* these are older names for newer structures that exist in openssl .9.7 */
 #define DES_key_schedule    des_key_schedule 
@@ -84,6 +85,7 @@
 #define DES_cbc_encrypt    des_cbc_encrypt
 #define OLD_DES
 #endif
+#endif
 
 #endif /* HAVE_OPENSSL */
 
@@ -116,14 +118,34 @@ sc_get_properlength(const oid * hashtype, u_int hashtype_len)
     /*
      * Determine transform type hash length.
      */
+#ifndef DISABLE_MD5
     if (ISTRANSFORM(hashtype, HMACMD5Auth)) {
         return BYTESIZE(SNMP_TRANS_AUTHLEN_HMACMD5);
-    } else if (ISTRANSFORM(hashtype, HMACSHA1Auth)) {
+    } else
+#endif
+        if (ISTRANSFORM(hashtype, HMACSHA1Auth)) {
         return BYTESIZE(SNMP_TRANS_AUTHLEN_HMACSHA1);
     }
     return SNMPERR_GENERR;
 }
 
+int
+sc_get_proper_priv_length(const oid * privtype, u_int privtype_len)
+{
+    int properlength = 0;
+#ifndef DISABLE_DES
+    if (ISTRANSFORM(privtype, DESPriv)) {
+        properlength = BYTESIZE(SNMP_TRANS_PRIVLEN_1DES);
+    }
+#endif
+#ifdef HAVE_AES
+    if (ISTRANSFORM(privtype, AESPriv)) {
+        properlength = BYTESIZE(SNMP_TRANS_PRIVLEN_AES);
+    }
+#endif
+    return properlength;
+}
+
 
 /*******************************************************************-o-******
  * sc_init
@@ -286,9 +308,12 @@ sc_generate_keyed_hash(const oid * authtype, size_t authtypelen,
     /*
      * Determine transform type.
      */
+#ifndef DISABLE_MD5
     if (ISTRANSFORM(authtype, HMACMD5Auth))
         HMAC(EVP_md5(), key, keylen, message, msglen, buf, &buf_len);
-    else if (ISTRANSFORM(authtype, HMACSHA1Auth))
+    else
+#endif
+        if (ISTRANSFORM(authtype, HMACSHA1Auth))
         HMAC(EVP_sha1(), key, keylen, message, msglen, buf, &buf_len);
     else {
         QUITFUN(SNMPERR_GENERR, sc_generate_keyed_hash_quit);
@@ -302,12 +327,15 @@ sc_generate_keyed_hash(const oid * authtype, size_t authtypelen,
 
 #elif USE_PKCS                    /* USE_PKCS */
 
+#ifndef DISABLE_MD5
     if (ISTRANSFORM(authtype, HMACMD5Auth)) {
 	if (pkcs_sign(CKM_MD5_HMAC,key, keylen, message,
 			msglen, buf, &buf_len) != SNMPERR_SUCCESS) {
             QUITFUN(SNMPERR_GENERR, sc_generate_keyed_hash_quit);
         }
-    } else if (ISTRANSFORM(authtype, HMACSHA1Auth)) {
+    } else
+#endif
+        if (ISTRANSFORM(authtype, HMACSHA1Auth)) {
 	if (pkcs_sign(CKM_SHA_1_HMAC,key, keylen, message,
 			msglen, buf, &buf_len) != SNMPERR_SUCCESS) {
             QUITFUN(SNMPERR_GENERR, sc_generate_keyed_hash_quit);
@@ -381,7 +409,6 @@ sc_hash(const oid * hashtype, size_t hashtypelen, u_char * buf,
 #ifdef USE_OPENSSL
     const EVP_MD         *hashfn;
     EVP_MD_CTX     ctx, *cptr;
-    unsigned int   tmp_len;
 #endif
 
     DEBUGTRACE;
@@ -395,9 +422,12 @@ sc_hash(const oid * hashtype, size_t hashtypelen, u_char * buf,
     /*
      * Determine transform type.
      */
+#ifndef DISABLE_MD5
     if (ISTRANSFORM(hashtype, HMACMD5Auth)) {
         hashfn = (const EVP_MD *) EVP_md5();
-    } else if (ISTRANSFORM(hashtype, HMACSHA1Auth)) {
+    } else
+#endif
+        if (ISTRANSFORM(hashtype, HMACSHA1Auth)) {
         hashfn = (const EVP_MD *) EVP_sha1();
     } else {
         return (SNMPERR_GENERR);
@@ -427,25 +457,25 @@ sc_hash(const oid * hashtype, size_t hashtypelen, u_char * buf,
 
 /** do the final pass */
 #if defined(OLD_DES)
-    EVP_DigestFinal(cptr, MAC, &tmp_len);
-    *MAC_len = tmp_len;
+    EVP_DigestFinal(cptr, MAC, MAC_len);
 #else /* !OLD_DES */
     if (SSLeay() < 0x907000) {
-        EVP_DigestFinal(cptr, MAC, &tmp_len);
-        *MAC_len = tmp_len;
+        EVP_DigestFinal(cptr, MAC, MAC_len);
         free(cptr);
     } else {
-        EVP_DigestFinal_ex(cptr, MAC, &tmp_len);
-        *MAC_len = tmp_len;
+        EVP_DigestFinal_ex(cptr, MAC, MAC_len);
         EVP_MD_CTX_cleanup(cptr);
     }
 #endif                          /* OLD_DES */
     return (rval);
 #elif USE_PKCS                  /* USE_PKCS */
 
+#ifndef DISABLE_MD5
     if (ISTRANSFORM(hashtype, HMACMD5Auth)) {
 	rval = pkcs_digest(CKM_MD5, buf, buf_len, MAC, MAC_len);
-    } else if (ISTRANSFORM(hashtype, HMACSHA1Auth)) {
+    } else
+#endif
+        if (ISTRANSFORM(hashtype, HMACSHA1Auth)) {
 	rval = pkcs_digest(CKM_SHA_1, buf, buf_len, MAC, MAC_len);
     } else {
         return (SNMPERR_GENERR);
@@ -586,10 +616,12 @@ sc_encrypt(const oid * privtype, size_t privtypelen,
 #if defined(USE_OPENSSL)
 {
     int             rval = SNMPERR_SUCCESS;
-    u_int           properlength, properlength_iv;
+    u_int           properlength = 0, properlength_iv = 0;
     u_char          pad_block[128];      /* bigger than anything I need */
     u_char          my_iv[128];  /* ditto */
     int             pad, plast, pad_size = 0;
+    int             have_trans;
+#ifndef DISABLE_DES
 #ifdef OLD_DES
     DES_key_schedule key_sch;
 #else
@@ -597,6 +629,7 @@ sc_encrypt(const oid * privtype, size_t privtypelen,
     DES_key_schedule *key_sch = &key_sched_store;
 #endif
     DES_cblock       key_struct;
+#endif
 #ifdef HAVE_AES
     AES_KEY aes_key;
     int new_ivlen = 0;
@@ -658,22 +691,23 @@ sc_encrypt(const oid * privtype, size_t privtypelen,
     /*
      * Determine privacy transform.
      */
+    have_trans = 0;
+#ifndef DISABLE_DES
     if (ISTRANSFORM(privtype, DESPriv)) {
         properlength = BYTESIZE(SNMP_TRANS_PRIVLEN_1DES);
         properlength_iv = BYTESIZE(SNMP_TRANS_PRIVLEN_1DES_IV);
         pad_size = properlength;
-#ifdef HAVE_AES
-    } else if (ISTRANSFORM(privtype, AES128Priv)) {
-        properlength = BYTESIZE(SNMP_TRANS_PRIVLEN_AES128);
-        properlength_iv = BYTESIZE(SNMP_TRANS_PRIVLEN_AES128_IV);
-    } else if (ISTRANSFORM(privtype, AES192Priv)) {
-        properlength = BYTESIZE(SNMP_TRANS_PRIVLEN_AES192);
-        properlength_iv = BYTESIZE(SNMP_TRANS_PRIVLEN_AES192_IV);
-    } else if (ISTRANSFORM(privtype, AES256Priv)) {
-        properlength = BYTESIZE(SNMP_TRANS_PRIVLEN_AES256);
-        properlength_iv = BYTESIZE(SNMP_TRANS_PRIVLEN_AES256_IV);
+        have_trans = 1;
+    }
 #endif
-    } else {
+#ifdef HAVE_AES
+    if (ISTRANSFORM(privtype, AESPriv)) {
+        properlength = BYTESIZE(SNMP_TRANS_PRIVLEN_AES);
+        properlength_iv = BYTESIZE(SNMP_TRANS_PRIVLEN_AES_IV);
+        have_trans = 1;
+    }
+#endif
+    if (!have_trans) {
         QUITFUN(SNMPERR_GENERR, sc_encrypt_quit);
     }
 
@@ -683,6 +717,7 @@ sc_encrypt(const oid * privtype, size_t privtypelen,
 
     memset(my_iv, 0, sizeof(my_iv));
 
+#ifndef DISABLE_DES
     if (ISTRANSFORM(privtype, DESPriv)) {
 
         /*
@@ -720,10 +755,9 @@ sc_encrypt(const oid * privtype, size_t privtypelen,
             *ctlen = plast;
         }
     }
+#endif
 #ifdef HAVE_AES
-    else if (ISTRANSFORM(privtype, AES128Priv) ||
-             ISTRANSFORM(privtype, AES192Priv) ||
-             ISTRANSFORM(privtype, AES256Priv)) {
+    if (ISTRANSFORM(privtype, AESPriv)) {
         (void) AES_set_encrypt_key(key, properlength*8, &aes_key);
 
         memcpy(my_iv, iv, ivlen);
@@ -741,12 +775,14 @@ sc_encrypt(const oid * privtype, size_t privtypelen,
      */
     memset(my_iv, 0, sizeof(my_iv));
     memset(pad_block, 0, sizeof(pad_block));
+#ifndef DISABLE_DES
     memset(key_struct, 0, sizeof(key_struct));
 #ifdef OLD_DES
     memset(&key_sch, 0, sizeof(key_sch));
 #else
     memset(&key_sched_store, 0, sizeof(key_sched_store));
 #endif
+#endif
 #ifdef HAVE_AES
     memset(&aes_key,0,sizeof(aes_key));
 #endif
@@ -855,6 +891,7 @@ sc_decrypt(const oid * privtype, size_t privtypelen,
 
     int             rval = SNMPERR_SUCCESS;
     u_char          my_iv[128];
+#ifndef DISABLE_DES
 #ifdef OLD_DES
     DES_key_schedule key_sch;
 #else
@@ -862,7 +899,9 @@ sc_decrypt(const oid * privtype, size_t privtypelen,
     DES_key_schedule *key_sch = &key_sched_store;
 #endif
     DES_cblock      key_struct;
-    u_int           properlength, properlength_iv;
+#endif
+    u_int           properlength = 0, properlength_iv = 0;
+    int             have_transform;
 #ifdef HAVE_AES
     int new_ivlen = 0;
     AES_KEY aes_key;
@@ -905,21 +944,22 @@ sc_decrypt(const oid * privtype, size_t privtypelen,
     /*
      * Determine privacy transform.
      */
+    have_transform = 0;
+#ifndef DISABLE_DES
     if (ISTRANSFORM(privtype, DESPriv)) {
         properlength = BYTESIZE(SNMP_TRANS_PRIVLEN_1DES);
         properlength_iv = BYTESIZE(SNMP_TRANS_PRIVLEN_1DES_IV);
-#ifdef HAVE_AES
-    } else if (ISTRANSFORM(privtype, AES128Priv)) {
-        properlength = BYTESIZE(SNMP_TRANS_PRIVLEN_AES128);
-        properlength_iv = BYTESIZE(SNMP_TRANS_PRIVLEN_AES128_IV);
-    } else if (ISTRANSFORM(privtype, AES192Priv)) {
-        properlength = BYTESIZE(SNMP_TRANS_PRIVLEN_AES192);
-        properlength_iv = BYTESIZE(SNMP_TRANS_PRIVLEN_AES192_IV);
-    } else if (ISTRANSFORM(privtype, AES256Priv)) {
-        properlength = BYTESIZE(SNMP_TRANS_PRIVLEN_AES256);
-        properlength_iv = BYTESIZE(SNMP_TRANS_PRIVLEN_AES256_IV);
+        have_transform = 1;
+    }
 #endif
-    } else {
+#ifdef HAVE_AES
+    if (ISTRANSFORM(privtype, AESPriv)) {
+        properlength = BYTESIZE(SNMP_TRANS_PRIVLEN_AES);
+        properlength_iv = BYTESIZE(SNMP_TRANS_PRIVLEN_AES_IV);
+        have_transform = 1;
+    }
+#endif
+    if (!have_transform) {
         QUITFUN(SNMPERR_GENERR, sc_decrypt_quit);
     }
 
@@ -928,6 +968,7 @@ sc_decrypt(const oid * privtype, size_t privtypelen,
     }
 
     memset(my_iv, 0, sizeof(my_iv));
+#ifndef DISABLE_DES
     if (ISTRANSFORM(privtype, DESPriv)) {
         memcpy(key_struct, key, sizeof(key_struct));
         (void) DES_key_sched(&key_struct, key_sch);
@@ -937,10 +978,9 @@ sc_decrypt(const oid * privtype, size_t privtypelen,
                         (DES_cblock *) my_iv, DES_DECRYPT);
         *ptlen = ctlen;
     }
+#endif
 #ifdef HAVE_AES
-    else if (ISTRANSFORM(privtype, AES128Priv) ||
-             ISTRANSFORM(privtype, AES192Priv) ||
-             ISTRANSFORM(privtype, AES256Priv)) {
+    if (ISTRANSFORM(privtype, AESPriv)) {
         (void) AES_set_encrypt_key(key, properlength*8, &aes_key);
 
         memcpy(my_iv, iv, ivlen);
@@ -957,12 +997,14 @@ sc_decrypt(const oid * privtype, size_t privtypelen,
      * exit cond 
      */
   sc_decrypt_quit:
+#ifndef DISABLE_DES
 #ifdef OLD_DES
     memset(&key_sch, 0, sizeof(key_sch));
 #else
     memset(&key_sched_store, 0, sizeof(key_sched_store));
 #endif
     memset(key_struct, 0, sizeof(key_struct));
+#endif
     memset(my_iv, 0, sizeof(my_iv));
     return rval;
 }				/* USE OPEN_SSL */
