@@ -44,7 +44,7 @@
 #include <net-snmp/library/snmpUDPDomain.h>
 #include <net-snmp/library/snmpTCPDomain.h>
 
-oid netsnmp_snmpTCPDomain[8] = { 1, 3, 6, 1, 3, 91, 1, 1 };
+oid netsnmp_snmpTCPDomain[] = { TRANSPORT_DOMAIN_TCP_IP };
 static netsnmp_tdomain tcpDomain;
 
 /*
@@ -68,13 +68,8 @@ netsnmp_tcp_fmtaddr(netsnmp_transport *t, void *data, int len)
     } else {
         char tmp[64];
 
-        /*
-         * Here we just print the IP address of the peer for compatibility
-         * purposes.  It would be nice if we could include the port number and
-         * some indication of the domain (c.f. AAL5PVC).  
-         */
-
-        sprintf(tmp, "%s", inet_ntoa(to->sin_addr));
+        sprintf(tmp, "TCP: [%s]:%hd",
+                inet_ntoa(to->sin_addr), ntohs(to->sin_port));
         return strdup(tmp);
     }
 }
@@ -215,6 +210,15 @@ netsnmp_tcp_accept(netsnmp_transport *t)
             DEBUGMSGTL(("netsnmp_tcp", "couldn't f_getfl of fd %d\n",newsock));
         }
 #endif
+
+        /*
+         * Allow user to override the send and receive buffers. Default is
+         * to use os default.  Don't worry too much about errors --
+         * just plough on regardless.  
+         */
+        netsnmp_sock_buffer_set(newsock, SO_SNDBUF, 1, 0);
+        netsnmp_sock_buffer_set(newsock, SO_RCVBUF, 1, 0);
+
         return newsock;
     } else {
         free(farend);
@@ -328,6 +332,11 @@ netsnmp_tcp_transport(struct sockaddr_in *addr, int local)
             netsnmp_transport_free(t);
             return NULL;
         }
+        
+        /*
+         * no buffer size on listen socket - doesn't make sense
+         */
+
     } else {
         t->remote = malloc(6);
         if (t->remote == NULL) {
@@ -355,6 +364,14 @@ netsnmp_tcp_transport(struct sockaddr_in *addr, int local)
             netsnmp_transport_free(t);
             return NULL;
         }
+
+        /*
+         * Allow user to override the send and receive buffers. Default is
+         * to use os default.  Don't worry too much about errors --
+         * just plough on regardless.  
+         */
+        netsnmp_sock_buffer_set(t->sock, SO_SNDBUF, local, 0);
+        netsnmp_sock_buffer_set(t->sock, SO_RCVBUF, local, 0);
     }
 
     /*
