@@ -62,6 +62,12 @@
 
 static netsnmp_tdomain udpDomain;
 
+/*
+ * not static, since snmpUDPIPv6Domain needs it, but not public, either.
+ * (ie don't put it in a public header.)
+ */
+void _netsnmp_udp_sockopt_set(int fd, int server);
+
 /*
  * Return a string representing the address in data, or else the "far end"
  * address if data is NULL.  
@@ -82,13 +88,8 @@ netsnmp_udp_fmtaddr(netsnmp_transport *t, void *data, int len)
     } else {
 	char tmp[64];
 
-        /*
-         * Here we just print the IP address of the peer for compatibility
-         * purposes.  It would be nice if we could include the port number and
-         * some indication of the domain (c.f. AAL5PVC).  
-         */
-
-        sprintf(tmp, "%s", inet_ntoa(to->sin_addr));
+        sprintf(tmp, "UDP: [%s]:%hd",
+                inet_ntoa(to->sin_addr), ntohs(to->sin_port));
         return strdup(tmp);
     }
 }
@@ -192,7 +193,281 @@ netsnmp_udp_close(netsnmp_transport *t)
     return rc;
 }
 
+/*
+ * find largest possible buffer between current size and specified size.
+ *
+ * Try to maximize the current buffer of type "optname"
+ * to the maximum allowable size by the OS (as close to
+ * size as possible)
+ */
+static int
+_sock_buffer_maximize(int s, int optname, const char *buftype, int size)
+{
+    int            curbuf = 0, curbuflen = sizeof(int);
+    int            lo, mid, hi;
 
+    /*
+     * First we need to determine our current buffer
+     */
+    if ((getsockopt(s, SOL_SOCKET, optname, (void *) &curbuf,
+                    &curbuflen) == 0) 
+            && (curbuflen == sizeof(int))) {
+
+        DEBUGMSGTL(("verbose:socket:buffer:max", "Current %s is %d\n",
+                    buftype, curbuf));
+
+        /*
+         * Let's not be stupid ... if we were asked for less than what we
+         * already have, then forget about it
+         */
+        if (size <= curbuf) {
+            DEBUGMSGTL(("verbose:socket:buffer:max",
+                        "Requested %s <= current buffer\n", buftype));
+            return curbuf;
+        }
+
+        /*
+         * Do a binary search the optimal buffer within 1k of the point of
+         * failure. This is rather bruteforce, but simple
+         */
+        hi = size;
+        lo = curbuf;
+
+        while (hi - lo > 1024) {
+            mid = (lo + hi) / 2;
+            if (setsockopt(s, SOL_SOCKET, optname, (void *) &mid,
+                        sizeof(int)) == 0) {
+                lo = mid; /* Success: search between mid and hi */
+            } else {
+                hi = mid; /* Failed: search between lo and mid */
+            }
+        }
+
+        /*
+         * Now print if this optimization helped or not
+         */
+        if (getsockopt(s,SOL_SOCKET, optname, (void *) &curbuf,
+                    &curbuflen) == 0) {
+            DEBUGMSGTL(("socket:buffer:max", 
+                        "Maximized %s: %d\n",buftype, curbuf));
+        } 
+    } else {
+        /*
+         * There is really not a lot we can do anymore.
+         * If the OS doesn't give us the current buffer, then what's the 
+         * point in trying to make it better
+         */
+        DEBUGMSGTL(("socket:buffer:max", "Get %s failed ... giving up!\n",
+                    buftype));
+        curbuf = -1;
+    }
+
+    return curbuf;
+}
+
+
+static const char *
+_sock_buf_type_get(int optname, int local)
+{
+    if (optname == SO_SNDBUF) {
+        if (local)
+            return "server send buffer";
+        else
+            return "client send buffer";
+    } else if (optname == SO_RCVBUF) {
+        if (local)
+            return "server receive buffer";
+        else
+            return "client receive buffer";
+    }
+
+    return "unknown buffer";
+}
+
+/*
+ *
+ * Get the requested buffersize, based on
+ * - sockettype : client (local = 0) or server (local = 1) 
+ * - buffertype : send (optname = SO_SNDBUF) or recv (SO_RCVBUF)
+ *
+ * In case a compile time buffer was specified, then use that one
+ * if there was no runtime configuration override
+ */
+static int
+_sock_buffer_size_get(int optname, int local, const char **buftype)
+{
+    int size;
+
+    if (NULL != buftype)
+        *buftype = _sock_buf_type_get(optname, local);
+
+    if (optname == SO_SNDBUF) {
+        if (local) {
+            size = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
+                    NETSNMP_DS_LIB_SERVERSENDBUF);
+#ifdef DEFAULT_SERVER_SEND_BUF
+            if (size <= 0)
+               size = DEFAULT_SERVER_SEND_BUF;
+#endif
+        } else {
+            size = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
+                    NETSNMP_DS_LIB_CLIENTSENDBUF);
+#ifdef DEFAULT_CLIENT_SEND_BUF
+            if (size <= 0)
+               size = DEFAULT_CLIENT_SEND_BUF;
+#endif
+        }
+    } else if (optname == SO_RCVBUF) {
+        if (local) {
+            size = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
+                    NETSNMP_DS_LIB_SERVERRECVBUF);
+#ifdef DEFAULT_SERVER_RECV_BUF
+            if (size <= 0)
+               size = DEFAULT_SERVER_RECV_BUF;
+#endif
+        } else {
+            size = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID, 
+                    NETSNMP_DS_LIB_CLIENTRECVBUF);
+#ifdef DEFAULT_CLIENT_RECV_BUF
+            if (size <= 0)
+               size = DEFAULT_CLIENT_RECV_BUF;
+#endif
+        }
+    } else {
+        size = 0;
+    }
+
+    DEBUGMSGTL(("socket:buffer", "Requested %s is %d\n",
+                (buftype) ? *buftype : "unknown buffer", size));
+
+    return(size);
+}
+
+/*
+ * set socket buffer size
+ *
+ * @param ss     : socket
+ * @param optname: SO_SNDBUF or SO_RCVBUF
+ * @param local  : 1 for server, 0 for client
+ * @param reqbuf : requested size, or 0 for default
+ *
+ * @retval    -1 : error
+ * @retval    >0 : new buffer size
+ */
+int
+netsnmp_sock_buffer_set(int s, int optname, int local, int size)
+{
+#if ! defined(SO_SNDBUF) && ! defined(SO_RCVBUF)
+    DEBUGMSGTL(("socket:buffer", "Changing socket buffer is not supported\n"));
+    return -1;
+#else
+    const char     *buftype;
+    int            curbuf = 0, curbuflen = sizeof(int);
+
+#   ifndef  SO_SNDBUF
+    if (SO_SNDBUF == optname) {
+        DEBUGMSGTL(("socket:buffer",
+                    "Changing socket send buffer is not supported\n"));
+        return -1;
+    }
+#   endif                          /*SO_SNDBUF */
+#   ifndef  SO_RCVBUF
+    if (SO_RCVBUF == optname) {
+        DEBUGMSGTL(("socket:buffer",
+                    "Changing socket receive buffer is not supported\n"));
+        return -1;
+    }
+#   endif                          /*SO_RCVBUF */
+
+    /*
+     * What is the requested buffer size ?
+     */
+    if (0 == size)
+        size = _sock_buffer_size_get(optname, local, &buftype);
+    else {
+        buftype = _sock_buf_type_get(optname, local);
+        DEBUGMSGT(("verbose:socket:buffer", "Requested %s is %d\n",
+                   buftype, size));
+    }
+
+    if ((getsockopt(s, SOL_SOCKET, optname, (void *) &curbuf,
+                    &curbuflen) == 0) 
+        && (curbuflen == sizeof(int))) {
+        
+        DEBUGMSGT(("verbose:socket:buffer", "Original %s is %d\n",
+                   buftype, curbuf));
+        if (curbuf >= size) {
+            DEBUGMSGT(("verbose:socket:buffer",
+                      "New %s size is smaller than original!\n", buftype));
+        }
+    }
+
+    /*
+     * If the buffersize was not specified or it was a negative value
+     * then don't change the OS buffers at all
+     */
+    if (size <= 0) {
+       DEBUGMSGT(("socket:buffer",
+                    "%s not valid or not specified; using OS default(%d)\n",
+                    buftype,curbuf));
+       return curbuf;
+    }
+
+    /*
+     * Try to set the requested send buffer
+     */
+    if (setsockopt(s, SOL_SOCKET, optname, (void *) &size, sizeof(int)) == 0) {
+        /*
+         * Because some platforms lie about the actual buffer that has been 
+         * set (Linux will always say it worked ...), we print some 
+         * diagnostic output for debugging
+         */
+        DEBUGIF("socket:buffer") {
+            DEBUGMSGT(("socket:buffer", "Set %s to %d\n",
+                       buftype, size));
+            if ((getsockopt(s, SOL_SOCKET, optname, (void *) &curbuf,
+                            &curbuflen) == 0) 
+                    && (curbuflen == sizeof(int))) {
+
+                DEBUGMSGT(("verbose:socket:buffer",
+                           "Now %s is %d\n", buftype, curbuf));
+            }
+        }
+        /*
+         * If the new buffer is smaller than the size we requested, we will
+         * try to increment the new buffer with 1k increments 
+         * (this will sometime allow us to reach a more optimal buffer.)
+         *   For example : On Solaris, if the max OS buffer is 100k and you
+         *   request 110k, you end up with the default 8k :-(
+         */
+        if (curbuf < size) {
+            curbuf = _sock_buffer_maximize(s, optname, buftype, size);
+            if(-1 != curbuf)
+                size = curbuf;
+        }
+
+    } else {
+        /*
+         * Obviously changing the buffer failed, most like like because we 
+         * requested a buffer greater than the OS limit.
+         * Therefore we need to search for an optimal buffer that is close
+         * enough to the point of failure.
+         * This will allow us to reach a more optimal buffer.
+         *   For example : On Solaris, if the max OS buffer is 100k and you 
+         *   request 110k, you end up with the default 8k :-(
+         *   After this quick seach we would get 1k close to 100k (the max)
+         */
+        DEBUGMSGTL(("socket:buffer", "couldn't set %s to %d\n",
+                    buftype, size));
+
+        curbuf = _sock_buffer_maximize(s, optname, buftype, size);
+        if(-1 != curbuf)
+            size = curbuf;
+    }
+
+    return size;
+#endif
+}
 
 /*
  * Open a UDP-based transport for SNMP.  Local is TRUE if addr is the local
@@ -204,7 +479,7 @@ netsnmp_transport *
 netsnmp_udp_transport(struct sockaddr_in *addr, int local)
 {
     netsnmp_transport *t = NULL;
-    int             rc = 0, udpbuf = (1 << 17);
+    int             rc = 0;
     char           *string = NULL;
     char           *client_socket = NULL;
 
@@ -219,8 +494,8 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
 
     string = netsnmp_udp_fmtaddr(NULL, (void *)addr, 
 				 sizeof(struct sockaddr_in));
-    DEBUGMSGTL(("netsnmp_udp", "open %s %s\n", local ? "local" : "remote",
-                string));
+    DEBUGMSGTL(("netsnmp_udp", "open %s %s:%d\n", local ? "local" : "remote",
+                string,addr->sin_port));
     free(string);
 
     memset(t, 0, sizeof(netsnmp_transport));
@@ -233,62 +508,8 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
         netsnmp_transport_free(t);
         return NULL;
     }
-#ifdef  SO_BSDCOMPAT
-    /*
-     * Patch for Linux.  Without this, UDP packets that fail get an ICMP
-     * response.  Linux turns the failed ICMP response into an error message
-     * and return value, unlike all other OS's.  
-     */
-    if (0 == netsnmp_os_prematch("Linux","2.4"))
-    {
-        int             one = 1;
-        setsockopt(t->sock, SOL_SOCKET, SO_BSDCOMPAT, (void *) &one,
-                   sizeof(one));
-    }
-#endif                          /*SO_BSDCOMPAT */
-    /*
-     * SO_REUSEADDR will allow multiple apps to open the same port at
-     * the same time. Only the last one to open the socket will get
-     * data. Obviously, for an agent, this is a bad thing. There should
-     * only be one listener.
-     */
-#ifdef ALLOW_PORT_HIJACKING
-#ifdef  SO_REUSEADDR
-    /*
-     * Allow the same port to be specified multiple times without failing.
-     *    (useful for a listener)
-     */
-    {
-        int             one = 1;
-        setsockopt(t->sock, SOL_SOCKET, SO_REUSEADDR, (void *) &one,
-                   sizeof(one));
-    }
-#endif                          /*SO_REUSEADDR */
-#endif
-    /*
-     * Try to set the send and receive buffers to a reasonably large value, so
-     * that we can send and receive big PDUs (defaults to 8192 bytes (!) on
-     * Solaris, for instance).  Don't worry too much about errors -- just
-     * plough on regardless.  
-     */
 
-#ifdef  SO_SNDBUF
-    if (setsockopt
-        (t->sock, SOL_SOCKET, SO_SNDBUF, (void *) &udpbuf,
-         sizeof(int)) != 0) {
-        DEBUGMSGTL(("netsnmp_udp", "couldn't set SO_SNDBUF to %d bytes: %s\n",
-                    udpbuf, strerror(errno)));
-    }
-#endif                          /*SO_SNDBUF */
-
-#ifdef  SO_RCVBUF
-    if (setsockopt
-        (t->sock, SOL_SOCKET, SO_RCVBUF, (void *) &udpbuf,
-         sizeof(int)) != 0) {
-        DEBUGMSGTL(("netsnmp_udp", "couldn't set SO_RCVBUF to %d bytes: %s\n",
-                    udpbuf, strerror(errno)));
-    }
-#endif                          /*SO_RCVBUF */
+    _netsnmp_udp_sockopt_set(t->sock, local);
 
     if (local) {
         /*
@@ -365,6 +586,53 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
 }
 
 
+void
+_netsnmp_udp_sockopt_set(int fd, int local)
+{
+#ifdef  SO_BSDCOMPAT
+    /*
+     * Patch for Linux.  Without this, UDP packets that fail get an ICMP
+     * response.  Linux turns the failed ICMP response into an error message
+     * and return value, unlike all other OS's.  
+     */
+    if (0 == netsnmp_os_prematch("Linux","2.4"))
+    {
+        int             one = 1;
+        DEBUGMSGTL(("socket:option", "setting socket option SO_BSDCOMPAT\n"));
+        setsockopt(fd, SOL_SOCKET, SO_BSDCOMPAT, (void *) &one,
+                   sizeof(one));
+    }
+#endif                          /*SO_BSDCOMPAT */
+    /*
+     * SO_REUSEADDR will allow multiple apps to open the same port at
+     * the same time. Only the last one to open the socket will get
+     * data. Obviously, for an agent, this is a bad thing. There should
+     * only be one listener.
+     */
+#ifdef ALLOW_PORT_HIJACKING
+#ifdef  SO_REUSEADDR
+    /*
+     * Allow the same port to be specified multiple times without failing.
+     *    (useful for a listener)
+     */
+    {
+        int             one = 1;
+        DEBUGMSGTL(("socket:option", "setting socket option SO_REUSEADDR\n"));
+        setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (void *) &one,
+                   sizeof(one));
+    }
+#endif                          /*SO_REUSEADDR */
+#endif
+
+    /*
+     * Try to set the send and receive buffers to a reasonably large value, so
+     * that we can send and receive big PDUs (defaults to 8192 bytes (!) on
+     * Solaris, for instance).  Don't worry too much about errors -- just
+     * plough on regardless.  
+     */
+    netsnmp_sock_buffer_set(fd, SO_SNDBUF, local, 0);
+    netsnmp_sock_buffer_set(fd, SO_RCVBUF, local, 0);
+}
 
 int
 netsnmp_sockaddr_in(struct sockaddr_in *addr,
@@ -473,6 +741,7 @@ netsnmp_sockaddr_in(struct sockaddr_in *addr,
 
 
 
+#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
 /*
  * The following functions provide the "com2sec" configuration token
  * functionality for compatibility.  
@@ -486,6 +755,7 @@ typedef struct _com2SecEntry {
     unsigned long   network;
     unsigned long   mask;
     char            secName[VACMSTRINGLEN];
+    char            contextName[VACMSTRINGLEN];
     struct _com2SecEntry *next;
 } com2SecEntry;
 
@@ -494,9 +764,10 @@ com2SecEntry   *com2SecList = NULL, *com2SecListLast = NULL;
 void
 netsnmp_udp_parse_security(const char *token, char *param)
 {
-    char           secName[VACMSTRINGLEN];
-    char           source[VACMSTRINGLEN];
-    char           community[VACMSTRINGLEN];
+    char            secName[VACMSTRINGLEN];
+    char            contextName[VACMSTRINGLEN];
+    char            community[VACMSTRINGLEN];
+    char            source[VACMSTRINGLEN];
     char           *cp = NULL;
     const char     *strmask = NULL;
     com2SecEntry   *e = NULL;
@@ -506,7 +777,17 @@ netsnmp_udp_parse_security(const char *token, char *param)
      * Get security, source address/netmask and community strings.  
      */
 
-    cp = copy_nword(param, secName, sizeof(secName));
+    cp = copy_nword( param, secName, sizeof(secName));
+    if (strcmp(secName, "-Cn") == 0) {
+        if (!cp) {
+            config_perror("missing CONTEXT_NAME parameter");
+            return;
+        }
+        cp = copy_nword( cp, contextName, sizeof(contextName));
+        cp = copy_nword( cp, secName, sizeof(secName));
+    } else {
+        contextName[0] = '\0';
+    }
     if (secName[0] == '\0') {
         config_perror("missing NAME parameter");
         return;
@@ -514,7 +795,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
         config_perror("security name too long");
         return;
     }
-    cp = copy_nword(cp, source, sizeof(source));
+    cp = copy_nword( cp, source, sizeof(source));
     if (source[0] == '\0') {
         config_perror("missing SOURCE parameter");
         return;
@@ -523,7 +804,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
         config_perror("example config NETWORK not properly configured");
         return;
     }
-    cp = copy_nword(cp, community, sizeof(community));
+    cp = copy_nword( cp, community, sizeof(community));
     if (community[0] == '\0') {
         config_perror("missing COMMUNITY parameter\n");
         return;
@@ -652,6 +933,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
                 "<\"%s\", 0x%08x/0x%08x> => \"%s\"\n", community, network,
                 mask, secName));
 
+    strcpy(e->contextName, contextName);
     strcpy(e->secName, secName);
     strcpy(e->community, community);
     e->network = network;
@@ -678,14 +960,16 @@ netsnmp_udp_com2SecList_free(void)
     }
     com2SecList = com2SecListLast = NULL;
 }
-
+#endif /* support for community based SNMP */
 
 void
 netsnmp_udp_agent_config_tokens_register(void)
 {
+#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
     register_app_config_handler("com2sec", netsnmp_udp_parse_security,
                                 netsnmp_udp_com2SecList_free,
-                                "name IPv4-network-address[/netmask] community");
+                                "[-Cn CONTEXT] secName IPv4-network-address[/netmask] community");
+#endif /* support for community based SNMP */
 }
 
 
@@ -697,10 +981,12 @@ netsnmp_udp_agent_config_tokens_register(void)
  * parameters did not match any com2sec entry.  
  */
 
+#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
 int
 netsnmp_udp_getSecName(void *opaque, int olength,
                        const char *community,
-                       size_t community_len, char **secName)
+                       size_t community_len, char **secName,
+                       char **contextName)
 {
     com2SecEntry   *c;
     struct sockaddr_in *from = (struct sockaddr_in *) opaque;
@@ -753,6 +1039,7 @@ netsnmp_udp_getSecName(void *opaque, int olength,
             DEBUGMSG(("netsnmp_udp_getSecName", "... SUCCESS\n"));
             if (secName != NULL) {
                 *secName = c->secName;
+                *contextName = c->contextName;
             }
             break;
         }
@@ -763,7 +1050,7 @@ netsnmp_udp_getSecName(void *opaque, int olength,
     }
     return 1;
 }
-
+#endif /* support for community based SNMP */
 
 
 netsnmp_transport *
