@@ -74,9 +74,15 @@ const struct in6_addr in6addr_any = IN6ADDR_ANY_INIT;
 #include <net-snmp/library/snmp_transport.h>
 #include <net-snmp/library/snmpUDPIPv6Domain.h>
 
-oid netsnmp_UDPIPv6Domain[10] = { ENTERPRISE_MIB, 3, 3, 4 };
+oid netsnmp_UDPIPv6Domain[] = { TRANSPORT_DOMAIN_UDP_IPV6 };
 static netsnmp_tdomain udp6Domain;
 
+/*
+ * from snmpUDPDomain. not static, but not public, either.
+ * (ie don't put it in a public header.)
+ */
+extern void _netsnmp_udp_sockopt_set(int fd, int server);
+
 /*
  * Return a string representing the address in data, or else the "far end"
  * address if data is NULL.  
@@ -100,7 +106,7 @@ netsnmp_udp6_fmtaddr(netsnmp_transport *t, void *data, int len)
         char addr[INET6_ADDRSTRLEN];
         char tmp[INET6_ADDRSTRLEN + 8];
 
-        sprintf(tmp, "[%s]:%hd",
+        sprintf(tmp, "UDP/IPv6: [%s]:%hd",
                 inet_ntop(AF_INET6, (void *) &(to->sin6_addr), addr,
                           INET6_ADDRSTRLEN), ntohs(to->sin6_port));
         return strdup(tmp);
@@ -219,7 +225,7 @@ netsnmp_transport *
 netsnmp_udp6_transport(struct sockaddr_in6 *addr, int local)
 {
     netsnmp_transport *t = NULL;
-    int             rc = 0, udpbuf = (1 << 17);
+    int             rc = 0;
     char           *string = NULL;
 
     if (addr == NULL || addr->sin6_family != AF_INET6) {
@@ -248,39 +254,8 @@ netsnmp_udp6_transport(struct sockaddr_in6 *addr, int local)
         netsnmp_transport_free(t);
         return NULL;
     }
-#ifdef  SO_BSDCOMPAT
-    /*
-     * Patch for Linux.  Without this, UDP packets that fail get an ICMP
-     * response.  Linux turns the failed ICMP response into an error message
-     * and return value, unlike all other OS's.  
-     */
-    if (0 == netsnmp_os_prematch("Linux","2.4"))
-    {
-        int             one = 1;
-        setsockopt(t->sock, SOL_SOCKET, SO_BSDCOMPAT, &one, sizeof(one));
-    }
-#endif                          /*SO_BSDCOMPAT */
 
-    /*
-     * Try to set the send and receive buffers to a reasonably large value, so
-     * that we can send and receive big PDUs (defaults to 8192 bytes (!) on
-     * Solaris, for instance).  Don't worry too much about errors -- just
-     * plough on regardless.  
-     */
-
-#ifdef  SO_SNDBUF
-    if (setsockopt(t->sock, SOL_SOCKET, SO_SNDBUF, (void *)&udpbuf, sizeof(int)) != 0){
-        DEBUGMSGTL(("netsnmp_udp6", "couldn't set SO_SNDBUF to %d bytes: %s\n",
-		    udpbuf, strerror(errno)));
-    }
-#endif                          /*SO_SNDBUF */
-
-#ifdef  SO_RCVBUF
-    if (setsockopt(t->sock, SOL_SOCKET, SO_RCVBUF, (void *)&udpbuf, sizeof(int)) != 0){
-        DEBUGMSGTL(("netsnmp_udp6", "couldn't set SO_RCVBUF to %d bytes: %s\n",
-		    udpbuf, strerror(errno)));
-    }
-#endif                          /*SO_RCVBUF */
+    _netsnmp_udp_sockopt_set(t->sock, local);
 
     if (local) {
         /*
@@ -790,10 +765,6 @@ inet_addrs_consistence(int pf, void *net, void *mask)
     switch (pf) {
     case PF_INET:
         tmp = (struct sockaddr_in *) malloc(sizeof(struct sockaddr_in));
-        if (!tmp) {
-            config_perror("Resource failure in inet_addr_consistence()");
-            return -1;
-        }
         memset(tmp, 0, sizeof(*tmp));
         tmp->sin_family = PF_INET;
         if (inet_addr_complement
@@ -803,11 +774,6 @@ inet_addrs_consistence(int pf, void *net, void *mask)
             return -1;
         }
         dst = (struct sockaddr_in *) malloc(sizeof(struct sockaddr_in));
-        if (!dst) {
-            config_perror("Resource failure in inet_addr_consistence()");
-            free(tmp);
-            return -1;
-        }
         memset(dst, 0, sizeof(*dst));
         dst->sin_family = PF_INET;
         if (inet_addr_and
@@ -824,10 +790,6 @@ inet_addrs_consistence(int pf, void *net, void *mask)
         break;
     case PF_INET6:
         tmp6 = (struct sockaddr_in6 *) malloc(sizeof(struct sockaddr_in6));
-        if (!tmp6) {
-            config_perror("Resource failure in inet_addr_consistence()");
-            return -1;
-        }
         memset(tmp6, 0, sizeof(*tmp6));
         tmp6->sin6_family = PF_INET6;
         if (inet_addr_complement
@@ -837,11 +799,6 @@ inet_addrs_consistence(int pf, void *net, void *mask)
             return -1;
         }
         dst6 = (struct sockaddr_in6 *) malloc(sizeof(struct sockaddr_in6));
-        if (!dst6) {
-            config_perror("Resource failure in inet_addr_consistence()");
-            free(tmp6);
-            return -1;
-        }
         memset(dst6, 0, sizeof(*dst6));
         dst6->sin6_family = PF_INET6;
         if (inet_addr_and
@@ -933,6 +890,7 @@ masked_address_are_equal(int af, struct sockaddr_storage *from,
     }
 }
 
+#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
 /*
  * The following functions provide the "com2sec6" configuration token
  * functionality for compatibility.  
@@ -946,6 +904,7 @@ typedef struct _com2Sec6Entry {
     struct sockaddr_in6 network;
     struct sockaddr_in6 mask;
     char            secName[VACMSTRINGLEN];
+    char            contextName[VACMSTRINGLEN];
     struct _com2Sec6Entry *next;
 } com2Sec6Entry;
 
@@ -956,9 +915,11 @@ void
 memmove_com2Sec6Entry(com2Sec6Entry * c,
                       char *secName,
                       char *community,
-                      struct sockaddr_in6 net, struct sockaddr_in6 mask)
+                      struct sockaddr_in6 net, struct sockaddr_in6 mask,
+                      char *contextName)
 {
     snprintf(c->secName, strlen(secName) + 1, "%s", secName);
+    snprintf(c->contextName, strlen(contextName) + 1, "%s", contextName);
     snprintf(c->community, strlen(community) + 1, "%s", community);
     memmove(&c->network, &net, sizeof(net));
     memmove(&c->mask, &mask, sizeof(mask));
@@ -969,9 +930,10 @@ memmove_com2Sec6Entry(com2Sec6Entry * c,
 void
 netsnmp_udp6_parse_security(const char *token, char *param)
 {
-    char           secName[VACMSTRINGLEN];
-    char           source[VACMSTRINGLEN];
-    char           community[VACMSTRINGLEN];
+    char            secName[VACMSTRINGLEN];
+    char            contextName[VACMSTRINGLEN];
+    char            community[VACMSTRINGLEN];
+    char            source[VACMSTRINGLEN];
     char           *cp = NULL, *strnetwork = NULL, *strmask = NULL;
     com2Sec6Entry  *e = NULL;
     struct sockaddr_in6 net, mask;
@@ -988,7 +950,17 @@ netsnmp_udp6_parse_security(const char *token, char *param)
     /*
      * Get security, source address/netmask and community strings.  
      */
-    cp = copy_nword(param, secName, sizeof(secName));
+    cp = copy_nword( param, secName, sizeof(secName));
+    if (strcmp(secName, "-Cn") == 0) {
+        if (!cp) {
+            config_perror("missing CONTEXT_NAME parameter");
+            return;
+        }
+        cp = copy_nword( cp, contextName, sizeof(contextName));
+        cp = copy_nword( cp, secName, sizeof(secName));
+    } else {
+        contextName[0] = '\0';
+    }
     if (secName[0] == '\0') {
         config_perror("missing NAME parameter");
         return;
@@ -996,7 +968,7 @@ netsnmp_udp6_parse_security(const char *token, char *param)
         config_perror("security name too long");
         return;
     }
-    cp = copy_nword(cp, source, sizeof(source));
+    cp = copy_nword( cp, source, sizeof(source));
     if (source[0] == '\0') {
         config_perror("missing SOURCE parameter");
         return;
@@ -1005,7 +977,7 @@ netsnmp_udp6_parse_security(const char *token, char *param)
         config_perror("example config NETWORK not properly configured");
         return;
     }
-    cp = copy_nword(cp, community, sizeof(community));
+    cp = copy_nword( cp, community, sizeof(community));
     if (community[0] == '\0') {
         config_perror("missing COMMUNITY parameter\n");
         return;
@@ -1061,7 +1033,7 @@ netsnmp_udp6_parse_security(const char *token, char *param)
             DEBUGMSGTL(("netsnmp_udp6_parse_security",
                         "Couldn't allocate enough memory\n"));
         }
-        memmove_com2Sec6Entry(e, secName, community, net, mask);
+        memmove_com2Sec6Entry(e, secName, community, net, mask, contextName);
         if (com2Sec6ListLast != NULL) {
             com2Sec6ListLast->next = e;
             com2Sec6ListLast = e;
@@ -1120,7 +1092,8 @@ netsnmp_udp6_parse_security(const char *token, char *param)
                 DEBUGMSGTL(("netsnmp_udp6_parse_security",
                             "Couldn't allocate enough memory\n"));
             }
-            memmove_com2Sec6Entry(e, secName, community, net, mask);
+            memmove_com2Sec6Entry(e, secName, community, net, mask,
+                                  contextName);
             if (com2Sec6ListLast != NULL) {
                 com2Sec6ListLast->next = e;
                 com2Sec6ListLast = e;
@@ -1131,7 +1104,6 @@ netsnmp_udp6_parse_security(const char *token, char *param)
 #if HAVE_GETADDRINFO
 
         } else {
-
             /*
              * Nope, Must be a hostname.  
              */
@@ -1169,7 +1141,8 @@ netsnmp_udp6_parse_security(const char *token, char *param)
                 DEBUGMSGTL(("netsnmp_udp6_parse_security",
                             "<\"%s\", %s> => \"%s\"\n", community, hbuf,
                             secName));
-                memmove_com2Sec6Entry(e, secName, community, net, mask);
+                memmove_com2Sec6Entry(e, secName, community, net, mask,
+                                      contextName);
                 if (com2Sec6ListLast != NULL) {
                     com2Sec6ListLast->next = e;
                     com2Sec6ListLast = e;
@@ -1183,7 +1156,6 @@ netsnmp_udp6_parse_security(const char *token, char *param)
 #endif /* HAVE_GETADDRINFO */
 
         }
-
         /*
          * free(strnetwork); 
          */
@@ -1202,17 +1174,21 @@ netsnmp_udp6_com2Sec6List_free(void)
     com2Sec6List = com2Sec6ListLast = NULL;
 }
 
+#endif /* support for community based SNMP */
 
 void
 netsnmp_udp6_agent_config_tokens_register(void)
 {
+#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
     register_app_config_handler("com2sec6", netsnmp_udp6_parse_security,
                                 netsnmp_udp6_com2Sec6List_free,
-                                "name IPv6-network-address[/netmask] community");
+                                "[-Cn CONTEXT] secName IPv6-network-address[/netmask] community");
+#endif /* support for community based SNMP */
 }
 
 
 
+#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
 /*
  * Return 0 if there are no com2sec entries, or return 1 if there ARE com2sec 
  * entries.  On return, if a com2sec entry matched the passed parameters,
@@ -1223,7 +1199,7 @@ netsnmp_udp6_agent_config_tokens_register(void)
 int
 netsnmp_udp6_getSecName(void *opaque, int olength,
                         const char *community,
-                        int community_len, char **secName)
+                        int community_len, char **secName, char **contextName)
 {
     com2Sec6Entry  *c;
     struct sockaddr_in6 *from = (struct sockaddr_in6 *) opaque;
@@ -1281,6 +1257,7 @@ netsnmp_udp6_getSecName(void *opaque, int olength,
             DEBUGMSG(("netsnmp_udp6_getSecName", "... SUCCESS\n"));
             if (secName != NULL) {
                 *secName = c->secName;
+                *contextName = c->contextName;
             }
             break;
         }
@@ -1291,6 +1268,7 @@ netsnmp_udp6_getSecName(void *opaque, int olength,
     }
     return 1;
 }
+#endif /* support for community based SNMP */
 
 netsnmp_transport *
 netsnmp_udp6_create_tstring(const char *string, int local)
