@@ -81,9 +81,6 @@ SOFTWARE.
 #if HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
-#if HAVE_SYSLOG_H
-#include <syslog.h>
-#endif
 
 #if HAVE_DMALLOC_H
 #include <dmalloc.h>
@@ -99,7 +96,7 @@ SOFTWARE.
 #include <net-snmp/library/snmp_client.h>
 #include <net-snmp/library/snmp_secmod.h>
 #include <net-snmp/library/mib.h>
-#include <net-snmp/net-snmp-includes.h>
+
 
 #ifndef BSD4_3
 #define BSD4_2
@@ -296,7 +293,8 @@ snmp_reset_var_buffers(netsnmp_variable_list * var)
 {
     while (var) {
         if (var->name != var->name_loc) {
-            free(var->name);
+            if(NULL != var->name)
+                free(var->name);
             var->name = var->name_loc;
             var->name_length = 0;
         }
@@ -716,151 +714,41 @@ find_varbind_of_type(netsnmp_variable_list * var_ptr, u_char type)
  */
 
 int
-snmp_set_var_value(netsnmp_variable_list * vars,
-                   const u_char * value, size_t len)
+snmp_set_var_value(netsnmp_variable_list * newvar,
+                   const u_char * val_str, size_t val_len)
 {
-    int             largeval = 1;
-    const long     *val_long = NULL;
-    const int      *val_int  = NULL;
-    const u_int    *val_uint  = NULL;
-
     /*
      * xxx-rks: why the unconditional free? why not use existing
-     * memory, if len < vars->val_len ?
+     * memory, if val_len < newvar->val_len ?
      */
-    if (vars->val.string && vars->val.string != vars->buf) {
-        free(vars->val.string);
+    if (newvar->val.string && newvar->val.string != newvar->buf) {
+        free(newvar->val.string);
     }
-    vars->val.string = 0;
-    vars->val_len = 0;
+
+    newvar->val.string = 0;
+    newvar->val_len = 0;
 
     /*
-     * use built-in storage for smaller values 
+     * need a pointer and a length to copy a string value. 
      */
-    if (len <= (sizeof(vars->buf) - 1)) {
-        vars->val.string = (u_char *) vars->buf;
-        largeval = 0;
-    }
-
-    if ((0 == len) || (NULL == value)) {
-        vars->val.string[0] = 0;
-        return 0;
-    }
-
-    vars->val_len = len;
-    switch (vars->type) {
-    case ASN_INTEGER:
-    case ASN_UNSIGNED:
-    case ASN_TIMETICKS:
-    case ASN_IPADDRESS:
-    case ASN_COUNTER:
-        if (value) {
-            if (largeval) {
-                snmp_log(LOG_ERR,"bad size for integer-like type (%d)\n",
-                         vars->val_len);
-                return (1);
-            } else if (vars->val_len == sizeof(int)) {
-                if (ASN_INTEGER == vars->type) {
-                    val_int = (const int *) value;
-                    *(vars->val.integer) = (long) *val_int;
-                } else {
-                    val_uint = (const u_int *) value;
-                    *(vars->val.integer) = (long) *val_uint;
-                }
-            } else {
-                val_long = (const long *) value;
-                *(vars->val.integer) = *val_long;
-            }
+    if (val_str && val_len) {
+        if (val_len <= sizeof(newvar->buf))
+            newvar->val.string = newvar->buf;
+        else {
+            newvar->val.string = (u_char *) malloc(val_len);
+            if (!newvar->val.string)
+                return 1;
         }
-        vars->val_len = sizeof(long);
-        break;
-
-    case ASN_OBJECT_ID:
-    case ASN_PRIV_IMPLIED_OBJECT_ID:
-    case ASN_PRIV_INCL_RANGE:
-    case ASN_PRIV_EXCL_RANGE:
-        if (largeval) {
-            vars->val.objid = (oid *) malloc(vars->val_len);
-        }
-        if (vars->val.objid == NULL) {
-            snmp_log(LOG_ERR,"no storage for OID\n");
-            return 1;
-        }
-        memmove(vars->val.objid, value, vars->val_len);
-        break;
-
-    case ASN_PRIV_IMPLIED_OCTET_STR:
-    case ASN_OCTET_STR:
-    case ASN_BIT_STR:
-    case ASN_OPAQUE:
-    case ASN_NSAP:
-        if (largeval) {
-            vars->val.string = (u_char *) malloc(vars->val_len + 1);
-        }
-        if (vars->val.string == NULL) {
-            snmp_log(LOG_ERR,"no storage for OID\n");
-            return 1;
-        }
-        memmove(vars->val.string, value, vars->val_len);
+        memmove(newvar->val.string, val_str, val_len);
+        newvar->val_len = val_len;
+    } else if (val_str) {
         /*
-         * Make sure the string is zero-terminated; some bits of code make
-         * this assumption.  Easier to do this here than fix all these wrong
-         * assumptions.  
+         * NULL STRING != NULL ptr 
          */
-        vars->val.string[vars->val_len] = '\0';
-        break;
-
-    case SNMP_NOSUCHOBJECT:
-    case SNMP_NOSUCHINSTANCE:
-    case SNMP_ENDOFMIBVIEW:
-    case ASN_NULL:
-        vars->val_len = 0;
-        vars->val.string = NULL;
-        break;
-
-#ifdef OPAQUE_SPECIAL_TYPES
-    case ASN_OPAQUE_U64:
-    case ASN_OPAQUE_I64:
-#endif                          /* OPAQUE_SPECIAL_TYPES */
-    case ASN_COUNTER64:
-        if (largeval) {
-            snmp_log(LOG_ERR,"bad size for counter 64 (%d)\n",
-                     vars->val_len);
-            return (1);
-        }
-        vars->val_len = sizeof(struct counter64);
-        memmove(vars->val.counter64, value, vars->val_len);
-        break;
-
-#ifdef OPAQUE_SPECIAL_TYPES
-    case ASN_OPAQUE_FLOAT:
-        if (largeval) {
-            snmp_log(LOG_ERR,"bad size for opaque float (%d)\n",
-                     vars->val_len);
-            return (1);
-        }
-        vars->val_len = sizeof(float);
-        memmove(vars->val.floatVal, value, vars->val_len);
-        break;
-
-    case ASN_OPAQUE_DOUBLE:
-        if (largeval) {
-            snmp_log(LOG_ERR,"bad size for opaque double (%d)\n",
-                     vars->val_len);
-            return (1);
-        }
-        vars->val_len = sizeof(double);
-        memmove(vars->val.doubleVal, value, vars->val_len);
-        break;
-
-#endif                          /* OPAQUE_SPECIAL_TYPES */
-
-    default:
-        snmp_log(LOG_ERR,"no storage for OID\n");
-        snmp_set_detail("Internal error in type switching\n");
-        return (1);
+        newvar->val.string = newvar->buf;
+        newvar->val.string[0] = '\0';
+        newvar->val_len = 0;
     }
-
     return 0;
 }
 
@@ -1048,14 +936,14 @@ const char     *error_string[19] = {
     "wrongLength (The set value has an illegal length from what the agent expects)",
     "wrongEncoding",
     "wrongValue (The set value is illegal or unsupported in some way)",
-    "noCreation (that table does not support row creation)",
+    "noCreation (That table does not support row creation or that object can not ever be created)",
     "inconsistentValue (The set value is illegal or unsupported in some way)",
     "resourceUnavailable (This is likely a out-of-memory failure within the agent)",
     "commitFailed",
     "undoFailed",
     "authorizationError (access denied to that object)",
-    "notWritable (that object does not support modification)",
-    "inconsistentName"
+    "notWritable (That object does not support modification)",
+    "inconsistentName (That object can not currently be created)"
 };
 
 const char     *
