@@ -283,7 +283,7 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
      * Finally, handle ".... -Lx value ...." syntax
      *   (*without* surrounding quotes)
      */
-    if ((!*optarg) && (NULL != argv)) {
+    if (!*optarg) {
         /*
          * We've run off the end of the argument
          *  so move on to the next.
@@ -379,6 +379,29 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
 	}
         break;
 
+    /*
+     * Don't log 
+     */
+    case 'N':
+        priority = decode_priority( optarg, &pri_max );
+        if (priority == -1)  return -1;
+        if (inc_optind)
+            optind++;
+        /* Fallthrough */
+    case 'n':
+        /*
+         * disable all logs to clean them up (close files, etc),
+         * remove all log handlers, then register a null handler.
+         */
+        snmp_disable_log();
+        while(NULL != logh_head)
+            netsnmp_remove_loghandler( logh_head );
+        logh = netsnmp_register_loghandler(NETSNMP_LOGHANDLER_NONE, priority);
+        if (logh) {
+            logh->pri_max = pri_max;
+	}
+        break;
+
     default:
         fprintf(stderr, "Unknown logging option passed to -L: %c.\n", *cp);
         return -1;
@@ -387,7 +410,7 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
 }
 
 char *
-snmp_log_syslogname(const char *pstr)
+snmp_log_syslogname(char *pstr)
 {
   if (pstr)
     strncpy (syslogname, pstr, sizeof(syslogname));
@@ -402,11 +425,12 @@ snmp_log_options_usage(const char *lead, FILE * outf)
     const char *pri2_msg = " for levels 'p1' to 'p2'";
     fprintf(outf, "%se:           log to standard error\n", lead);
     fprintf(outf, "%so:           log to standard output\n", lead);
+    fprintf(outf, "%sn:           don't log at all\n", lead);
     fprintf(outf, "%sf file:      log to the specified file\n", lead);
     fprintf(outf, "%ss facility:  log to syslog (via the specified facility)\n", lead);
     fprintf(outf, "\n%s(variants)\n", lead);
-    fprintf(outf, "%s[EO] pri:    log to standard error/output%s\n", lead, pri1_msg);
-    fprintf(outf, "%s[EO] p1-p2:  log to standard error/output%s\n", lead, pri2_msg);
+    fprintf(outf, "%s[EON] pri:   log to standard error, output or /dev/null%s\n", lead, pri1_msg);
+    fprintf(outf, "%s[EON] p1-p2: log to standard error, output or /dev/null%s\n", lead, pri2_msg);
     fprintf(outf, "%s[FS] pri token:    log to file/syslog%s\n", lead, pri1_msg);
     fprintf(outf, "%s[FS] p1-p2 token:  log to file/syslog%s\n", lead, pri2_msg);
 }
@@ -470,26 +494,48 @@ sprintf_stamp(time_t * now, char *sbuf)
     return sbuf;
 }
 
+void
+snmp_disable_syslog_entry(netsnmp_log_handler *logh)
+{
+    if (!logh || !logh->enabled || logh->type != NETSNMP_LOGHANDLER_SYSLOG)
+        return;
+
+#ifdef WIN32
+    if (logh->magic) {
+        HANDLE eventlog_h = (HANDLE)logh->magic;
+        CloseEventLog(eventlog_h);
+        logh->magic = NULL;
+    }
+#else
+    closelog();
+    logh->imagic  = 0;
+#endif
+
+    logh->enabled = 0;
+}
+
 void
 snmp_disable_syslog(void)
 {
     netsnmp_log_handler *logh;
 
     for (logh = logh_head; logh; logh = logh->next)
-        if (logh->enabled && logh->type == NETSNMP_LOGHANDLER_SYSLOG) {
+        if (logh->enabled && logh->type == NETSNMP_LOGHANDLER_SYSLOG)
+            snmp_disable_syslog_entry(logh);
+}
 
-#ifdef WIN32
-            if (logh->magic) {
-                HANDLE eventlog_h = (HANDLE)logh->magic;
-                CloseEventLog(eventlog_h);
-                logh->magic = NULL;
-            }
-#else
-            closelog();
-            logh->imagic  = 0;
-#endif
-            logh->enabled = 0;
-	}
+void
+snmp_disable_filelog_entry(netsnmp_log_handler *logh)
+{
+    if (!logh /* || !logh->enabled */ || logh->type != NETSNMP_LOGHANDLER_FILE)
+        return;
+
+    if (logh->magic) {
+        fputs("\n", (FILE*)logh->magic);	/* XXX - why? */
+        fclose((FILE*)logh->magic);
+        logh->magic   = NULL;
+    }
+    logh->enabled = 0;
 }
 
 void
@@ -498,33 +544,8 @@ snmp_disable_filelog(void)
     netsnmp_log_handler *logh;
 
     for (logh = logh_head; logh; logh = logh->next)
-        if (logh->enabled && logh->type == NETSNMP_LOGHANDLER_FILE) {
-            if (logh->magic) {
-                fputs("\n", (FILE*)logh->magic);	/* XXX - why? */
-                fclose((FILE*)logh->magic);
-                logh->magic   = NULL;
-	    }
-            logh->enabled = 0;
-	}
-}
-
-/*
- * returns that status of stderr logging
- *
- * @retval 0 : stderr logging disabled
- * @retval 1 : stderr logging enabled
- */
-snmp_stderrlog_status(void)
-{
-    netsnmp_log_handler *logh;
-
-    for (logh = logh_head; logh; logh = logh->next)
-        if (logh->enabled && (logh->type == NETSNMP_LOGHANDLER_STDOUT ||
-                              logh->type == NETSNMP_LOGHANDLER_STDERR)) {
-            return 1;
-	}
-
-    return 0;
+        if (logh->enabled && logh->type == NETSNMP_LOGHANDLER_FILE)
+            snmp_disable_filelog_entry(logh);
 }
 
 void
@@ -555,8 +576,13 @@ snmp_disable_log(void)
 {
     netsnmp_log_handler *logh;
 
-    for (logh = logh_head; logh; logh = logh->next)
-        logh->enabled = 0;	/* XXX - any per-handler closedown? */
+    for (logh = logh_head; logh; logh = logh->next) {
+        if (logh->type == NETSNMP_LOGHANDLER_SYSLOG)
+            snmp_disable_syslog_entry(logh);
+        if (logh->type == NETSNMP_LOGHANDLER_FILE)
+            snmp_disable_filelog_entry(logh);
+        logh->enabled = 0;
+    }
 }
 
 /* ================================================== */
@@ -647,14 +673,7 @@ snmp_enable_filelog(const char *logfilename, int dont_zero_log)
 {
     netsnmp_log_handler *logh;
 
-    /*
-     * don't disable ALL filelogs whenever a new one is enabled.
-     * this prevents '-Lf file' from working in snmpd, as the
-     * call to set up /var/log/snmpd.log will disable the previous
-     * log setup. again, this new linked list of log handlers
-     * needs rethinking/cleanup. xxx-rks
-     * snmp_disable_filelog();
-     */
+    snmp_disable_filelog();	/* XXX ??? */
 
     if (logfilename) {
         logh = netsnmp_find_loghandler( logfilename );
@@ -812,6 +831,9 @@ netsnmp_register_loghandler( int type, int priority )
     case NETSNMP_LOGHANDLER_CALLBACK:
         logh->handler = log_handler_callback;
         break;
+    case NETSNMP_LOGHANDLER_NONE:
+        logh->handler = log_handler_null;
+        break;
     default:
         free(logh);
         return NULL;
@@ -1062,11 +1084,9 @@ snmp_log_string(int priority, const char *string)
     for ( ; logh; logh = logh->next ) {
         /*
          * ... but skipping any handlers with a "maximum priority"
-         *     that we have already exceeded. And don't forget to
-         *     ensure this logging is turned on (see snmp_disable_stderrlog
-         *     and its cohorts).
+	 *     that we have already exceeded.
          */
-        if (logh->enabled && (priority >= logh->pri_max))
+        if (priority >= logh->pri_max)
             logh->handler( logh, priority, string );
     }
 }
