@@ -1,6 +1,16 @@
 /*
  * snmp_parse_args.c
  */
+/* Portions of this file are subject to the following copyright(s).  See
+ * the Net-SNMP's COPYING file for more details and other copyrights
+ * that may apply:
+ */
+/*
+ * Portions of this file are copyrighted by:
+ * Copyright @ 2003 Sun Microsystems, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
+ */
 
 #include <net-snmp/net-snmp-config.h>
 
@@ -88,8 +98,10 @@ snmp_parse_args_descriptions(FILE * outf)
             "  -H\t\t\tdisplay configuration file directives understood\n");
     fprintf(outf, "  -v 1|2c|3\t\tspecifies SNMP version to use\n");
     fprintf(outf, "  -V, --version\t\tdisplay package version number\n");
+#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
     fprintf(outf, "SNMP Version 1 or 2c specific\n");
     fprintf(outf, "  -c COMMUNITY\t\tset the community string\n");
+#endif /* support for community based SNMP */
     fprintf(outf, "SNMP Version 3 specific\n");
     fprintf(outf,
             "  -a PROTOCOL\t\tset authentication protocol (MD5|SHA)\n");
@@ -151,8 +163,6 @@ handle_long_opt(const char *myoptarg)
      * else it's a long option, so process it like name=value 
      */
     cp = malloc(strlen(myoptarg) + 3);
-    if (!cp)
-        return;
     strcpy(cp, myoptarg);
     cp2 = strchr(cp, '=');
     if (!cp2 && !strchr(cp, ' ')) {
@@ -178,18 +188,26 @@ handle_long_opt(const char *myoptarg)
 extern int      snmpv3_options(char *optarg, netsnmp_session * session,
                                char **Apsz, char **Xpsz, int argc,
                                char *const *argv);
+
+/*
+ * This method does the real work for snmp_parse_args.  It takes an
+ * extra argument, proxy, and uses this to decide how to handle the lack of
+ * of a community string.
+ */
 int
-snmp_parse_args(int argc, char **argv, netsnmp_session *session, 
-		const char *localOpts, void (*proc)(int, char *const *, int))
+snmp_parse_args(int argc,
+                char **argv,
+                netsnmp_session * session, const char *localOpts,
+                void (*proc) (int, char *const *, int))
 {
     static char	   *sensitive[4] = { NULL, NULL, NULL, NULL };
-    int             arg, sp = 0, zero_sensitive = 1;
+    int             arg, sp = 0, zero_sensitive = 1, testcase = 0;
     char           *cp;
     char           *Apsz = NULL;
     char           *Xpsz = NULL;
     char           *Cpsz = NULL;
     char            Opts[BUF_SIZE];
-    int            logopt = 0;
+    int             logopt = 0;
 
     /*
      * initialize session to default values 
@@ -299,13 +317,21 @@ snmp_parse_args(int argc, char **argv, netsnmp_session *session,
             break;
 
         case 'v':
+            session->version = -1;
+#ifndef DISABLE_SNMPV1
             if (!strcmp(optarg, "1")) {
                 session->version = SNMP_VERSION_1;
-            } else if (!strcasecmp(optarg, "2c")) {
+            }
+#endif
+#ifndef DISABLE_SNMPV2C
+            if (!strcasecmp(optarg, "2c")) {
                 session->version = SNMP_VERSION_2c;
-            } else if (!strcasecmp(optarg, "3")) {
+            }
+#endif
+            if (!strcasecmp(optarg, "3")) {
                 session->version = SNMP_VERSION_3;
-            } else {
+            }
+            if (session->version == -1) {
                 fprintf(stderr,
                         "Invalid version specified after -v flag: %s\n",
                         optarg);
@@ -454,14 +480,17 @@ snmp_parse_args(int argc, char **argv, netsnmp_session *session,
 
         case 'l':
             if (!strcasecmp(optarg, "noAuthNoPriv") || !strcmp(optarg, "1")
+                || !strcasecmp(optarg, "noauth")
                 || !strcasecmp(optarg, "nanp")) {
                 session->securityLevel = SNMP_SEC_LEVEL_NOAUTH;
             } else if (!strcasecmp(optarg, "authNoPriv")
                        || !strcmp(optarg, "2")
+                       || !strcasecmp(optarg, "auth")
                        || !strcasecmp(optarg, "anp")) {
                 session->securityLevel = SNMP_SEC_LEVEL_AUTHNOPRIV;
             } else if (!strcasecmp(optarg, "authPriv")
                        || !strcmp(optarg, "3")
+                       || !strcasecmp(optarg, "priv")
                        || !strcasecmp(optarg, "ap")) {
                 session->securityLevel = SNMP_SEC_LEVEL_AUTHPRIV;
             } else {
@@ -474,10 +503,13 @@ snmp_parse_args(int argc, char **argv, netsnmp_session *session,
             break;
 
         case 'a':
+#ifndef DISABLE_MD5
             if (!strcasecmp(optarg, "MD5")) {
                 session->securityAuthProto = usmHMACMD5AuthProtocol;
                 session->securityAuthProtoLen = USM_AUTH_PROTO_MD5_LEN;
-            } else if (!strcasecmp(optarg, "SHA")) {
+            } else
+#endif
+                if (!strcasecmp(optarg, "SHA")) {
                 session->securityAuthProto = usmHMACSHA1AuthProtocol;
                 session->securityAuthProtoLen = USM_AUTH_PROTO_SHA_LEN;
             } else {
@@ -489,21 +521,23 @@ snmp_parse_args(int argc, char **argv, netsnmp_session *session,
             break;
 
         case 'x':
+            testcase = 0;
+#ifndef DISABLE_DES
             if (!strcasecmp(optarg, "DES")) {
+                testcase = 1;
                 session->securityPrivProto = usmDESPrivProtocol;
                 session->securityPrivProtoLen = USM_PRIV_PROTO_DES_LEN;
-#ifdef HAVE_AES
-            } else if (!strcasecmp(optarg, "AES128")) {
-                session->securityPrivProto = usmAES128PrivProtocol;
-                session->securityPrivProtoLen = USM_PRIV_PROTO_AES128_LEN;
-            } else if (!strcasecmp(optarg, "AES192")) {
-                session->securityPrivProto = usmAES192PrivProtocol;
-                session->securityPrivProtoLen = USM_PRIV_PROTO_AES192_LEN;
-            } else if (!strcasecmp(optarg, "AES256")) {
-                session->securityPrivProto = usmAES256PrivProtocol;
-                session->securityPrivProtoLen = USM_PRIV_PROTO_AES256_LEN;
+            }
 #endif
-            } else {
+#ifdef HAVE_AES
+            if (!strcasecmp(optarg, "AES128") ||
+                !strcasecmp(optarg, "AES")) {
+                testcase = 1;
+                session->securityPrivProto = usmAESPrivProtocol;
+                session->securityPrivProtoLen = USM_PRIV_PROTO_AES_LEN;
+            }
+#endif
+            if (testcase == 0) {
                 fprintf(stderr,
                       "Invalid privacy protocol specified after -x flag: %s\n",
                         optarg);
@@ -552,7 +586,7 @@ snmp_parse_args(int argc, char **argv, netsnmp_session *session,
         }
     }
     DEBUGMSGTL(("snmp_parse_args", "finished: %d/%d\n", optind, argc));
-
+    
     if (!logopt)
 	snmp_enable_stderrlog();
 
@@ -576,21 +610,31 @@ snmp_parse_args(int argc, char **argv, netsnmp_session *session,
          */
         if (!session->version) {
             switch (DEFAULT_SNMP_VERSION) {
+#ifndef DISABLE_SNMPV1
             case 1:
                 session->version = SNMP_VERSION_1;
                 break;
+#endif
 
+#ifndef DISABLE_SNMPV2C
             case 2:
                 session->version = SNMP_VERSION_2c;
                 break;
+#endif
 
             case 3:
                 session->version = SNMP_VERSION_3;
                 break;
+
+            default:
+                snmp_log(LOG_ERR, "Can't determine a valid SNMP version for the session\n");
+                return(-2);
             }
         } else {
+#ifndef DISABLE_SNMPV1
             if (session->version == NETSNMP_DS_SNMP_VERSION_1)  /* bogus value.  version 1 actually = 0 */
                 session->version = SNMP_VERSION_1;
+#endif
         }
     }
 
@@ -609,13 +653,20 @@ snmp_parse_args(int argc, char **argv, netsnmp_session *session,
                 snmp_duplicate_objid(def, session->securityAuthProtoLen);
         }
         if (session->securityAuthProto == NULL) {
+#ifndef DISABLE_MD5
             /*
-             * assume MD5 
+             * assume MD5
              */
             session->securityAuthProto =
                 snmp_duplicate_objid(usmHMACMD5AuthProtocol,
                                      USM_AUTH_PROTO_MD5_LEN);
             session->securityAuthProtoLen = USM_AUTH_PROTO_MD5_LEN;
+#else
+            session->securityAuthProto =
+                snmp_duplicate_objid(usmHMACSHA1AuthProtocol,
+                                     USM_AUTH_PROTO_SHA_LEN);
+            session->securityAuthProtoLen = USM_AUTH_PROTO_SHA_LEN;
+#endif
         }
         if (generate_Ku(session->securityAuthProto,
                         session->securityAuthProtoLen,
@@ -643,10 +694,18 @@ snmp_parse_args(int argc, char **argv, netsnmp_session *session,
             /*
              * assume DES 
              */
+#ifndef DISABLE_DES
             session->securityPrivProto =
                 snmp_duplicate_objid(usmDESPrivProtocol,
                                      USM_PRIV_PROTO_DES_LEN);
             session->securityPrivProtoLen = USM_PRIV_PROTO_DES_LEN;
+#else
+            session->securityPrivProto =
+                snmp_duplicate_objid(usmAESPrivProtocol,
+                                     USM_PRIV_PROTO_AES_LEN);
+            session->securityPrivProtoLen = USM_PRIV_PROTO_AES_LEN;
+#endif
+
         }
         if (generate_Ku(session->securityAuthProto,
                         session->securityAuthProtoLen,
@@ -668,9 +727,11 @@ snmp_parse_args(int argc, char **argv, netsnmp_session *session,
     }
     session->peername = argv[optind++]; /* hostname */
 
+#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
     /*
      * If v1 or v2c, check community has been set, either by a -c option above,
      * or via a default token somewhere.  
+     * If neither, it will be taken from the incoming request PDU.
      */
 
     if (session->version == SNMP_VERSION_1 ||
@@ -679,13 +740,23 @@ snmp_parse_args(int argc, char **argv, netsnmp_session *session,
             Cpsz = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
 					 NETSNMP_DS_LIB_COMMUNITY);
 	    if (Cpsz == NULL) {
-		fprintf(stderr, "No community name specified.\n");
-		return (-1);
+                if (netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
+                                           NETSNMP_DS_LIB_IGNORE_NO_COMMUNITY)){
+                    DEBUGMSGTL(("snmp_parse_args",
+                                "ignoring that the community string is not present\n"));
+                    session->community = NULL;
+                    session->community_len = 0;
+                } else {
+                    fprintf(stderr, "No community name specified.\n");
+                    return (-1);
+                }
 	    }
-	}
-        session->community = (unsigned char *)Cpsz;
-        session->community_len = strlen(Cpsz);
+	} else {
+            session->community = (unsigned char *)Cpsz;
+            session->community_len = strlen(Cpsz);
+        }
     }
+#endif /* support for community based SNMP */
 
     return optind;
 }
