@@ -208,8 +208,6 @@ netsnmp_daemonize(int quit_immediately, int stderr_log)
         }
 #ifndef WIN32
         else {
-            int cnt;
-
             /* Child. */
             
             DEBUGMSGT(("daemonize","child continuing\n"));
@@ -234,12 +232,6 @@ netsnmp_daemonize(int quit_immediately, int stderr_log)
                 dup(0);
                 dup(0);
             }
-
-            /*
-             * close all non-standard open file descriptors
-             */
-            for (cnt = getdtablesize() - 1; cnt >= 2; --cnt)
-                (void) close(cnt);
         }
 #endif /* !WIN32 */
     }
@@ -510,8 +502,7 @@ get_uptime(void)
      * min requirement is one PERF_DATA_BLOCK plus one PERF_OBJECT_TYPE 
      */
     perfdata = (PPERF_DATA_BLOCK) malloc(buffersize);
-    if (!perfdata)
-        return 0;
+
 
     memset(perfdata, 0, buffersize);
 
