@@ -20,6 +20,7 @@
 #define LEAVE goto labelFIN
 #define FINALLY do { \
 labelFIN: \
+	; \
 } while(0); if(1)
 
 #else
@@ -707,12 +708,6 @@ ServiceMain (DWORD argc, LPTSTR argv[])
 	       * Allocate memory to hold strings 
 	       */
 	      ArgArray = (LPTSTR *) malloc (sizeof (LPTSTR) * ArgCount);
-	      if (ArgArray == 0)
-                {
-                    WriteToEventLog (EVENTLOG_ERROR_TYPE,
-		       _T ("Resource failure"));
-                    return;
-                }
 
 	      /*
 	       * Copy first argument 
