@@ -89,16 +89,13 @@ if test -x /bin/netstat ; then
     NETSTAT=/bin/netstat
 elif test -x /usr/bin/netstat ; then
     NETSTAT=/usr/bin/netstat
-elif test -x /usr/sbin/netstat ; then
-    # e.g. Tru64 Unix
-    NETSTAT=/usr/sbin/netstat
 else
     NETSTAT=""
 fi
 if [ "x$SNMP_SNMPD_PORT" = "x" ]; then
     SNMP_SNMPD_PORT="8765"
     MAX_RETRIES=3
-    if test -x "$NETSTAT" ; then
+    if test -x $NETSTAT ; then
         if test -z "$RANDOM"; then
             RANDOM=2
         fi
